/* Calculate the "Ion Accessible Volume" from a PQR file 
   and write it into a file. molname.iav.dat or 
   molname.iav.txt if -ascii is given on command line

   the Volume is based on the SolvAccVol with the essentially 
   identical analytic representation, except that the atom 
   radii are enlarged by the Stern Layer thickness (sterln) 
   The probe radius is also given by the sterln.

   Options: sterln, blab1, blab2, blab3 as other MEAD programs.
   Option ascii for output format in ASCII, binary else.

   This program is based on MEAD by Don Bashford and licensed under
   the GPL.

   Thomas Ullmann, 2007

   /usr/bin/g++-3.4 -c -I. -I../.. -I.  -DUSE_EXCEPTIONS   -O2 pqr2IonAccVol.cc
   /usr/bin/g++-3.4 -I. -I../.. -I.  -DUSE_EXCEPTIONS    -O2  -o pqr2IonAccVol pqr2IonAccVol.o -L../../libmead -lmead  -lm

*/



#include "MEAD/IonAccVol.h"
#include "MEAD/PhysCond.h"
#include "MEAD/AtomSet.h"
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <sstream>
using std::istringstream;
#include <fstream>
using std::ifstream;
#include <string>
using std::string;

int main(int argc, char* argv[])
{
  string molname; // name of the input and output file
  float sterln = PhysCond::get_sterln ();   // probe radius
  bool ascii = 0; // Use ASCII output? Else, binary
  
  // Parse the command line
  for (int iarg=1; iarg<argc; ++iarg) {
    if (iarg+1==argc) {  // These is the last one, must be molname
      molname = argv[iarg];
    }
    else if ((string) argv[iarg] == (string) "-sterln") {
      if (iarg+1>=argc) error ("main: ERROR, not enough args after -solrad");
      istringstream ist(argv[++iarg]);
      float f;
      ist >> f;
      if (!(ist.good() || ist.eof()))
        error ("main: FAILED trying to read value after -sterln option");
      PhysCond::set_sterln(f);
      sterln = f;
    }
    else if ((string) argv[iarg] == (string) "-ascii") {
      ascii = 1;
    }
    // Process -blab flags
    else if ((string) argv[iarg] == (string) "-blab1") {
      blab1pt = &cout;
    }
    else if ((string) argv[iarg] == (string) "-blab2") {
      blab1pt = &cout;
      blab2pt = &cout;
    }
    else if ((string) argv[iarg] == (string) "-blab3") {
      blab1pt = &cout;
      blab2pt = &cout;
      blab3pt = &cout;
    }
    else {
      cerr << "main: ERROR command option, " << argv[iarg]
        << ", not recognized" << endl;
      error();
    }
  }

  if (molname == (string) "")
   error("main: ERROR, molname not given on command line");

  AtomSet a(molname);
  a.read();
  IonAccVol s(a, sterln);
  s.anal_calc();
  if (ascii) {
    s.write_top_in_ascii(molname + ".iav.txt");
  }
  else {
    s.write_top_in_binary(molname +".iav.dat");
  }

  return 0;
}
