#!/usr/bin/perl -w
use strict;
use Cwd;
use POSIX qw(ceil floor);
use Sys::Hostname;

################################
my %global = (
              qsub => "/sw/sge/bin/glinux/qsub -cwd",
              qmpb_path => "/sw/sci/app/mead_64/my_mead_09",
              curvescript => "/home/ullmannt/bin/scripts/get-curves-gmct.pl",
              epath => ".",
              opath => ".",
              queues => "barney.q,bart.q,burns.q,carl.q,homer.q,krusty.q,lenny.q,lisa.q,maggie.q,marge.q,milhouse.q,moe.q,patty.q,quimby.q,selma.q,smithers.q,wiggum.q,node-01-01.q,node-01-02.q,node-01-03.q,node-01-04.q",
	      memory => "200",
#	      threads => 8,              
	      threads => 3,              
             );
################################
#queues => "barney.q,bart.q,burns.q,carl.q,homer.q,krusty.q,lenny.q,lisa.q,maggie.q,marge.q,milhouse.q,moe.q,patty.q,quimby.q,selma.q,smithers.q,wiggum.q",
#queues => "patty.q,selma.q,node-01-01.q,node-01-02.q,node-01-03.q,node-01-04.q",	
# takes job.sh from make-rotamers, run pqr2(Solv|Ion)AccVol, make submit script for each my_?diel_solver and submit

if ($global{threads} < 1){
 $global{threads} = `cat /proc/cpuinfo | grep -c processor`;
}

printf("will use %d threads\n", $global{threads});

my $wd = cwd();
my @directory = split("/", sprintf("%s", cwd()));
my $numlevel = 0;
foreach my $level (@directory){
 $numlevel ++;
}
my $thirdlevel = $numlevel - 3;

my $jobname = "QMPB.$directory[$thirdlevel]";

my $numberjobs = 0;
my $numberscripts = 0;
my $jobnum = 0;
my @joblines;
my $njobstotal=0;
my @lines;
my $hostname = hostname();

while (<>) {
 next if (/bin\/sh|^#|^\s*\n/);
 s/[\\]//g;
 if (/pqr2SolvAccVol/){
  print $_;
  system ("$_");
 } 
 elsif (/((\d+|N)diel|memb)_solver\s+.+[\s\^s]*\s+[>]{1}\s+([^\s]+)\n/){
  my $tmpfile = $3;
  my $redo = 0;
  if (-e "$tmpfile"){
   my $tmp = $_;
   my $ok = 0;
   open (INB, $tmpfile) || die "Input file ".$tmpfile."not found\n";
    while (<INB>){
     if (/^Total runtime/){$ok = 1;}
    }
   close (INB);
   if ($ok != 1) {
    $redo = 1;
   }
   $_ = $tmp;
  }
  if ((not (-e $tmpfile)) || $redo == 1){
   $njobstotal++;
   push (@lines, $_);
  }
 }
}

my $maxjobsinscript = ceil($njobstotal/$global{threads});
my $thisjob=0;
printf("Will redo %d jobs in %d threads with %d jobs per thread.\n", $njobstotal, $global{threads}, $maxjobsinscript);


for (my $tl=0; $tl<=$#lines; $tl++) {
 if ($lines[$tl]  =~ /((\d+|N)diel|memb)_solver\s+.+[\s\^s]*\s+[>]{1}\s+([^\s]+)\n/){
  my $redo = 0;
  if ($jobnum == $maxjobsinscript){
   close (FH);
   system ("/bin/tcsh submit.$hostname.$thisjob.csh 1>&2 /dev/null &");
   $jobnum = 0;
  }
  if ($jobnum == 0){
   $thisjob++;
   open (FH, ">submit.$hostname.$thisjob.csh") || die "Can't open submit.$hostname.$thisjob.csh for writing!\n";
   print FH "#!/bin/tcsh\n#\n#\$ -l mem=200\n#\$ -e /home/ullmannt/sungrid/error\n#\$ -o /home/ullmannt/sungrid/out\ncd $wd\n";
   $numberscripts++;
  }
  if ($jobnum < $maxjobsinscript){
   print $lines[$tl];
   print FH $lines[$tl]."\n";
   $jobnum++;
   $numberjobs++;
  } 
 }
}


if ($jobnum <= $maxjobsinscript && $jobnum != 0){
 close (FH);
 system ("/bin/tcsh submit.$hostname.$thisjob.csh 1>&2 /dev/null &");
}

print "\n\nExecuted $numberscripts scripts containing $numberjobs jobs.\n";  
