// This is -*- C++ -*-
/*
 TRU 09.2007
 ID of an atom consisting of residue name and atom name
*/
#ifndef _CharmmID_h
#define _CharmmID_h 1

#include <string>
using std::string;
#include <iostream>
#include "MEAD_e/Atom.h"

//!wrap!
class CharmmID {
public:
  CharmmID ();
  CharmmID ( const string& residue_name, const string& atom_name);
  CharmmID (const CharmmID&);
  CharmmID (const Atom&);
  ~CharmmID () {}
  //!nowrap!
  CharmmID& operator= (const CharmmID&);
  bool operator== (const CharmmID&) const;
  bool operator!= (const CharmmID& a) const;
  bool operator<(const CharmmID&) const;
  bool operator>(const CharmmID&) const;
  //!nowrap!+
  ostream& print(ostream&);
  string get_atname() const {return atname;}
  string get_resname() const {return resname;}
  //!nowrap!-
private:
  string resname;
  string atname;
};

#if SWIGPP_LITERAL_INCLUDE

%addmethods CharmmID {
// Wrap these private members as read-only attributes for Python, but
// this is a bit of a hack (see below).
// It'd probably be better to just wrap the get_member() functions,
// but for Python's use it seems more natural to treat these simple types
// as attributes.
  string resname;
  string atname;

// swigpp.el strips the default value of the string argument in the
// above constructor. So, wrap this constructor with the default
// argument removed and...
  CharmmID (int residue_name, const string& atom_name);
};

%wrapper %{
// ...provide the wrapped constructor with the default argument supplied
CharmmID * new_CharmmID(int residue_number, const string& atom_name){
  return new CharmmID(residue_number, atom_name, "");
}
// Provide the read-only get/set functions for the aboved wrapped attributes
void CharmmID_resname_set(CharmmID *self, int resnum){
   _SWIG_exception(SWIG_ValueError, "The resname attribute is read-only");
}
void CharmmID_atname_set(CharmmID *self, string *atname){
   _SWIG_exception(SWIG_ValueError, "The atname attribute is read-only");
}
// These strings are returned by reference, so create global placeholders
// to stash the results to be returned (read-only).
// Note that this works fine from Python since assignment of
// simple objects like numbers and strings effectively creates a copy.
// However, using these global references certainly is not thread safe!
string _CharmmID_atname_readonly;
string _CharmmID_resname_readonly;
string * CharmmID_atname_get(CharmmID *self){_CharmmID_atname_readonly = self->get_atname(); return &_CharmmID_atname_readonly;}
string * CharmmID_resname_get(CharmmID *self){_CharmmID_resname_readonly = self->get_resname(); return &_CharmmID_resname_readonly;}
%}

#endif // SWIGPP_LITERAL_INCLUDE

inline CharmmID::CharmmID (const CharmmID& a)
  : resname(a.resname), atname(a.atname) {}

inline CharmmID::CharmmID (const Atom& a)
  : resname(a.resname), atname(a.atname) {}
 
inline bool CharmmID::operator== (const CharmmID& a) const
{return resname == a.resname && atname == a.atname;}

inline bool CharmmID::operator!= (const CharmmID& a) const
{return !(*this == a);}

inline CharmmID& CharmmID::operator= (const CharmmID& a)
{resname = a.resname; atname = a.atname; return *this;}

inline bool CharmmID::operator<(const CharmmID& x) const
{
 if (resname == x.resname){
  return atname < x.atname;
 }
 else {
  return resname < x.resname;
 }
}

// Added by Bergsma 5/2/01
inline bool CharmmID::operator>(const CharmmID& x) const
{
 if (resname == x.resname){
  return atname > x.atname;
 }
 else {
  return resname > x.resname;
 }
}

inline ostream& operator<<(ostream& ost, CharmmID aid)
{return aid.print(ost);}

#endif

// CharmmID.h ends here
