/*  A set of CHARMM angle parameters -*- C++ -*-
    
    Copyright (C) 2007 Thomas Ullmann 
    thomas.ullmann@uni-bayreuth.de
    
    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#ifndef _CharmmParaAngle_h
#define _CharmmParaAngle_h 1

#include <string>
using std::string;
using std::ostream;
using std::istream;

//!wrap!
struct CharmmParaAngle {
  string attype1;
  string attype2;
  string attype3;
  float phi;
  float kphi;
  float rub;
  float kub;
  CharmmParaAngle ();
  CharmmParaAngle (const CharmmParaAngle &);
  ~CharmmParaAngle() {}
  //!nowrap!
  CharmmParaAngle& operator = (const CharmmParaAngle& a);
  bool operator== (const CharmmParaAngle& b) const;
  bool operator!= (const CharmmParaAngle& a) const;
  bool operator< (const CharmmParaAngle& b) const;
  bool operator> (const CharmmParaAngle& a) const;
  //!nowrap!+
  void print() const;
  void print(ostream& os) const;
  //!nowrap!-
};

inline bool CharmmParaAngle::operator== (const CharmmParaAngle& b) const
{
 if ((attype1 == b.attype1 && attype2 == b.attype2 && attype3 == b.attype3) || 
     (attype1 == b.attype3 && attype2 == b.attype2 && attype3 == b.attype1)){
  return phi == b.phi && kphi == b.kphi && rub == b.rub && kub == b.kub;
 } else {return false;}
}

inline bool CharmmParaAngle::operator!= (const CharmmParaAngle& a) const
{
  return !(*this == a);
}

ostream& operator <<(ostream& os, const CharmmParaAngle& a);

extern CharmmParaAngle nil_CharmmParaAngle;

inline bool CharmmParaAngle::operator<(const CharmmParaAngle& a) const
{
 if ((a.attype1 == attype1 && a.attype2 == attype2 && a.attype3 == attype3) || 
     (a.attype1 == attype3 && a.attype2 == attype2 && a.attype3 == attype1)){ 
  if (a.phi == phi) {
   if (a.kphi == kphi) {
    if (a.rub == rub) {
     return a.kub < kub;
    } else {return a.rub < rub;}
   } else {return a.kphi < kphi;}  
  } else {return a.phi < phi;}
 } 
 else if (a.attype1 == attype1 && a.attype2 == attype2){return a.attype3 < attype3;}
 else if (a.attype3 == attype3 && a.attype2 == attype2){return a.attype1 < attype1;}
 else if (a.attype3 == attype1 && a.attype2 == attype2){return a.attype1 < attype3;}
 else if (a.attype1 == attype3 && a.attype2 == attype2){return a.attype3 < attype1;}
 else if (a.attype2 == attype2){
  if      ((a.attype1 < attype1 && a.attype3 < attype3) || 
           (a.attype1 < attype3 && a.attype3 < attype1)){return false;}
  else if ((a.attype1 < attype1 && a.attype3 < attype3) || 
           (a.attype1 < attype3 && a.attype3 < attype1)){return true;} 
 }
 else if (a.attype2 < attype2){return true;}
 else {return false;}
}

inline bool CharmmParaAngle::operator>(const CharmmParaAngle& a) const
{
 if ((a.attype1 == attype1 && a.attype2 == attype2 && a.attype3 == attype3) || 
     (a.attype1 == attype3 && a.attype2 == attype2 && a.attype3 == attype1)){ 
  if (a.phi == phi) {
   if (a.kphi == kphi) {
    if (a.rub == rub) {
     return a.kub > kub;
    } else {return a.rub > rub;}
   } else {return a.kphi > kphi;}  
  } else {return a.phi > phi;}  
 } 
 else if (a.attype1 == attype1 && a.attype2 == attype2){return a.attype3 > attype3;}
 else if (a.attype3 == attype3 && a.attype2 == attype2){return a.attype1 > attype1;}
 else if (a.attype3 == attype1 && a.attype2 == attype2){return a.attype1 > attype3;}
 else if (a.attype1 == attype3 && a.attype2 == attype2){return a.attype3 > attype1;}
 else if (a.attype2 == attype2){
  if      ((a.attype1 > attype1 && a.attype3 > attype3) || 
           (a.attype1 > attype3 && a.attype3 > attype1)){return false;}
  else if ((a.attype1 > attype1 && a.attype3 > attype3) || 
           (a.attype1 > attype3 && a.attype3 > attype1)){return true;} 
 }
 else if (a.attype2 > attype2){return true;}
 else {return false;}
}      

#if SWIGPP_LITERAL_INCLUDE

%addmethods CharmmParaAngle {
   void write() { self->print(cout); }
   bool operator<(const CharmmParaAngle& a, const CharmmParaAngle& b);
   bool operator>(const CharmmParaAngle& a, const CharmmParaAngle& b);
};

#endif // SWIGPP_LITERAL_INCLUDE

#endif

// CharmmParaAngle.h ends here
