/*  A set of CHARMM nonbonded parameters -*- C++ -*-
    
    Copyright (C) 2007 Thomas Ullmann 
    thomas.ullmann@uni-bayreuth.de
    
    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#ifndef _CharmmParaNB_h
#define _CharmmParaNB_h 1

#include <string>
using std::string;
using std::ostream;
using std::istream;

//!wrap!
struct CharmmParaNB {
  string attype;
  float ljr;
  float lje;
  float ljr14;
  float lje14;
  float charge;
  CharmmParaNB ();
  CharmmParaNB (const CharmmParaNB &);
  ~CharmmParaNB() {}
  //!nowrap!
  CharmmParaNB& operator = (const CharmmParaNB& a);
  bool operator== (const CharmmParaNB&) const;
  bool operator!= (const CharmmParaNB&) const;
  bool operator< (const CharmmParaNB&) const;
  bool operator> (const CharmmParaNB&) const;
  //!nowrap!+
  void print() const;
  void print(ostream& os) const;
  //!nowrap!-
};

inline bool CharmmParaNB::operator== (const CharmmParaNB& b) const
{
  return attype == attype && lje == b.lje && ljr == b.ljr && 
         lje14 == b.lje14 && ljr14 == b.ljr14 && charge == b.charge ;
}

inline bool CharmmParaNB::operator!= (const CharmmParaNB& a) const
{
  return !(*this == a);
}

ostream& operator <<(ostream& os, const CharmmParaNB& a);

extern CharmmParaNB nil_CharmmParaNB;

inline bool CharmmParaNB::operator<(const CharmmParaNB& a) const
{
 if (a.attype == attype) {
  if (a.ljr == ljr) {
   if (a.lje == lje) {
    if (a.ljr14 == ljr14) {
     if (a.lje14 == lje14) {
      return a.charge < charge;
     } else {return a.lje14 < lje14;}
    } else {return a.ljr14 < ljr14;}
   } else {return a.lje < lje;}
  } else {return a.ljr < ljr;}
 } else {return a.attype < attype;}
}

inline bool CharmmParaNB::operator>(const CharmmParaNB& a) const
{
 if (a.attype == attype) {
  if (a.ljr == ljr) {
   if (a.lje == lje) {
    if (a.ljr14 == ljr14) {
     if (a.lje14 == lje14) {
      return a.charge > charge;
     } else {return a.lje14 > lje14;}
    } else {return a.ljr14 > ljr14;}
   } else {return a.lje > lje;}
  } else {return a.ljr > ljr;}
 } else {return a.attype > attype;}
}

#if SWIGPP_LITERAL_INCLUDE

%addmethods CharmmParaNB {
   void write() { self->print(cout); }
   bool operator<(const CharmmParaNB& a, const CharmmParaNB& b);
   bool operator>(const CharmmParaNB& a, const CharmmParaNB& b);
};

#endif // SWIGPP_LITERAL_INCLUDE

#endif

// CharmmParaNB.h ends here
