#include "MEAD/ElectrolyteMembAtoms.h"
#include "MEAD/AtomSet.h"
#include "MEAD/CubeLatSpec.h"
#include "MEAD/ElySlab.h"
#include "MEAD/IonAccVol.h"
#include "MEAD/PhysCond.h"
#include <stdio.h>
#include <iostream>


/*
   added, TRU 2007
   This is a 2-value electrolyte separated by a 3-slab membrane and
   1 atom set for the united QM, normal + cavity regions
   It is meant to complement the SixValueDielMembThreeAtoms
   A cylindrical hole through the membrane may be used to mimick a channel
*/

TwoElectrolyteMembAtoms::TwoElectrolyteMembAtoms(const AtomSet& a, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2, float stl)
 :ElectrolyteEnvironment_lett(), _ats(a), _ats_defined(true),
    ion_acc(a, stl), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2), sternl(stl)
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }

}

TwoElectrolyteMembAtoms::TwoElectrolyteMembAtoms (const AtomSet & a, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2)
 : ElectrolyteEnvironment_lett(), _ats(a), _ats_defined(true),
    ion_acc(a, PhysCond::get_sterln()), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}


TwoElectrolyteMembAtoms::TwoElectrolyteMembAtoms (const AtomSet & a, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr)
 : ElectrolyteEnvironment_lett(), _ats(a), _ats_defined(true),
    ion_acc(a, PhysCond::get_sterln()), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(PhysCond::get_ionicstr()), ionic_str2(PhysCond::get_ionicstr()),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembAtoms::TwoElectrolyteMembAtoms (const IonAccVol & a, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2, float stl)
 : ElectrolyteEnvironment_lett(), _ats(), _ats_defined(false),
    ion_acc(a), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2), sternl(stl)
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembAtoms::TwoElectrolyteMembAtoms (const IonAccVol & a, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr)
 : ElectrolyteEnvironment_lett(), _ats(), _ats_defined(false),
    ion_acc(a), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(PhysCond::get_ionicstr()), ionic_str2(PhysCond::get_ionicstr()),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembAtoms::TwoElectrolyteMembAtoms (const IonAccVol & a, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2)
 : ElectrolyteEnvironment_lett(), _ats(), _ats_defined(false),
    ion_acc(a), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

// For GB/DL's sake
const AtomSet& TwoElectrolyteMembAtoms::get_atomset() const
{
  if (!_ats_defined)
    ::error("TwoElectrolyteMembAtoms::get_atomset: atom set not defined");
  return _ats;
}

ElyCubeRep
TwoElectrolyteMembAtoms::get_cuberep (const CubeLatSpec& cls)
{
  blab3 << "TwoElectrolyteMembAtoms::get_cuberep entered"
    << endl;

  // we have to extend the solvent inaccessible volume by the Stern layer
  float spacing = cls.get_spacing();
  Coord grid_center_in_space = cls.get_center();
  int grid_dim = cls.get_grid_dim();
  int nsq = grid_dim*grid_dim;
  int ncube = grid_dim*grid_dim*grid_dim;
  float grlen = (float) (grid_dim - 1);
  float halfgrlen = grlen/2;
  Coord grid_center_in_grid (halfgrlen, halfgrlen, halfgrlen);
  float sternl_in_grid = sternl/spacing;
  float stlsq = sternl_in_grid * sternl_in_grid;
  int stli = (int) ceil(sternl_in_grid);
  int stlin = -stli;
  int grid_dim_plus_stli = grid_dim + stli;
  float halfgrlen_in_space = halfgrlen * spacing;
  Coord grid_diag_in_space;
  grid_diag_in_space.x = halfgrlen_in_space;
  grid_diag_in_space.y = halfgrlen_in_space;
  grid_diag_in_space.z = halfgrlen_in_space;
  Coord grid_origin_in_space = grid_center_in_space - grid_diag_in_space;
  float zlhbou = (zlowerhead - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zuhbou = (zupperhead - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zlcbou = (zlowercore - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zucbou = (zuppercore - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  Coord holecen_in_grid = (hole_center - grid_center_in_space)/spacing
    + grid_center_in_grid;
  float holerad_in_grid = holerad/spacing;
  float holesq = holerad_in_grid*holerad_in_grid;

  ElyCubeRep dcr(cls);
  AccTag* acc_array = new AccTag[ncube];
  AccTag* tmp_array = new AccTag[ncube];

  if (ion_acc.check_is_calculated()) {
    blab2 << "TwoElectrolyteMembAtoms::get_cuberep using existing "
      << "analytical ion accessible volume representation to make a cubic representation" << endl;
    ion_acc.calc_cuberep(cls, tmp_array);
  } else if (!_ats_defined){
    blab2 << "TwoElectrolyteMembAtoms::get_cuberep calculating "
      << "analytical ion accessible volume representation to make a cubic representation" << endl;
    // anal_calc is automatically called by calc_cuberep if flag is_calculated is false
    ion_acc.calc_cuberep(cls, tmp_array);
  } else {
    blab2 << "TwoElectrolyteMembAtoms::get_cuberep calculating "
      << " simple ion accessible volume representation to make a cubic representation" << endl;

    for (int h=0; h<ncube; ++h) {
      tmp_array[h] = exterior;
    }

    for(AtomSet::const_iterator ip = _ats.begin(); ip!=_ats.end(); ++ip ) {

      int rad = (int) ceil ((ip->second.rad + sternl)/spacing);
      int radstlsq = rad * rad;
      Coord atom_coord = ip->second.coord - grid_origin_in_space;
      int k = (int) round((atom_coord.z/spacing));
      int i = (int) round((atom_coord.x/spacing));
      int j = (int) round((atom_coord.y/spacing));
      int h = (i * nsq) + (j * grid_dim) + k;
      // skip atom if not in grid
      if (k < stlin || i < stlin || j < stlin ||
          k > grid_dim_plus_stli ||
          i > grid_dim_plus_stli ||
          j > grid_dim_plus_stli){continue;}
      int lower_l = k - rad;
      int lower_m = i - rad;
      int lower_n = j - rad;
      int upper_l = k + rad + 1;
      int upper_m = i + rad + 1;
      int upper_n = j + rad + 1;
      bool fastswitch = true;
      if (lower_l < 0) {lower_l = 0; fastswitch = false;}
      if (lower_m < 0) {lower_m = 0; fastswitch = false;}
      if (lower_n < 0) {lower_n = 0; fastswitch = false;}
      if (upper_l > grid_dim) {upper_l = grid_dim; fastswitch = false;}
      if (upper_m > grid_dim) {upper_m = grid_dim; fastswitch = false;}
      if (upper_n > grid_dim) {upper_n = grid_dim; fastswitch = false;}
      if (fastswitch) {
        // in the inner grid we have a full sphere around the point
        // and can save time by doing loops and distance
        // only in +,+,+ octand of sphere
        // tried to improve by computing dist only for 1/48th sphere
        // also permutating x,y,z and doing loops and distance only
        // in in x >= y >= z but result is slower
        int dl = -1;
        int lterm = -grid_dim;
        for (int l=k; l<upper_l; ++l) {
          dl++;
          int dlsq = dl * dl;
          lterm += grid_dim;
          int lsqterm = dl*nsq;
          int dm = -1;
          int mterm = -grid_dim;
          for (int m=i; m<upper_m; ++m) {
            dm++;
            int dmsq = dm * dm;
            int dmlsq = dmsq + dlsq;
            // out of sphere already in plane
            if (dmlsq > radstlsq){break;}
            mterm += grid_dim;
            int msqterm = dm * nsq;
            int dn = -1;
            int nterm = -grid_dim;
            for (int n=j; n<upper_n; ++n) {
              dn++;
              int dissq = (dn * dn) + dmlsq;
              if (dissq > radstlsq){break;}
              int nsqterm = dn*nsq;
              nterm += grid_dim;
              // 1: x, y, z
              // here we permutate +/- signs in the three components
              // + + +
              int hh = h + msqterm + nterm + dl;
              tmp_array[hh] = interior;
              // + + -
              hh = hh - dl  - dl;
              tmp_array[hh] = interior;
              // + - -
              hh = hh - nterm - nterm;
              tmp_array[hh] = interior;
              // + - +
              hh = hh + dl + dl;
              tmp_array[hh] = interior;
              // - - +
              hh = hh - msqterm - msqterm;
              tmp_array[hh] = interior;
              // - - -
              hh = hh - dl - dl;
              tmp_array[hh] = interior;
              // - + -
              hh = hh + nterm + nterm;
              tmp_array[hh] = interior;
              // - + +
              hh = hh + dl + dl;
              tmp_array[hh] = interior;
            }
          }
        }
      }
      else {
        // these are atoms at the grid border
        int dl = -1;
        for (int l=lower_l; l<upper_l; ++l) {
          dl++;
          int dlsq = dl * dl;
          int dm = -1;
          for (int m=lower_m; m<upper_m; ++m) {
            dm++;
            int dmsq = dm * dm;
            int dmlsq = dmsq + dlsq;
            if (dmlsq > radstlsq){break;}
            int msqterm = m*nsq;
            int dn = -1;
            for (int n=lower_n; n<upper_n; ++n) {
              dn++;
              int dissq = (dn * dn) + dmlsq;
              if (dissq <= radstlsq){
                int hh = msqterm + n*grid_dim + l;
                tmp_array[hh] = interior;
              }
            }
          }
        }
      }
    }


  }

  // initialize acc_array

  for (int h=0; h<ncube; ++h) {
    acc_array[h] = exterior;
  }

  for (int k=0; k<grid_dim; ++k) {
    float fk = (float) k;
    // check if we are inside the membrane core
    if (fk >= zlcbou && fk <= zucbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
	float xr = ((float) i) - holecen_in_grid.x;
	float xrsq = xr*xr;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          float yr = ((float) j) - holecen_in_grid.y;
	  float rsq = xrsq + (yr*yr);
          // within the membrane core ions are not allowed
          // if we are outside the hole or inside the
          // solvent inaccessible surface + sternlayer
          if (tmp_array[h] == interior) {
            acc_array[h] = interior;
          }
          else if (rsq >= holesq) {
            acc_array[h] = interior;
          }
        }
      }
    } // end inside membrane core
    else {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (tmp_array[h] == interior) {
            acc_array[h] = interior;
          }
        }
      }
    } // end outside membrane core
  }
  delete [] tmp_array;


  // now we are ready construct the ElyCubeRep,
  // acc_array holds the ion accessible volume

  float corewidth = zucbou - zlcbou;

  for (int k=0; k<grid_dim; ++k) {
    float fk = (float) k;
    // check if we are inside the membrane core
    if (fk >= zlcbou && fk <= zucbou) {
      float membrane_pos = (fk - zlcbou) / corewidth;
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            // linear gradient of ionic strength within membrane pore;
            dcr[h] = (ionic_str1 * membrane_pos) + (ionic_str2 * (1 - membrane_pos));
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
    // ~ above the membrane core
    else if (fk > zucbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i * nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            dcr[h] = ionic_str1;
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
    // ~ below the membrane core
    else if (fk < zlcbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i * nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            dcr[h] = ionic_str2;
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }

  }

  delete [] acc_array;
  dcr.declare_defined();
  blab3 << "exit from TwoElectrolyteMembAtoms::get_cuberep\n" << endl;
  return dcr;

}

TwoElectrolyteMembAtoms::~TwoElectrolyteMembAtoms()
{
}



/*
   added, TRU 2007
   This is a 2-value electrolyte separated by a 3-slab membrane and
   atom set 1 for the united QM, normal + cavity regions and
   atom set 2 for a ion accessible region within the membrane
   It is meant to complement the SixValueDielMembThreeAtoms
   A cylindrical hole through the membrane may be used to mimick a channel
*/

TwoElectrolyteMembTwoAtoms::TwoElectrolyteMembTwoAtoms(const AtomSet& a1, const AtomSet& a2, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2, float stl)
 :ElectrolyteEnvironment_lett(), _ats1(a1), _ats2(a2), _ats_defined(true),
    ion_acc1(a1, stl), ion_acc2(a2, stl),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2), sternl(stl)
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }

}

TwoElectrolyteMembTwoAtoms::TwoElectrolyteMembTwoAtoms (const AtomSet & a1, const AtomSet & a2, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2)
 : ElectrolyteEnvironment_lett(), _ats1(a1), _ats2(a2), _ats_defined(true),
    ion_acc1(a1, PhysCond::get_sterln()), ion_acc2(a2, PhysCond::get_sterln()),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}


TwoElectrolyteMembTwoAtoms::TwoElectrolyteMembTwoAtoms (const AtomSet & a1, const AtomSet & a2, float zlh,
 float zuh, float zlc, float zuc, Coord c, float hr)
 : ElectrolyteEnvironment_lett(), _ats1(a1), _ats2(a2), _ats_defined(true),
    ion_acc1(a1, PhysCond::get_sterln()), ion_acc2(a2, PhysCond::get_sterln()),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(PhysCond::get_ionicstr()), ionic_str2(PhysCond::get_ionicstr()),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembTwoAtoms::TwoElectrolyteMembTwoAtoms (const IonAccVol & a1, const IonAccVol & a2, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2, float stl)
 : ElectrolyteEnvironment_lett(), _ats1(), _ats2(), _ats_defined(false),
    ion_acc1(a1), ion_acc2(a2), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2), sternl(stl)
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembTwoAtoms::TwoElectrolyteMembTwoAtoms (const IonAccVol& a1, const IonAccVol& a2,
 float zlh, float zuh, float zlc, float zuc, Coord c, float hr)
 : ElectrolyteEnvironment_lett(), _ats1(), _ats2(), _ats_defined(false),
    ion_acc1(a1), ion_acc2(a2), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    hole_center(c), holerad(hr), ionic_str1(PhysCond::get_ionicstr()), ionic_str2(PhysCond::get_ionicstr()),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembTwoAtoms::TwoElectrolyteMembTwoAtoms (const IonAccVol & a1, const IonAccVol & a2, float zlh, float zuh, float zlc, float zuc,
 Coord c, float hr, float ionic1, float ionic2)
 : ElectrolyteEnvironment_lett(), _ats1(), _ats2(), _ats_defined(false),
    ion_acc1(a1), ion_acc2(a2), zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    holerad(hr), ionic_str1(ionic1), ionic_str2(ionic2),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (holerad < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembTwoAtoms ctor called with sternl < 0.0\n"
      << "(" << holerad << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

// For GB/DL's sake
const AtomSet& TwoElectrolyteMembTwoAtoms::get_atomset() const
{
  if (!_ats_defined)
    ::error("TwoElectrolyteMembTwoAtoms::get_atomset: atom set not defined");
  return _ats1;
}

ElyCubeRep
TwoElectrolyteMembTwoAtoms::get_cuberep (const CubeLatSpec& cls)
{
  blab3 << "TwoElectrolyteMembTwoAtoms::get_cuberep entered"
    << endl;

  // we have to extend the solvent inaccessible volume by the Stern layer
  float spacing = cls.get_spacing();
  Coord grid_center_in_space = cls.get_center();
  int grid_dim = cls.get_grid_dim();
  int nsq = grid_dim*grid_dim;
  int ncube = grid_dim*grid_dim*grid_dim;
  float grlen = (float) (grid_dim - 1);
  float halfgrlen = grlen/2;
  Coord grid_center_in_grid (halfgrlen, halfgrlen, halfgrlen);
  float sternl_in_grid = sternl/spacing;
  float stlsq = sternl_in_grid * sternl_in_grid;
  int stli = (int) ceil(sternl_in_grid);
  int stlin = -stli;
  int grid_dim_plus_stli = grid_dim + stli;
  float halfgrlen_in_space = halfgrlen * spacing;
  Coord grid_diag_in_space;
  grid_diag_in_space.x = halfgrlen_in_space;
  grid_diag_in_space.y = halfgrlen_in_space;
  grid_diag_in_space.z = halfgrlen_in_space;
  Coord grid_origin_in_space = grid_center_in_space - grid_diag_in_space;
  float zlhbou = (zlowerhead - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zuhbou = (zupperhead - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zlcbou = (zlowercore - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zucbou = (zuppercore - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  Coord holecen_in_grid = (hole_center - grid_center_in_space)/spacing
    + grid_center_in_grid;
  float holerad_in_grid = holerad/spacing;
  float holesq = holerad_in_grid*holerad_in_grid;

  ElyCubeRep dcr(cls);
  AccTag  *acc_array    = new AccTag[ncube];
  AccTag **tmp_array    = new AccTag*[2];
           tmp_array[0] = new AccTag[ncube];
           tmp_array[1] = new AccTag[ncube];

  if (ion_acc1.check_is_calculated() && ion_acc2.check_is_calculated()) {
    blab2 << "TwoElectrolyteMembTwoAtoms::get_cuberep using existing "
      << "analytical ion accessible volume representation to make a cubic representation" << endl;
    ion_acc1.calc_cuberep(cls, tmp_array[0]);
    ion_acc2.calc_cuberep(cls, tmp_array[1]);
  } else if (!_ats_defined){
    blab2 << "TwoElectrolyteMembTwoAtoms::get_cuberep calculating "
      << "analytical ion accessible volume representations to make a cubic representation" << endl;
    // anal_calc is automatically called by calc_cuberep if flag is_calculated is false
    ion_acc1.calc_cuberep(cls, tmp_array[0]);
    ion_acc2.calc_cuberep(cls, tmp_array[1]);
  } else {
    blab2 << "TwoElectrolyteMembTwoAtoms::get_cuberep calculating "
      << " simple ion accessible volume representations to make a cubic representation" << endl;

    // initialize tmp_array
    for (int g=0; g<2; ++g)
     for (int h=0; h<ncube; ++h)
      tmp_array[g][h] = exterior;

    AtomSet:: const_iterator startats;
    AtomSet:: const_iterator endats;

    for (int g=0; g<2; ++g){
      if        (g==0){
       startats = _ats1.begin(); endats = _ats1.end();
      } else if (g==1){
       startats = _ats2.begin(); endats = _ats2.end();
      }
      for(AtomSet::const_iterator ip = startats; ip!=endats; ++ip ) {
        int rad = (int) ceil ((ip->second.rad + sternl)/spacing);
        int radstlsq = rad * rad;
        Coord atom_coord = ip->second.coord - grid_origin_in_space;
        int k = (int) round((atom_coord.z/spacing));
        int i = (int) round((atom_coord.x/spacing));
        int j = (int) round((atom_coord.y/spacing));
        int h = (i * nsq) + (j * grid_dim) + k;
        // skip atom if not in grid
        if (k < stlin || i < stlin || j < stlin ||
            k > grid_dim_plus_stli ||
            i > grid_dim_plus_stli ||
            j > grid_dim_plus_stli){continue;}
        int lower_l = k - rad;
        int lower_m = i - rad;
        int lower_n = j - rad;
        int upper_l = k + rad + 1;
        int upper_m = i + rad + 1;
        int upper_n = j + rad + 1;
        bool fastswitch = true;
        if (lower_l < 0) {lower_l = 0; fastswitch = false;}
        if (lower_m < 0) {lower_m = 0; fastswitch = false;}
        if (lower_n < 0) {lower_n = 0; fastswitch = false;}
        if (upper_l > grid_dim) {upper_l = grid_dim; fastswitch = false;}
        if (upper_m > grid_dim) {upper_m = grid_dim; fastswitch = false;}
        if (upper_n > grid_dim) {upper_n = grid_dim; fastswitch = false;}
        if (fastswitch) {
          // in the inner grid we have a full sphere around the point
          // and can save time by doing loops and distance
          // only in +,+,+ octand of sphere
          // tried to improve by computing dist only for 1/48th sphere
          // also permutating x,y,z and doing loops and distance only
          // in in x >= y >= z but result is slower
          int dl = -1;
          int lterm = -grid_dim;
          for (int l=k; l<upper_l; ++l) {
            dl++;
            int dlsq = dl * dl;
            lterm += grid_dim;
            int lsqterm = dl*nsq;
            int dm = -1;
            int mterm = -grid_dim;
            for (int m=i; m<upper_m; ++m) {
              dm++;
              int dmsq = dm * dm;
              int dmlsq = dmsq + dlsq;
              // out of sphere already in plane
              if (dmlsq > radstlsq){break;}
              mterm += grid_dim;
              int msqterm = dm * nsq;
              int dn = -1;
              int nterm = -grid_dim;
              for (int n=j; n<upper_n; ++n) {
                dn++;
                int dissq = (dn * dn) + dmlsq;
                if (dissq > radstlsq){break;}
                int nsqterm = dn*nsq;
                nterm += grid_dim;
                // 1: x, y, z
                // here we permutate +/- signs in the three components
                // + + +
                int hh = h + msqterm + nterm + dl;
                tmp_array[g][hh] = interior;
                // + + -
                hh = hh - dl  - dl;
                tmp_array[g][hh] = interior;
                // + - -
                hh = hh - nterm - nterm;
                tmp_array[g][hh] = interior;
                // + - +
                hh = hh + dl + dl;
                tmp_array[g][hh] = interior;
                // - - +
                hh = hh - msqterm - msqterm;
                tmp_array[g][hh] = interior;
                // - - -
                hh = hh - dl - dl;
                tmp_array[g][hh] = interior;
                // - + -
                hh = hh + nterm + nterm;
                tmp_array[g][hh] = interior;
                // - + +
                hh = hh + dl + dl;
                tmp_array[g][hh] = interior;
              }
            }
          }
        }
        else {
          // these are atoms at the grid border
          int dl = -1;
          for (int l=lower_l; l<upper_l; ++l) {
            dl++;
            int dlsq = dl * dl;
            int dm = -1;
            for (int m=lower_m; m<upper_m; ++m) {
              dm++;
              int dmsq = dm * dm;
              int dmlsq = dmsq + dlsq;
              if (dmlsq > radstlsq){break;}
              int msqterm = m*nsq;
              int dn = -1;
              for (int n=lower_n; n<upper_n; ++n) {
                dn++;
                int dissq = (dn * dn) + dmlsq;
                if (dissq <= radstlsq){
                  int hh = msqterm + n*grid_dim + l;
                  tmp_array[g][hh] = interior;
                }
              }
            }
          }
        }
      }
    }

  }

  // initialize acc_array

  for (int h=0; h<ncube; ++h) {
    acc_array[h] = exterior;
  }

  for (int k=0; k<grid_dim; ++k) {
    float fk = (float) k;
    // check if we are inside the membrane core
    if (fk >= zlcbou && fk <= zucbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
	float xr = ((float) i) - holecen_in_grid.x;
	float xrsq = xr*xr;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          float yr = ((float) j) - holecen_in_grid.y;
	  float rsq = xrsq + (yr*yr);
          // within the membrane core ions are not allowed
          // if we are outside the hole or inside the
          // solvent inaccessible surface + sternlayer
          if (tmp_array[0][h] == interior) {
            acc_array[h] = interior;
          }
          else {
            // membrane core
            // case 1: ion accessible volume given by inside of
            //  dummy atom set elycavset
            //  use AccTag exterior
            // case 2: ion accessible volume given by cylindrical hole
            //  use AccTag undecided to distinguish channel volume
            //  from bulk electrolyte
            if (tmp_array[1][h] == interior) {
              // inside elycav
              acc_array[h] = exterior;
            }
            else if (rsq <= holesq) {
              // outside elycav and inside cylinder
              acc_array[h] = undecided;
            }
            else {
              // outside elycav and outside cylinder
              acc_array[h] = interior;
            }
          }
        }
      }
    } // end inside membrane core
    else {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (tmp_array[0][h] == interior) {
            acc_array[h] = interior;
          }
        }
      }
    } // end outside membrane core
  }
  delete [] tmp_array[0];
  delete [] tmp_array[1];
  delete [] tmp_array;

  // now we are ready construct the ElyCubeRep,
  // acc_array holds the ion accessible volume

  float corewidth = zucbou - zlcbou;

  for (int k=0; k<grid_dim; ++k) {
    float fk = (float) k;
    // check if we are inside the membrane core
    if (fk >= zlcbou && fk <= zucbou) {
      float membrane_pos = (fk - zlcbou) / corewidth;
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior){
            dcr[h] = 0.0;
          }
          else if (acc_array[h] == exterior){
            // ionic strength of closest region;
            // ambigous if a large ion accessible cavity
            // happens to lie in the center of the
            // membrane core - seems very improbable
            if (membrane_pos >= 0.5){
              dcr[h] = ionic_str1;
            }
            else{
              dcr[h] = ionic_str2;
            }
          }
          else if (acc_array[h] == undecided){
            // linear gradient of ionic strength within membrane pore;
            dcr[h] = (ionic_str1 * membrane_pos) + (ionic_str2 * (1- membrane_pos));
          }
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
    // ~ above the membrane core
    else if (fk > zucbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i * nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            dcr[h] = ionic_str1;
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
    // ~ below the membrane core
    else if (fk < zlcbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i * nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            dcr[h] = ionic_str2;
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }

  }

  delete [] acc_array;
  dcr.declare_defined();
  blab3 << "exit from TwoElectrolyteMembTwoAtoms::get_cuberep\n" << endl;
  return dcr;

}

TwoElectrolyteMembTwoAtoms::~TwoElectrolyteMembTwoAtoms()
{
}

/*
   added, TRU 2007
   This is a 2-value electrolyte separated by a 3-slab membrane and
   atom set 1 for the united QM, normal + cavity regions and
   atom set 2 for a ion accessible region within the membrane boundaries
   atom set 3 for a ion accessible region traversing the membrane
   It is meant to complement the SixValueDielMembThreeAtoms
*/


TwoElectrolyteMembThreeAtoms::TwoElectrolyteMembThreeAtoms(const AtomSet& a1, const AtomSet& a2,
 const AtomSet& a3, float zlh, float zuh, float zlc, float zuc,
 float ionic1, float ionic2, float stl)
 :ElectrolyteEnvironment_lett(), _ats1(a1), _ats2(a2), _ats3(a3), _ats_defined(true),
    ion_acc1(a1, stl), ion_acc2(a2, stl), ion_acc3(a3, stl),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    ionic_str1(ionic1), ionic_str2(ionic2), sternl(stl)
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembThreeAtoms::TwoElectrolyteMembThreeAtoms (const AtomSet & a1, const AtomSet & a2,
 const AtomSet & a3, float zlh, float zuh, float zlc, float zuc, float ionic1, float ionic2)
 : ElectrolyteEnvironment_lett(), _ats1(a1), _ats2(a2), _ats3(a3), _ats_defined(true),
    ion_acc1(a1, PhysCond::get_sterln()), ion_acc2(a2, PhysCond::get_sterln()),
    ion_acc3(a3, PhysCond::get_sterln()),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    ionic_str1(ionic1), ionic_str2(ionic2),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}


TwoElectrolyteMembThreeAtoms::TwoElectrolyteMembThreeAtoms (const AtomSet & a1, const AtomSet & a2,
 const AtomSet & a3, float zlh, float zuh, float zlc, float zuc)
 : ElectrolyteEnvironment_lett(), _ats1(a1), _ats2(a2), _ats3(a3), _ats_defined(true),
    ion_acc1(a1, PhysCond::get_sterln()), ion_acc2(a2, PhysCond::get_sterln()),
    ion_acc3(a3, PhysCond::get_sterln()),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    ionic_str1(PhysCond::get_ionicstr()), ionic_str2(PhysCond::get_ionicstr()),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembThreeAtoms::TwoElectrolyteMembThreeAtoms (const IonAccVol & a1, const IonAccVol & a2,
 const IonAccVol & a3, float zlh, float zuh, float zlc, float zuc,
 float ionic1, float ionic2, float stl)
 : ElectrolyteEnvironment_lett(), _ats1(), _ats2(), _ats3(), _ats_defined(false),
    ion_acc1(a1), ion_acc2(a2), ion_acc3(a3),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    ionic_str1(ionic1), ionic_str2(ionic2), sternl(stl)
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembThreeAtoms::TwoElectrolyteMembThreeAtoms (const IonAccVol& a1, const IonAccVol& a2,
 const IonAccVol& a3, float zlh, float zuh, float zlc, float zuc)
 : ElectrolyteEnvironment_lett(), _ats1(), _ats2(), _ats3(), _ats_defined(false),
    ion_acc1(a1), ion_acc2(a2), ion_acc3(a3),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    ionic_str1(PhysCond::get_ionicstr()), ionic_str2(PhysCond::get_ionicstr()),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

TwoElectrolyteMembThreeAtoms::TwoElectrolyteMembThreeAtoms (const IonAccVol & a1, const IonAccVol & a2,
 const IonAccVol & a3, float zlh, float zuh, float zlc, float zuc, float ionic1, float ionic2)
 : ElectrolyteEnvironment_lett(), _ats1(), _ats2(), _ats3(), _ats_defined(false),
    ion_acc1(a1), ion_acc2(a2), ion_acc3(a3),
    zlowerhead(zlh), zupperhead(zuh), zlowercore(zlc), zuppercore(zuc),
    ionic_str1(ionic1), ionic_str2(ionic2),
    sternl(PhysCond::get_sterln())
{
  // Sanity checks
  if (zlowercore >= zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowercore >= zuppercore\n"
      << "(" << zlowercore << " >= " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead >= zupperhead) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead >= zupperhead\n"
      << "(" << zlowerhead << " >= " << zupperhead << ")" << endl;
    ::error("ERROR in ");
  }
  if (zlowerhead > zlowercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zlowerhead > zlowercore\n"
      << "(" << zlowerhead << " > " << zlowercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (zupperhead < zuppercore) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with zupperhead < zuppercore\n"
      << "(" << zupperhead << " > " << zuppercore << ")" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str1 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str1 < 0.0\n"
      << "(" << ionic_str1 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (ionic_str2 < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with ionic_str2 < 0.0\n"
      << "(" << ionic_str2 << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
  if (sternl < 0.00) {
    cerr << "ERROR: TwoElectrolyteMembThreeAtoms ctor called with sternl < 0.0\n"
      << "(" << sternl << " < 0.00)" << endl;
    ::error("ERROR in ");
  }
}

// For GB/DL's sake
const AtomSet& TwoElectrolyteMembThreeAtoms::get_atomset() const
{
  if (!_ats_defined)
    ::error("TwoElectrolyteMembThreeAtoms::get_atomset: atom set not defined");
  return _ats1;
}

ElyCubeRep
TwoElectrolyteMembThreeAtoms::get_cuberep (const CubeLatSpec& cls)
{
  blab3 << "TwoElectrolyteMembThreeAtoms::get_cuberep entered"
    << endl;

  // we have to extend the solvent inaccessible volume by the Stern layer
  float spacing = cls.get_spacing();
  Coord grid_center_in_space = cls.get_center();
  int grid_dim = cls.get_grid_dim();
  int nsq = grid_dim*grid_dim;
  int ncube = grid_dim*grid_dim*grid_dim;
  float grlen = (float) (grid_dim - 1);
  float halfgrlen = grlen/2;
  Coord grid_center_in_grid (halfgrlen, halfgrlen, halfgrlen);
  float sternl_in_grid = sternl/spacing;
  float stlsq = sternl_in_grid * sternl_in_grid;
  int stli = (int) ceil(sternl_in_grid);
  int stlin = -stli;
  int grid_dim_plus_stli = grid_dim + stli;
  float halfgrlen_in_space = halfgrlen * spacing;
  Coord grid_diag_in_space;
  grid_diag_in_space.x = halfgrlen_in_space;
  grid_diag_in_space.y = halfgrlen_in_space;
  grid_diag_in_space.z = halfgrlen_in_space;
  Coord grid_origin_in_space = grid_center_in_space - grid_diag_in_space;
  float zlhbou = (zlowerhead - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zuhbou = (zupperhead - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zlcbou = (zlowercore - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;
  float zucbou = (zuppercore - grid_center_in_space.z)/spacing
    + grid_center_in_grid.z;

  ElyCubeRep dcr(cls);
  AccTag  *acc_array    = new AccTag[ncube];
  AccTag **tmp_array    = new AccTag*[3];
           tmp_array[0] = new AccTag[ncube];
           tmp_array[1] = new AccTag[ncube];
           tmp_array[2] = new AccTag[ncube];

  if (ion_acc1.check_is_calculated() && ion_acc2.check_is_calculated() && ion_acc3.check_is_calculated()) {
    blab2 << "TwoElectrolyteMembThreeAtoms::get_cuberep using existing "
      << "analytical ion accessible volume representation to make a cubic representation" << endl;
    ion_acc1.calc_cuberep(cls, tmp_array[0]);
    ion_acc2.calc_cuberep(cls, tmp_array[1]);
    ion_acc3.calc_cuberep(cls, tmp_array[2]);
  } else if (!_ats_defined){
    blab2 << "TwoElectrolyteMembThreeAtoms::get_cuberep calculating "
      << "analytical ion accessible volume representations to make a cubic representation" << endl;
    // anal_calc is automatically called by calc_cuberep if flag is_calculated is false
    ion_acc1.calc_cuberep(cls, tmp_array[0]);
    ion_acc2.calc_cuberep(cls, tmp_array[1]);
    ion_acc3.calc_cuberep(cls, tmp_array[2]);
  } else {
    blab2 << "TwoElectrolyteMembThreeAtoms::get_cuberep calculating "
      << " simple ion accessible volume representations to make a cubic representation" << endl;

    // initialize tmp_array
    for (int g=0; g<3; ++g)
     for (int h=0; h<ncube; ++h)
      tmp_array[g][h] = exterior;

    AtomSet:: const_iterator startats;
    AtomSet:: const_iterator endats;

    for (int g=0; g<3; ++g){
      if        (g==0){
       startats = _ats1.begin(); endats = _ats1.end();
      } else if (g==1){
       startats = _ats2.begin(); endats = _ats2.end();
      } else if (g==2){
       startats = _ats3.begin(); endats = _ats3.end();
      }

      for(AtomSet::const_iterator ip = startats; ip!=endats; ++ip ) {

        int rad = (int) ceil ((ip->second.rad + sternl)/spacing);
        int radstlsq = rad * rad;
        Coord atom_coord = ip->second.coord - grid_origin_in_space;
        int k = (int) round((atom_coord.z/spacing));
        int i = (int) round((atom_coord.x/spacing));
        int j = (int) round((atom_coord.y/spacing));
        int h = (i * nsq) + (j * grid_dim) + k;
        // skip atom if not in grid
        if (k < stlin || i < stlin || j < stlin ||
            k > grid_dim_plus_stli ||
            i > grid_dim_plus_stli ||
            j > grid_dim_plus_stli){continue;}
        int lower_l = k - rad;
        int lower_m = i - rad;
        int lower_n = j - rad;
        int upper_l = k + rad + 1;
        int upper_m = i + rad + 1;
        int upper_n = j + rad + 1;
        bool fastswitch = true;
        if (lower_l < 0) {lower_l = 0; fastswitch = false;}
        if (lower_m < 0) {lower_m = 0; fastswitch = false;}
        if (lower_n < 0) {lower_n = 0; fastswitch = false;}
        if (upper_l > grid_dim) {upper_l = grid_dim; fastswitch = false;}
        if (upper_m > grid_dim) {upper_m = grid_dim; fastswitch = false;}
        if (upper_n > grid_dim) {upper_n = grid_dim; fastswitch = false;}
        if (fastswitch) {
          // in the inner grid we have a full sphere around the point
          // and can save time by doing loops and distance
          // only in +,+,+ octand of sphere
          // tried to improve by computing dist only for 1/48th sphere
          // also permutating x,y,z and doing loops and distance only
          // in in x >= y >= z but result is slower
          int dl = -1;
          int lterm = -grid_dim;
          for (int l=k; l<upper_l; ++l) {
            dl++;
            int dlsq = dl * dl;
            lterm += grid_dim;
            int lsqterm = dl*nsq;
            int dm = -1;
            int mterm = -grid_dim;
            for (int m=i; m<upper_m; ++m) {
              dm++;
              int dmsq = dm * dm;
              int dmlsq = dmsq + dlsq;
              // out of sphere already in plane
              if (dmlsq > radstlsq){break;}
              mterm += grid_dim;
              int msqterm = dm * nsq;
              int dn = -1;
              int nterm = -grid_dim;
              for (int n=j; n<upper_n; ++n) {
                dn++;
                int dissq = (dn * dn) + dmlsq;
                if (dissq > radstlsq){break;}
                int nsqterm = dn*nsq;
                nterm += grid_dim;
                // 1: x, y, z
                // here we permutate +/- signs in the three components
                // + + +
                int hh = h + msqterm + nterm + dl;
                tmp_array[g][hh] = interior;
                // + + -
                hh = hh - dl  - dl;
                tmp_array[g][hh] = interior;
                // + - -
                hh = hh - nterm - nterm;
                tmp_array[g][hh] = interior;
                // + - +
                hh = hh + dl + dl;
                tmp_array[g][hh] = interior;
                // - - +
                hh = hh - msqterm - msqterm;
                tmp_array[g][hh] = interior;
                // - - -
                hh = hh - dl - dl;
                tmp_array[g][hh] = interior;
                // - + -
                hh = hh + nterm + nterm;
                tmp_array[g][hh] = interior;
                // - + +
                hh = hh + dl + dl;
                tmp_array[g][hh] = interior;
              }
            }
          }
        }
        else {
          // these are atoms at the grid border
          int dl = -1;
          for (int l=lower_l; l<upper_l; ++l) {
            dl++;
            int dlsq = dl * dl;
            int dm = -1;
            for (int m=lower_m; m<upper_m; ++m) {
              dm++;
              int dmsq = dm * dm;
              int dmlsq = dmsq + dlsq;
              if (dmlsq > radstlsq){break;}
              int msqterm = m*nsq;
              int dn = -1;
              for (int n=lower_n; n<upper_n; ++n) {
                dn++;
                int dissq = (dn * dn) + dmlsq;
                if (dissq <= radstlsq){
                  int hh = msqterm + n*grid_dim + l;
                  tmp_array[g][hh] = interior;
                }
              }
            }
          }
        }
      }
    }

  }

  // initialize acc_array
  for (int h=0; h<ncube; ++h) {
    acc_array[h] = exterior;
  }

  for (int k=0; k<grid_dim; ++k) {
    float fk = (float) k;
    // check if we are inside the membrane core
    if (fk >= zlcbou && fk <= zucbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          // within the membrane core ions are not allowed
          // if we are outside IAV 2 and outside IAV3
          // or inside the IAV 1 + sternlayer
          // within IAV 3 the "undecided" AccTag is used
          // for assignment of an electrolyte gradient
          if (tmp_array[0][h] == interior) {
            acc_array[h] = interior;
          }
          // points not already set to interior for IAV 1
          else {
            // membrane core
            // ion accessible volume given by IAV 3 overrides IAV 2
            // use AccTag undecided to distinguish channel volume
            // from bulk electrolyte
            if (tmp_array[2][h] == interior) {
              // inside channelset
              acc_array[h] = undecided;
            }
            else if (tmp_array[1][h] == interior) {
              // inside elycavset and outside channelset
              acc_array[h] = exterior;
            }
            else {
              // outside elycavset and outside channelset
              acc_array[h] = interior;
            }
          }
        }
      }
    } // end inside membrane core
    else {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (tmp_array[0][h] == interior) {
            acc_array[h] = interior;
          }
        }
      }
    } // end outside membrane core
  }
  delete [] tmp_array[0];
  delete [] tmp_array[1];
  delete [] tmp_array[2];
  delete [] tmp_array;

  // now we are ready construct the ElyCubeRep,
  // acc_array holds the ion accessible volume

  float corewidth = zucbou - zlcbou;

  for (int k=0; k<grid_dim; ++k) {
    float fk = (float) k;
    // check if we are inside the membrane core
    if (fk >= zlcbou && fk <= zucbou) {
      float membrane_pos = (fk - zlcbou) / corewidth;
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i*nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior){
            dcr[h] = 0.0;
          }
          else if (acc_array[h] == exterior){
            // ionic strength of closest region;
            // ambigous if a large ion accessible cavity
            // happens to lie in the center of the
            // membrane core - seems very improbable
            if (membrane_pos >= 0.5){
              dcr[h] = ionic_str1;
            }
            else{
              dcr[h] = ionic_str2;
            }
          }
          else if (acc_array[h] == undecided){
            // linear gradient of ionic strength within membrane pore;
            dcr[h] = (ionic_str1 * membrane_pos) + (ionic_str2 * (1- membrane_pos));
          }
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
    // ~ above the membrane core
    else if (fk > zucbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i * nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            dcr[h] = ionic_str1;
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
    // ~ below the membrane core
    else if (fk < zlcbou) {
      for (int i = 0; i<grid_dim; ++i) {
        int iterm = i * nsq;
        for (int j=0; j<grid_dim; ++j) {
          int h = iterm + j*grid_dim + k;
          if (acc_array[h] == interior)
            dcr[h] = 0.0;
          else if (acc_array[h] == exterior)
            dcr[h] = ionic_str2;
          else {
            cerr << "acc_array[" << h << "] has unexpected value, "
            << acc_array[h] << endl;
            ::error("ERROR in ");
          }
        }
      }
    }
  }


  delete [] acc_array;
  dcr.declare_defined();
  blab3 << "exit from TwoElectrolyteMembThreeAtoms::get_cuberep\n" << endl;
  return dcr;

}

TwoElectrolyteMembThreeAtoms::~TwoElectrolyteMembThreeAtoms()
{
}


// DielByAtoms.cc ends here
