// -*- C++ -*-
#ifndef ElectrolyteMembAtoms_h
#define ElectrolyteMembAtoms_h 1

#include "MEAD/ElectrolyteEnvironment.h"
#include "MEAD/AtomSet.h"
#include "MEAD/IonAccVol.h"
#include "MEAD/PhysCond.h"



/* added, TRU 2007
   This is a 2-value electrolyte separated by a 3-slab membrane and
   the atom set for the united QM, normal + cavity regions
   It is meant to complement the SixValueDielMembThreeAtoms
   A cylindrical hole through the membrane filled with a
   linear gradient between the electrolyte 1 and 2 in space unoccupied by
   atomset + membrane + sternlayer. The hole may be used to mimick a channel
   analogously to the one in SixValueDielMembThreeAtoms
*/

//!wrap!
class TwoElectrolyteMembAtoms : public ElectrolyteEnvironment_lett {
// LANGUAGE BUG!!  The following friend declaration is OK with gcc
// but not cfront.  According to the ARM Sec 11.4 p. 251, cfront is right!
//    friend ElstatPot::ElstatPot (DielectricEnvironment, ChargeDist,
//                                 ElectrolyteEnvironment);
// So must get "overly friendly"...
  //!nowrap!+
  friend class ElstatPot;
  //!nowrap!-
public:
  TwoElectrolyteMembAtoms (const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2, float stl);
  TwoElectrolyteMembAtoms (const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2);
  TwoElectrolyteMembAtoms (const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr);
  TwoElectrolyteMembAtoms (const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2, float stl);
  TwoElectrolyteMembAtoms (const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2);
  TwoElectrolyteMembAtoms (const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr);
  virtual ~TwoElectrolyteMembAtoms();
  float ionic_strength1() const {return ionic_str1;}
  float ionic_strength2() const {return ionic_str2;}
  float ionic_strength() const {return ((ionic_str1 + ionic_str2) * 0.5);}
  //!nowrap!+
  virtual ElyCubeRep get_cuberep(const CubeLatSpec& clsp);
  const AtomSet& get_atomset() const; // For GB/DL's sake
  //!nowrap!-
private:
  IonAccVol ion_acc;
  AtomSet _ats;   // For GB/DL's sake
  bool _ats_defined;
  Coord hole_center;
  float zlowerhead, zupperhead, zlowercore, zuppercore, holerad,
        ionic_str1, ionic_str2, sternl;
};

/*
   added, TRU 2007
   This is a 2-value electrolyte separated by a 3-slab membrane and
   atom set 1 for the united QM, normal + cavity regions and
   atom set 2 for a solvent region within the membrane boundaries
   (e.g., a funnel, a pore or large cavity which shall contain ions)
   It is meant to complement the SixValueDielMembThreeAtoms
   Volume occupied by atom set 2 and within the cylindrical hole is considered
   to be a channel through the membrane. It is filled by a linear gradient
   between the electrolyte 1 and 2 in space unoccupied by atomset 1 + sternlayer
   and occupied by atom set 2. The hole may be used to mimick a channel
   analogously to the one in SixValueDielMembThreeAtoms
*/

//!wrap!
class TwoElectrolyteMembTwoAtoms : public ElectrolyteEnvironment_lett {
// LANGUAGE BUG!!  The following friend declaration is OK with gcc
// but not cfront.  According to the ARM Sec 11.4 p. 251, cfront is right!
//    friend ElstatPot::ElstatPot (DielectricEnvironment, ChargeDist,
//                                 ElectrolyteEnvironment);
// So must get "overly friendly"...
  //!nowrap!+
  friend class ElstatPot;
  //!nowrap!-
public:
  TwoElectrolyteMembTwoAtoms (const AtomSet&, const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2, float stl);
  TwoElectrolyteMembTwoAtoms (const AtomSet&, const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2);
  TwoElectrolyteMembTwoAtoms (const AtomSet&, const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr);
  TwoElectrolyteMembTwoAtoms (const IonAccVol&, const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2, float stl);
  TwoElectrolyteMembTwoAtoms (const IonAccVol&, const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr, float ionic1, float ionic2);
  TwoElectrolyteMembTwoAtoms (const IonAccVol&, const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                           Coord c, float hr);
  virtual ~TwoElectrolyteMembTwoAtoms();
  float ionic_strength1() const {return ionic_str1;}
  float ionic_strength2() const {return ionic_str2;}
  float ionic_strength() const {return ((ionic_str1 + ionic_str2) * 0.5);}
  //!nowrap!+
  virtual ElyCubeRep get_cuberep(const CubeLatSpec& clsp);
  const AtomSet& get_atomset() const; // For GB/DL's sake
  //!nowrap!-
private:
  IonAccVol ion_acc1, ion_acc2;
  AtomSet _ats1, _ats2; // For GB/DL's sake
  bool _ats_defined;
  Coord hole_center;
  float zlowerhead, zupperhead, zlowercore, zuppercore, holerad,
        ionic_str1, ionic_str2, sternl;
};

/*
   added, TRU 2007
   This is a 2-value electrolyte separated by a 3-slab membrane and
   atom set 1 for the united QM, normal + cavity regions and
   atom set 2 for a ion accessible region within the membrane boundaries
   (e.g., a funnel, a pore or large cavity which shall contain ions)
   atom set 3 for a ion accessible region traversing the membrane
   It is meant to complement the SixValueDielMembThreeAtoms
   Volume occupied by atom set 2 but  unoccupied by
   atomset 1 + sternlayer is considered to be ion accessible
   and filled with the electrolyte of the closest membrane side.
   This Electrolyte Environment does not consider a cylindrical hole,
   instead volume occupied by atom set 3 but  unoccupied by
   atomset 1 + sternlayer is considered to be an ion accessible channel
   through the membrane. It is filled by a linear gradient between
   electrolyte 1 and 2.
*/

//!wrap!
class TwoElectrolyteMembThreeAtoms : public ElectrolyteEnvironment_lett {
// LANGUAGE BUG!!  The following friend declaration is OK with gcc
// but not cfront.  According to the ARM Sec 11.4 p. 251, cfront is right!
//    friend ElstatPot::ElstatPot (DielectricEnvironment, ChargeDist,
//                                 ElectrolyteEnvironment);
// So must get "overly friendly"...
  //!nowrap!+
  friend class ElstatPot;
  //!nowrap!-
public:
  TwoElectrolyteMembThreeAtoms (const AtomSet&, const AtomSet&, const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                                float ionic1, float ionic2, float stl);
  TwoElectrolyteMembThreeAtoms (const AtomSet&, const AtomSet&, const AtomSet&, float zlh, float zuh, float zlc, float zuc,
                                float ionic1, float ionic2);
  TwoElectrolyteMembThreeAtoms (const AtomSet&, const AtomSet&, const AtomSet&, float zlh, float zuh, float zlc, float zuc);
  TwoElectrolyteMembThreeAtoms (const IonAccVol&, const IonAccVol&, const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                                float ionic1, float ionic2, float stl);
  TwoElectrolyteMembThreeAtoms (const IonAccVol&, const IonAccVol&, const IonAccVol&, float zlh, float zuh, float zlc, float zuc,
                                float ionic1, float ionic2);
  TwoElectrolyteMembThreeAtoms (const IonAccVol&, const IonAccVol&, const IonAccVol&, float zlh, float zuh, float zlc, float zuc);
  virtual ~TwoElectrolyteMembThreeAtoms();
  float ionic_strength1() const {return ionic_str1;}
  float ionic_strength2() const {return ionic_str2;}
  float ionic_strength() const {return ((ionic_str1 + ionic_str2) * 0.5);}
  //!nowrap!+
  virtual ElyCubeRep get_cuberep(const CubeLatSpec& clsp);
  const AtomSet& get_atomset() const; // For GB/DL's sake
  //!nowrap!-
private:
  IonAccVol ion_acc1, ion_acc2, ion_acc3;
  AtomSet _ats1, _ats2, _ats3; // For GB/DL's sake
  bool _ats_defined;
  float zlowerhead, zupperhead, zlowercore, zuppercore,
        ionic_str1, ionic_str2, sternl;
};

#endif

// ElectrolyteMembAtoms.h ends here
