/* The value of the ionic strength on a cubic lattice. -*- C++ -*-

    Copyright (c) 1993--1995 by Donald Bashford

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

$Id: ElyCubeRep.h,v 2.4 2004/12/06 17:58:11 bashford Exp $
*/
#ifndef _ElyCubeRep_h
#define _ElyCubeRep_h 1

#include "MEAD/CubeLatSpec.h"
#include "MEAD/globals.h"

class ElyCubeRep {
public:
  ElyCubeRep(void){}
  ElyCubeRep(const CubeLatSpec&);
  ElyCubeRep(const ElyCubeRep&);
  ElyCubeRep& operator= (const ElyCubeRep&);
  ~ElyCubeRep();
  int istr_array_defined() const {return istr_def;}
  void declare_defined();
  void declare_undefined();
  const float& istr(int i, int j, int k) const;
  float operator[](int h) const;
  float& operator[](int h);
  float istr_neg_z(int i, int j, int k);
  float istr_pos_z(int i, int j, int k);
  float istr_neg_y(int i, int j, int k);
  float istr_pos_y(int i, int j, int k);
  float istr_neg_x(int i, int j, int k);
  float istr_pos_x(int i, int j, int k);
private:
  int grid_dim;
  int nsq;
  int ncube;
  int istr_def;
  struct ElyCubeRepRep {
    unsigned int referenceCount;
    float *istr_array;
  };
  ElyCubeRepRep *rep;
};

inline float ElyCubeRep::operator[] (int h) const
{
  return rep->istr_array[h];
}

inline float& ElyCubeRep::operator[] (int h)
{
  if (istr_def) {
    ::error("ERROR: illegal application of operator[] to a ElyCubeRep",
	    "with istr_array defined");
  }
  return rep->istr_array[h];
}

inline const float& ElyCubeRep::istr(int i, int j, int k) const
{
  return rep->istr_array[i*nsq + j*grid_dim + k];
}

inline float ElyCubeRep::istr_neg_z(int i, int j, int k)
{
  return 4.0 / (1.0/istr(i,j,k) + 1.0/istr(i,j+1,k)
		+ 1.0/istr(i+1,j,k) + 1.0/istr(i+1,j+1,k));
}

inline float ElyCubeRep::istr_pos_z(int i, int j, int k)
{
  return 4.0 / (1.0/istr(i,j,k+1) + 1.0/istr(i,j+1,k+1)
		+ 1.0/istr(i+1,j,k+1) + 1.0/istr(i+1,j+1,k+1));
}

inline float ElyCubeRep::istr_neg_y(int i, int j, int k)
{
  return 4.0 / (1.0/istr(i,j,k) + 1.0/istr(i,j,k+1)
		+ 1.0/istr(i+1,j,k) + 1.0/istr(i+1,j,k+1));
}

inline float ElyCubeRep::istr_pos_y(int i, int j, int k)
{
  return 4.0 / (1.0/istr(i,j+1,k) + 1.0/istr(i,j+1,k+1)
		+ 1.0/istr(i+1,j+1,k) + 1.0/istr(i+1,j+1,k+1));
}

inline float ElyCubeRep::istr_neg_x(int i, int j, int k)
{
  return 4.0 / (1.0/istr(i,j,k) + 1.0/istr(i,j,k+1)
		+ 1.0/istr(i,j+1,k) + 1.0/istr(i,j+1,k+1));
}

inline float ElyCubeRep::istr_pos_x(int i, int j, int k)
{
  return 4.0 / (1.0/istr(i+1,j,k) + 1.0/istr(i+1,j,k+1)
		+ 1.0/istr(i+1,j+1,k) + 1.0/istr(i+1,j+1,k+1));
}


#endif

// ElyCubeRep.h ends here
