/*
    Read dielectric regions from file
    filename is given with constructor
    file with name filename.diel is read by function read

    file format:
       prefix epsin solvent_probe_radius
          "       "            "

    For each solvent accessible volume it is tried to read a precalculated
    analytical representation from file.
    
    Binary file is read from prefix.dat if the file exists
    otherwise, ASCII file is read from prefix.txt if the file exists
    otherwise, the analytical SAV is recalculated from prefix.pqr
    
    Priority increases from the beginning to the end of the list.
    Volumes overide previous volumes on the list.
    
    e.g.
    mm_region_plus_qm_region     4 1.4
    surface_atoms_of_mm_region  20 1.4
    qm_region                    1 0.5

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.
*/

#ifndef NDielByFile_h
#define NDielByFile_h 1

#include "MEAD/DielectricEnvironment.h"
#include "MEAD/SolvAccVol.h"
#include "MEAD/AtomSet.h"
#include "MEAD/PhysCond.h"

//!wrap!
class NDielByFile : public DielectricEnvironment_lett {
 public:
  //!nowrap!
  NDielByFile(const NDielByFile& a);
  NDielByFile(const string& nm);
  ~NDielByFile(){}
  void read();
  void read(const string& fn);
  float epsext_value() const {return epsext;}
  //!nowrap!
  virtual DielCubeRep get_cuberep(const CubeLatSpec& cls);
  string get_name(){return name;}
  void set_name(string a){name = a;}
 private:
  string name;
  vector<float> eps;
  vector<SolvAccVol> solv_acc_vol;
  float epsext;
};

#endif

// NDielByFile.h ends here
