/*  A QMPB Site 

    label, forms
    
    I/O functions for .est files

    Copyright (C) 2007 Thomas Ullmann 
    thomas.ullmann@uni-bayreuth.de
    
    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.
*/
#include "MEAD_e/QmpbSite.h"
#include "MEAD_e/globals.h"
#include <stdio.h>
#include <ostream>
#include <iostream>

using std::istringstream;
using std::string;

using namespace std;

QmpbSite::QmpbSite(const QmpbSite& a)
: name(a.name), sid(a.sid), instances(a.instances)
{
}

QmpbSite::QmpbSite()
: name("")
{
}

QmpbSite::QmpbSite (const SiteID& id)
: name(id.get_sitename()), sid(id)
{
}

void QmpbSite::print() const
{
 cout << "///////////////////////////////////" << endl;
 cout << sid << " " << name << ":" << endl; 
 print_instances();
}

void QmpbSite::print_instances() const {
  for (list<QmpbInstance>::const_iterator i = instances.begin(); i != instances.end(); ++i){
    (*i).print();
  } 
}

void QmpbSite::renumber_instances() {
  instances.sort();
  int c = 0;
  for (list<QmpbInstance>::iterator i = instances.begin(); i != instances.end(); ++i){
    c++;
    InstanceID oid((*i).get_instid());
    InstanceID nid(oid.get_sitename(),oid.get_instname(),oid.get_conformer(),oid.get_chain(),oid.get_site(),c);
    (*i).set_instid(nid);
  } 
}

// QmpbSite.cc ends here
