/*  container for sidechain rotamer 
    torsion angles and occupation probability 
    
    Copyright (C) 2007 Thomas Ullmann 
    thomas.ullmann@uni-bayreuth.de
    
    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/

#include "MEAD_e/RotData.h"
#include <iostream>

RotData nil_RotData;

RotData::RotData ()
: p(),
  x1(),
  x2(),
  x3(),
  x4()
{}

RotData::RotData (const RotData & a)
: p(a.p), x1(a.x1), x2(a.x2), x3(a.x3), x4(a.x4)
{
}

RotData::RotData (const float tp, const float tx1, const float tx2, const float tx3, const float tx4)
: p(tp), x1(tx1), x2(tx2), x3(tx3), x4(tx4)
{
}

RotData& RotData::operator = (const RotData& a)
{
  p  = a.p;
  x1 = a.x1;
  x2 = a.x2;
  x3 = a.x3;
  x3 = a.x4;
  return *this;
}

void RotData::print() const
{
  std::cout << "chi 1 = " << x1 << " chi 2 = " << x2 << 
  " chi 3 = " << x3 << " chi 4 = " << x4 <<
  " occupation probability = " << p << std::endl;
}

void RotData::print(ostream& os) const
{
  os << "chi 1 = " << x1 << " chi 2 = " << x2 <<
  " chi 3 = " << x3 << " chi 4 = " << x4 <<
  " occupation probability = " << p << "\n";
}

ostream& operator <<(ostream& os, const RotData& a)
{
  a.print(os);
  return os;
}

// RotData.cc ends here
