/* Electrostatic potential for a dielectric slab -*- C++ -*-

    Copyright (c) 1993--1995 by Donald Bashford

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

$Id: AnalyMemb2.h,v 2.7 2004/12/06 17:57:51 bashford Exp $
*/
#ifndef _AnalyMemb2_h
#define _AnalyMemb2_h 1

#include "MEAD_e/ElstatPot.h"

#define AnalyMemb2_MAXTERM 20

class DielectricSlab;

//!wrap!
class AnalyMemb2 : public AnalyticEP {
public:
  //!nowrap!
//  AnalyMemb2 (DielectricSlab*, ChargeDist_lett*, ElectrolyteEnvironment_lett*,
//	int maxterm=AnalyMemb2_MAXTERM);  // No. of terms in image expression.
  AnalyMemb2 (DielectricSlab*, ChargeDist_lett*, ElectrolyteEnvironment_lett*);
  ~AnalyMemb2() {}
  virtual void solve();
  virtual float value(Coord) const ;
  virtual Coord field(Coord x) const ;
  virtual Coord displacement(Coord x) const ;
  float get_zlower() const;
  float get_zupper() const;
  int get_maxterm() const;
private:
  ChargeDist_lett* rho;
  DielectricSlab* eps;
  ElectrolyteEnvironment_lett* electrolyte;
  float epsslab, epsext;
  float zupper, zlower;

  float zmiddle, e_kappam_l, lhalf;
  float membl, istr1, istr2, istr, em_by_es, kappam, kappas;
  float kappam_by_kappas, em_by_es_times_kappam_by_kappas;
  float fac1, membpothalf;

  int maxterm;

  int solved;
};

inline float AnalyMemb2::get_zlower() const {return zlower;}
inline float AnalyMemb2::get_zupper() const {return zupper;}
inline int AnalyMemb2::get_maxterm() const {return maxterm;}

#if SWIGPP_LITERAL_INCLUDE

%addmethods AnalyMemb2 {
//
// swigpp.el strips the default value of the maxterm argument in the
// above constructor. So, wrap this constructor with the default
// argument both present and removed.
  AnalyMemb2 (DielectricSlab*, ChargeDist_lett*, ElectrolyteEnvironment_lett*, int);
  AnalyMemb2 (DielectricSlab*, ChargeDist_lett*, ElectrolyteEnvironment_lett*);
//
// Multiply with a ChargeDist_lett derived class.
// (The __rmul__ version is defined in the ChargeDist_lett derived classes
//
  float operator* (const AnalyMemb2& asl, const ChargeDist_lett& cdl);
//
// Scale operations
//
  ElstatPotCombination operator* (AnalyMemb2& asl, float scale);
  ElstatPotCombination operator* (float scale, AnalyMemb2& asl);
  ElstatPotCombination operator/ (AnalyMemb2& asl, float scale);
};

%wrapper %{

BEGIN_CPLUSPLUS_SECTION

// Provide the wrapped constructors.
AnalyMemb2 * new_AnalyMemb2 (DielectricSlab* ds, ChargeDist_lett* cd, ElectrolyteEnvironment_lett* ee, int maxterm){
  return new AnalyMemb2(ds, cd, ee, maxterm);
}
AnalyMemb2 * new_AnalyMemb2 (DielectricSlab* ds, ChargeDist_lett* cd, ElectrolyteEnvironment_lett* ee){
  return new AnalyMemb2(ds, cd, ee, AnalyMemb2_MAXTERM);
}

END_CPLUSPLUS_SECTION

%}

#endif // SWIGPP_LITERAL_INCLUDE

#endif

// AnalyMemb2.h ends here
