// This is -*- C++ -*-
/*
$Id: AtomID.h,v 2.11 2004/12/06 17:57:57 bashford Exp $
*/
#ifndef _AtomID_h
#define _AtomID_h 1

#include <stdio.h>
#include <string>
#include <ios>
#include <sstream>
#include "MEAD_e/Atom.h"
#include <boost/archive/basic_archive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/serialization/string.hpp>

using std::string;
using std::ios;
using std::iostream;
using std::istringstream;
using std::cout;
using std::cerr;
using std::endl;

//!wrap!
class AtomID {
public:
  AtomID ();
  AtomID (const int residue_number, const string& atom_name, const int confomer = 0, const int chain = 0, const int site = 0, const int instance = 0);
  //const string& chainid = "");
  AtomID (const AtomID&);
  AtomID (const Atom&);
  ~AtomID () {}
  //!nowrap!
  AtomID& operator= (const AtomID&);
  bool operator== (const AtomID&) const;
  bool operator!= (const AtomID& a) const;
  bool operator<(const AtomID&) const;
  bool operator>(const AtomID&) const;
  //!nowrap!+
  ostream& print(ostream&) const;
  istream& read(istream& ist);
  friend istream& operator >>(istream &is,AtomID &obj);
  friend ostream& operator <<(ostream &os,const AtomID &obj);
  int get_resnum() const {return resnum;}
  bool is_equal_up_to_inst(const AtomID &a) const;
  string get_atname() const {return atname;}
//  string get_chainid() const {return chainid;}
  //!nowrap!-
private:
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version)
  {
    ar & resnum;
    ar & conformer;
    ar & chain;
    ar & site;
    ar & instance;
    ar & atname;
  }
  int resnum;
  string atname;
//  string chainid;
  int conformer;
  int chain;
  int site;
  int instance;
};

#if SWIGPP_LITERAL_INCLUDE

%addmethods AtomID {
// Wrap these private members as read-only attributes for Python, but
// this is a bit of a hack (see below).
// It'd probably be better to just wrap the get_member() functions,
// but for Python's use it seems more natural to treat these simple types
// as attributes.
  int resnum;
  string atname;
//  string chainid;
  int conformer;
  int chain;
  int site;
  int instance

// swigpp.el strips the default value of the string argument in the
// above constructor. So, wrap this constructor with the default
// argument removed and...
  AtomID (int residue_number, const string& atom_name);
};

%wrapper %{
// ...provide the wrapped constructor with the default argument supplied
AtomID * new_AtomID(int residue_number, const string& atom_name){
  return new AtomID(residue_number, atom_name, "");
}
// Provide the read-only get/set functions for the aboved wrapped attributes
void AtomID_resnum_set(AtomID *self, int resnum){
   _SWIG_exception(SWIG_ValueError, "The resnum attribute is read-only");
}
void AtomID_atname_set(AtomID *self, string *atname){
   _SWIG_exception(SWIG_ValueError, "The atname attribute is read-only");
}
/*void AtomID_chainid_set(AtomID *self, string *chainid){
   _SWIG_exception(SWIG_ValueError, "The chainid attribute is read-only");
}*/
int AtomID_resnum_get(AtomID *self){return self->get_resnum();}
// These strings are returned by reference, so create global placeholders
// to stash the results to be returned (read-only).
// Note that this works fine from Python since assignment of
// simple objects like numbers and strings effectively creates a copy.
// However, using these global references certainly is not thread safe!
string _AtomID_atname_readonly;
//string _AtomID_chainid_readonly;
string * AtomID_atname_get(AtomID *self){_AtomID_atname_readonly = self->get_atname(); return &_AtomID_atname_readonly;}
//string * AtomID_chainid_get(AtomID *self){_AtomID_chainid_readonly = self->get_chainid(); return &_AtomID_chainid_readonly;}
%}

#endif // SWIGPP_LITERAL_INCLUDE

// compare without instance
inline bool AtomID::is_equal_up_to_inst(const AtomID &a) const
{return resnum == a.resnum && atname == a.atname && conformer == a.conformer && chain == a.chain && site == a.site;}

inline bool AtomID::operator== (const AtomID& a) const
//{return resnum == a.resnum && atname == a.atname && chainid == a.chainid;}
{return resnum == a.resnum && atname == a.atname && conformer == a.conformer && chain == a.chain && site == a.site && instance == a.instance;}


inline bool AtomID::operator!= (const AtomID& a) const
{return !(*this == a);}

inline AtomID& AtomID::operator= (const AtomID& a)
{resnum = a.resnum; atname = a.atname;  //chainid = a.chainid;
  conformer = a.conformer; chain = a.chain; site = a.site; instance = a.instance;
 return *this;}

inline bool AtomID::operator<(const AtomID& x) const
{
/*  if (chainid == x.chainid) {
    if (resnum == x.resnum)
      return atname < x.atname;
    else
      return resnum < x.resnum;
    }
  else
    return chainid < x.chainid;*/
    if (conformer == x.conformer) {
    if (chain == x.chain) {
      if (site == x.site) {
	if (instance == x.instance) {
	  if (resnum == x.resnum)
	    return atname < x.atname;
	  else
	    return resnum<x.resnum;
	} else
	  return instance<x.instance;
      } else
	return site<x.site;
    } else
      return chain<x.chain;
  } else
    return conformer<x.conformer;
}

// Added by Bergsma 5/2/01
inline bool AtomID::operator>(const AtomID& x) const
{
/*  if (chainid == x.chainid) {
    if (resnum == x.resnum)
      return atname > x.atname;
    else
      return resnum > x.resnum;
    }
  else
    return chainid > x.chainid;*/
    if (conformer == x.conformer) {
    if (chain == x.chain) {
      if (site == x.site) {
	if (instance == x.instance) {
	  if (resnum == x.resnum)
	    return atname > x.atname;
	  else
	    return resnum>x.resnum;
	} else
	  return instance>x.instance;
      } else
	return site>x.site;
    } else
      return chain>x.chain;
  } else
    return conformer>x.conformer;
}

inline istream& AtomID::read(istream& ist)
{
  // Read input of form, "string-int-int-int" corresponding to "sitename-conformer-chain-site"
  // hold values here till read is known OK.
  int rconformer = 0, rchain = 0, rsite = 0, rinstance = 0, rresnum = 0;
  string ratname, tmpstr;
  const char separator = '-', space = ' ';

  // extract the whole ID as one string from the stream
  ist >> tmpstr;
  
  // replace all occurrences of separator with space
  for (string::iterator i=tmpstr.begin(); i!=tmpstr.end(); i++){
   if (*i == separator){
    *i = space;
   }
  }

  // create new stream to extract the separate items
  istringstream ss1(tmpstr);
  ss1 >> rconformer >> rchain >> rsite >> rinstance >> rresnum >> ratname;

  if (!ss1.bad() && !ss1.fail()){
   conformer = rconformer;
   chain     = rchain;
   site      = rsite;   
   instance  = rinstance;   
   resnum    = rresnum;
   atname    = ratname;
  } else {
   if (ss1.bad()) ist.setstate(ios::badbit);
   if (ss1.fail()) ist.setstate(ios::failbit);
  }

  return ist;
}

inline ostream& operator<<(ostream& ost, const AtomID &aid)
{return aid.print(ost);}

inline istream& operator>>(istream& s, AtomID& aid)
{return aid.read(s);}

#endif

// AtomID.h ends here
