/*  container for CHARMM angle parameters

    Copyright (C) 2007 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/

#include "MEAD_e/CharmmParaAngle.h"
#include <iostream>

CharmmParaAngle nil_CharmmParaAngle;

CharmmParaAngle::CharmmParaAngle ()
: attype1(),
  attype2(),
  attype3(),
  phi(0),
  kphi(0),
  rub(0),
  kub(0)
{}

CharmmParaAngle::CharmmParaAngle (const CharmmParaAngle & a)
: attype1(a.attype1), attype2(a.attype2), attype3(a.attype3),
  phi(a.phi), kphi(a.kphi), rub(a.rub), kub(a.kub)
{
}

CharmmParaAngle& CharmmParaAngle::operator = (const CharmmParaAngle& a)
{
  attype1 = a.attype1;
  attype2 = a.attype2;
  attype3 = a.attype3;
  phi = a.phi;
  kphi = a.kphi;
  rub = a.rub;
  kub = a.kub;
  return *this;
}

void CharmmParaAngle::print() const
{
  std::cout << "Atom type 1 = " << attype1 << " Atom type 2 = " << attype2 <<
  " Atom type 3 = " << attype3 <<
  " Theta0 = " << phi << " rad " << " Ktheta  = " << kphi << " kcal/mole/rad**2 " <<
  " S0 = " << rub << " A " << " Kub  = " << kub << " kcal/mole/A**2" << std::endl;
}

void CharmmParaAngle::print(ostream& os) const
{
  os << "Atom type 1 = " << attype1 << " Atom type 2 = " << attype2 <<
  " Atom type 3 = " << attype3 <<
  " Theta0 = " << phi << " rad " << " Ktheta  = " << kphi << " kcal/mole/rad**2 " <<
  " S0 = " << rub << " A " << " Kub  = " << kub << " kcal/mole/A**2" << "\n";
}

ostream& operator <<(ostream& os, const CharmmParaAngle& a)
{
  a.print(os);
  return os;
}

// CharmmParaAngle.cc ends here
