/*  A set of CHARMM bond parameters -*- C++ -*-

    Copyright (C) 2007 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#ifndef _CharmmParaBond_h
#define _CharmmParaBond_h 1

#include <string>
#include <boost/archive/basic_archive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/serialization/string.hpp>

using std::string;
using std::ostream;
using std::istream;

//!wrap!
class CharmmParaBond {
public:
  string attype1;
  string attype2;
  float r;
  float k;
  CharmmParaBond ();
  CharmmParaBond (const CharmmParaBond &);
  ~CharmmParaBond() {}
  //!nowrap!
  CharmmParaBond& operator = (const CharmmParaBond& a);
  bool operator== (const CharmmParaBond&) const;
  bool operator!= (const CharmmParaBond&) const;
  bool operator> (const CharmmParaBond&) const;
  bool operator< (const CharmmParaBond&) const;
  //!nowrap!+
  void print() const;
  void print(ostream& os) const;
  //!nowrap!-
private:
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version)
  {
    ar & attype1;
    ar & attype2;
    ar & r;
    ar & k;
  }
};

inline bool CharmmParaBond::operator== (const CharmmParaBond& b) const
{
 if ((attype1 == b.attype1 && attype2 == b.attype2) ||
     (attype1 == b.attype2 && attype2 == b.attype1)){
  return r == b.r && k == b.k;
 } else {return false;}
}

inline bool CharmmParaBond::operator!= (const CharmmParaBond& a) const
{
  return !(*this == a);
}

ostream& operator <<(ostream& os, const CharmmParaBond& a);

extern CharmmParaBond nil_CharmmParaBond;

inline bool CharmmParaBond::operator<(const CharmmParaBond& a) const
{
 if ((a.attype1 == attype1 && a.attype2 == attype2) ||
     (a.attype1 == attype2 && a.attype2 == attype1)){
  if (a.r == r) {
    return a.k < k;
  } else {return a.r < r;}
 }
 else if (a.attype1 == attype1){return a.attype2 < attype2;}
 else if (a.attype1 == attype2){return a.attype2 < attype1;}
 else if (a.attype2 == attype1){return a.attype1 < attype2;}
 else if (a.attype2 == attype2){return a.attype1 < attype1;}
 else if ((a.attype1 > attype1 && a.attype2 > attype2) ||
          (a.attype1 > attype2 && a.attype2 > attype1)){return false;}
 else if ((a.attype1 < attype1 && a.attype2 < attype2) ||
          (a.attype1 < attype2 && a.attype2 < attype1)){return true;}
 else if (a.attype1 < attype1){return a.attype2 < attype2;}
 else if (a.attype1 < attype2){return a.attype2 < attype1;}
 else if (a.attype2 < attype1){return a.attype1 < attype2;}
 else if (a.attype2 < attype2){return a.attype1 < attype1;}
 return false;
}

inline bool CharmmParaBond::operator>(const CharmmParaBond& a) const
{
 if ((a.attype1 == attype1 && a.attype2 == attype2) ||
     (a.attype1 == attype2 && a.attype2 == attype1)){
  if (a.r == r) {
    return a.k > k;
  } else {return a.r > r;}
 }
 else if (a.attype1 == attype1){return a.attype2 > attype2;}
 else if (a.attype1 == attype2){return a.attype2 > attype1;}
 else if (a.attype2 == attype1){return a.attype1 > attype2;}
 else if (a.attype2 == attype2){return a.attype1 > attype1;}
 else if ((a.attype1 > attype1 && a.attype2 > attype2) ||
          (a.attype1 > attype2 && a.attype2 > attype1)){return false;}
 else if ((a.attype1 > attype1 && a.attype2 > attype2) ||
          (a.attype1 > attype2 && a.attype2 > attype1)){return true;}
 else if (a.attype1 > attype1){return a.attype2 > attype2;}
 else if (a.attype1 > attype2){return a.attype2 > attype1;}
 else if (a.attype2 > attype1){return a.attype1 > attype2;}
 else if (a.attype2 > attype2){return a.attype1 > attype1;}
 return false;
}

#if SWIGPP_LITERAL_INCLUDE

%addmethods CharmmParaBond {
   void write() { self->print(cout); }
   bool operator<(const CharmmParaBond& a, const CharmmParaBond& b);
   bool operator>(const CharmmParaBond& a, const CharmmParaBond& b);
};

#endif // SWIGPP_LITERAL_INCLUDE

#endif

// CharmmParaBond.h ends here
