/*  A set of CHARMM improper parameters -*- C++ -*-

    Copyright (C) 2007 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#ifndef _CharmmParaImproper_h
#define _CharmmParaImproper_h 1

#include <string>
#include <boost/archive/basic_archive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/serialization/string.hpp>

using std::string;
using std::ostream;
using std::istream;

//!wrap!
class CharmmParaImproper {
public:
  string attype1;
  string attype2;
  string attype3;
  string attype4;
  float psi;
  float kpsi;
  CharmmParaImproper ();
  CharmmParaImproper (const CharmmParaImproper &);
  ~CharmmParaImproper() {}
  //!nowrap!
  CharmmParaImproper& operator = (const CharmmParaImproper& a);
  bool operator== (const CharmmParaImproper&) const;
  bool operator!= (const CharmmParaImproper&) const;
  bool operator> (const CharmmParaImproper&) const;
  bool operator< (const CharmmParaImproper&) const;
  //!nowrap!+
  void print() const;
  void print(ostream& os) const;
  //!nowrap!-
private:
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version)
  {
    ar & attype1;
    ar & attype2;
    ar & attype3;
    ar & attype4;
    ar & psi;
    ar & kpsi;
  }
};

inline bool CharmmParaImproper::operator== (const CharmmParaImproper& b) const
{
 if        (attype1 == b.attype1 && attype2 == b.attype2 && attype3 == b.attype3 && attype4 == b.attype4){
  return psi == b.psi && kpsi == b.kpsi;
 } else if (attype4 == b.attype1 && attype2 == b.attype2 && attype3 == b.attype3 && attype1 == b.attype4){
  return psi == b.psi && kpsi == b.kpsi;
 } else {return false;}
}

inline bool CharmmParaImproper::operator!= (const CharmmParaImproper& a) const
{
  return !(*this == a);
}

ostream& operator <<(ostream& os, const CharmmParaImproper& a);

extern CharmmParaImproper nil_CharmmParaImproper;

inline bool CharmmParaImproper::operator<(const CharmmParaImproper& a) const
{
 if (a.attype1 == attype1){
  if (a.attype2 == attype2){
   if (a.attype3 == attype3){
    if (a.attype4 == attype4){
     if (a.psi < psi){return a.kpsi < kpsi;
     } else {return a.psi < psi;}
    } else {return a.attype4 < attype4;}
   } else {return a.attype3 < attype3;}
  } else {return a.attype2 < attype2;}
 } else {return a.attype1 < attype1;}
}

inline bool CharmmParaImproper::operator>(const CharmmParaImproper& a) const
{
 if (a.attype1 == attype1){
  if (a.attype2 == attype2){
   if (a.attype3 == attype3){
    if (a.attype4 == attype4){
     if (a.psi > psi){return a.kpsi > kpsi;
     } else {return a.psi > psi;}
    } else {return a.attype4 > attype4;}
   } else {return a.attype3 > attype3;}
  } else {return a.attype2 > attype2;}
 } else {return a.attype1 > attype1;}
}

#if SWIGPP_LITERAL_INCLUDE

%addmethods CharmmParaImproper {
   void write() { self->print(cout); }
   bool operator<(const CharmmParaImproper& a, const CharmmParaImproper& b);
   bool operator>(const CharmmParaImproper& a, const CharmmParaImproper& b);
};

#endif // SWIGPP_LITERAL_INCLUDE

#endif

// CharmmParaImproper.h ends here
