// -*- C++ -*-
#ifndef DielByAtoms_h
#define DielByAtoms_h 1

#include "MEAD_e/DielectricEnvironment.h"
#include "MEAD_e/SolvAccVol.h"
#include "MEAD_e/AtomSet.h"


//!wrap!
class TwoValueDielectricByAtoms : public DielectricEnvironment_lett {
public:
//  TwoValueDielectricByAtoms (const AtomSet&);
  TwoValueDielectricByAtoms (const AtomSet&, float ein);
  TwoValueDielectricByAtoms (const AtomSet&, float ein, float eext, float rad);
  TwoValueDielectricByAtoms (const SolvAccVol&, float ein);
  TwoValueDielectricByAtoms (const SolvAccVol&, float ein, float eext);
  virtual ~TwoValueDielectricByAtoms();
  float epsext_value() const {return epsext;}
  float epsin_value() const {return epsin;}
  //!nowrap!+
  virtual DielCubeRep get_cuberep(const CubeLatSpec& clsp);
  const AtomSet& get_atomset() const; // For GB/DL's sake
  //!nowrap!-
private:
  float epsin, epsext;
  SolvAccVol solv_acc;
  AtomSet _ats;   // For GB/DL's sake
  bool _ats_defined;
};

//!wrap!
class ThreeValueDielectricByAtoms : public DielectricEnvironment_lett {
public:
  ThreeValueDielectricByAtoms(const AtomSet& a1, float ein1, const AtomSet& a2, float ein2);
  ThreeValueDielectricByAtoms(const AtomSet& a1, float ein1, float solrad1, const AtomSet& a2, float ein2, float solrad2, float epsext);
  ThreeValueDielectricByAtoms(const SolvAccVol& a1, float ein1, const SolvAccVol& a2, float ein2);
  ThreeValueDielectricByAtoms(const SolvAccVol& a1, float ein1, const SolvAccVol& a2, float ein2, float epsext);
  virtual ~ThreeValueDielectricByAtoms();
  float epsext_value() const {return epsext;}
  //!nowrap!
  virtual DielCubeRep get_cuberep(const CubeLatSpec& clsp);
private:
  float epsin1, epsin2, epsext;
  SolvAccVol solv_acc1, solv_acc2;
};

/*
  TRU 2007,
  same as ThreeValueDielectricByAtoms but
  AtomSet 1 should contain union of qm and mm AtomSet,
  AtomSet 2 should contain mm AtomSet
  to avoid epsext at interface of QM/MM regions and
  to ensure that the QM region encloses the interface region
   (nothing polarizable found there)
*/

//!wrap!
class MyThreeValueDielectricByAtoms : public DielectricEnvironment_lett {
public:
  MyThreeValueDielectricByAtoms(const AtomSet& a1, float ein1, const AtomSet& a2, float ein2);
  MyThreeValueDielectricByAtoms(const AtomSet& a1, float ein1, float solrad1, const AtomSet& a2, float ein2, float solrad2, float epsext);
  MyThreeValueDielectricByAtoms(const SolvAccVol& a1, float ein1, const SolvAccVol& a2, float ein2);
  MyThreeValueDielectricByAtoms(const SolvAccVol& a1, float ein1, const SolvAccVol& a2, float ein2, float epsext);
  virtual ~MyThreeValueDielectricByAtoms();
  float epsext_value() const {return epsext;}
  //!nowrap!
  virtual DielCubeRep get_cuberep(const CubeLatSpec& clsp);
private:
  float epsin1, epsin2, epsext;
  SolvAccVol solv_acc1, solv_acc2;
};

#endif

// DielByAtoms.h ends here
