// -*- C++ -*-
#ifndef ElectrolyteByAtoms_h
#define ElectrolyteByAtoms_h 1

#include "MEAD_e/ElectrolyteEnvironment.h"
#include "MEAD_e/AtomSet.h"
#include "MEAD_e/SolvAccVol.h"
#include "MEAD_e/IonAccVol.h"
#include "MEAD_e/PhysCond.h"

//class ElectrolyteEnvironment_lett;
//class ElstatPot;


/* added, TRU 2007
   This is a simple 1-value electrolyte environment corresponding to
   the ElectrolyteByAtoms formerly used in MEAD and called by all
   original MEAD apps. It is built much faster than my new one and
   also faster than the original implementation.

   If called with an AtomSet the ion accessible surface is defined by
   atomset molecular surface (interlocking spheres) + sternlayer.
   The ion accessible surface defined in this way is less smooth than
   the new one and I think also incorrect since Ions are permitted
   to access voids at the interface of extended interlocking spheres.

   If called with an SolvAccVol the SolvAccVol will be used as is
   (no Stern Layer).
*/


//!wrap!
class SimpleElectrolyteByAtoms : public ElectrolyteEnvironment_lett {
// LANGUAGE BUG!!  The following friend declaration is OK with gcc
// but not cfront.  According to the ARM Sec 11.4 p. 251, cfront is right!
//    friend ElstatPot::ElstatPot (DielectricEnvironment, ChargeDist,
//                                 ElectrolyteEnvironment);
// So must get "overly friendly"...
  //!nowrap!+
  friend class ElstatPot;
  //!nowrap!-
  public:
  SimpleElectrolyteByAtoms (const AtomSet&);
  SimpleElectrolyteByAtoms (const AtomSet&, float ionic, float stl);
  SimpleElectrolyteByAtoms (const AtomSet&, float ionic, float rad, float stl);
  SimpleElectrolyteByAtoms (const SolvAccVol&);
  SimpleElectrolyteByAtoms (const SolvAccVol&, float ionic, float stl);
  // used by multiflex, dunno what the AtomSet is good for
  SimpleElectrolyteByAtoms (const AtomSet&, float ionic, const SolvAccVol& solv_acc);
  virtual ~SimpleElectrolyteByAtoms();
  float ionic_strength() const {return ionic_str;}
  //!nowrap!+
  virtual ElyCubeRep get_cuberep(const CubeLatSpec& clsp);
  const AtomSet& get_atomset() const; // For GB/DL's sake
  //!nowrap!-
private:
  float ionic_str, radius, sternl;
  SolvAccVol* solv_acc;
  AtomSet _ats;   // For GB/DL's sake
  bool _ats_defined;
};


/* added, TRU 2007
   This is a simple 1-value electrolyte environment In contrast to the
   ElectrolyteByAtoms this one has an improved Surface definition.
   The ion accessible surface is defined by solvent accessible surface + sternlayer.
*/


//!wrap!
class ElectrolyteByAtoms : public ElectrolyteEnvironment_lett {
// LANGUAGE BUG!!  The following friend declaration is OK with gcc
// but not cfront.  According to the ARM Sec 11.4 p. 251, cfront is right!
//    friend ElstatPot::ElstatPot (DielectricEnvironment, ChargeDist,
//                                 ElectrolyteEnvironment);
// So must get "overly friendly"...
  //!nowrap!+
  friend class ElstatPot;
  //!nowrap!-
  public:
  ElectrolyteByAtoms (const AtomSet&);
  ElectrolyteByAtoms (const AtomSet&, float ionic, float stl);
  ElectrolyteByAtoms (const AtomSet&, float ionic, float rad, float stl);
  ElectrolyteByAtoms (const IonAccVol&);
  ElectrolyteByAtoms (const IonAccVol&, float ionic, float stl);
  // used by multiflex, dunno what the AtomSet is good for
  ElectrolyteByAtoms (const AtomSet&, float ionic, const IonAccVol&);
  virtual ~ElectrolyteByAtoms();
  float ionic_strength() const {return ionic_str;}
  //!nowrap!+
  virtual ElyCubeRep get_cuberep(const CubeLatSpec& clsp);
  const AtomSet& get_atomset() const; // For GB/DL's sake
  //!nowrap!-
private:
  float ionic_str, radius, sternl;
  IonAccVol ion_acc;
  AtomSet _ats;   // For GB/DL's sake
  bool _ats_defined;
};


#endif

// ElectrolyteByAtoms.h ends here
