#include "MEAD_e/ElySlab.h"
#include "MEAD_e/globals.h"

ElySlab::ElySlab(float ionic, float zl, float zu)
 : ElectrolyteEnvironment_lett(),
   ionic_str(ionic), zlower(zl), zupper(zu)
{
  // Sanity checks
  if (zlower >= zupper) {
    cerr << "ERROR: ElySlab ctor called with zlower >= zupper\n"
      << "(" << zlower << " >= " << zupper << ")" << endl;
    ::error();
  }
  if (ionic_str < 0.00) {
    cerr << "ERROR: ElySlab ctor called with ionic_str < 0.00\n"
      << "(" << ionic_str << " < 0.00)" << endl;
    ::error();
  }
}


ElyCubeRep ElySlab::get_cuberep (const CubeLatSpec& cls)
{
  blab3 << "ElySlab::get_cuberep called.\n"
    << "\nSetting all cubic electrolyte grid points between z = " << zupper
      << " and z = " << zlower << " to a value of zero\n"
        << "and other points to " << ionic_str << endl;

  int grid_dim = cls.get_grid_dim();
  int nsq = grid_dim*grid_dim;
  float spacing = cls.get_spacing();
  Coord grid_center_in_space = cls.get_center();

  float grlen = (float) (grid_dim - 1);
  float halfgrlen = grlen/2;

  Coord grid_center_in_grid (halfgrlen, halfgrlen, halfgrlen);

  float zupper_in_grid = (zupper - grid_center_in_space.z)
    / spacing + grid_center_in_grid.z;
  float zlower_in_grid = (zlower - grid_center_in_space.z)
    / spacing + grid_center_in_grid.z;


  ElyCubeRep dcr(cls);
  for (int i = 0; i<grid_dim; ++i) {
    int iterm = i*nsq;
    for (int j = 0; j<grid_dim; ++j) {
      int jterm = j*grid_dim;
      int ijterms = iterm + jterm;
      for (int k = 0; k<grid_dim; ++k) {
	int h = ijterms + k;
	float z = (float) k;
	dcr[h] = z > zlower_in_grid && z < zupper_in_grid ? 0.0 : ionic_str;
      }
    }
  }

  dcr.declare_defined();
  return dcr;
}

// ElySlab.cc ends here
