#include "MEAD_e/ElySphere.h"
#include "MEAD_e/CubeLatSpec.h"

ElySphere::ElySphere (float ionic, float rad, Coord cntr)
 : ElectrolyteEnvironment_lett(), ionic_str (ionic), radius(rad), center(cntr)
{
}

ElyCubeRep ElySphere::get_cuberep (const CubeLatSpec& cls)
{
  blab3 << "ElySphere::get_cuberep called.\n"
    << "Setting all cubic electrolyte grid points not within " << radius
      << " of center to zero\nand other points to " << ionic_str << endl;

// FIXME?  If I was sure inlining of the cls functions produced just
// as fast code I could use cls.get.. () directly instead of these
// local variables.

  int grid_dim = cls.get_grid_dim();
  int nsq = grid_dim*grid_dim;
  float spacing = cls.get_spacing();
  Coord grid_center_in_space = cls.get_center();

  float grlen = (float) (grid_dim - 1);
  float halfgrlen = grlen/2;

  Coord grid_center_in_grid (halfgrlen, halfgrlen, halfgrlen);
  Coord sphere_center_in_grid = (center - grid_center_in_space)
    / spacing + grid_center_in_grid;
  float radius_squared = radius / spacing;
  radius_squared *= radius_squared;

  ElyCubeRep dcr(cls);
  for (int i = 0; i<grid_dim; ++i) {
    int iterm = i*nsq;
    float dxsq = ((float) i) - sphere_center_in_grid.x;
    dxsq *= dxsq;
    for (int j = 0; j<grid_dim; ++j) {
      int jterm = j*grid_dim;
      int ijterms = iterm + jterm;
      float dysq = ((float) j) - sphere_center_in_grid.y;
      dysq *= dysq;
      float dxsq_plus_dysq = dxsq + dysq;
      for (int k = 0; k<grid_dim; ++k) {
	int h = ijterms + k;
	float dzsq = ((float) k) - sphere_center_in_grid.z;
	dzsq *= dzsq;
	float rsq = dxsq_plus_dysq + dzsq;
	dcr[h] = rsq > radius_squared ? ionic_str : 0.0;
      }
    }
  }

  dcr.declare_defined();
  return dcr;
}


// ElySphere.cc ends here
