/*  A form of a GCEM Site

    label, bound ligands, charge and occupancy of atoms

    Copyright (C) 2007 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.
*/
#include "MEAD_e/GcemFormDef.h"
#include "MEAD_e/globals.h"
#include <fstream>

GcemFormDef::GcemFormDef(const GcemFormDef& a)
: name(a.name), gmodel(a.gmodel), epsref(a.epsref), istrref(a.istrref),
  linkmap(a.linkmap), ligandmap(a.ligandmap), atommap(a.atommap)
{
}
GcemFormDef::GcemFormDef()
: name(""), gmodel(0), epsref(-1), istrref(-1)
{
}

GcemFormDef::GcemFormDef (const string& nm)
: name(nm), gmodel(0), epsref(-1), istrref(-1)
{
}

GcemFormDef::GcemFormDef(const string& nm, const float gm, const float er, const float ir)
: name(nm), gmodel(gm), epsref(er), istrref(ir)
{
}

void GcemFormDef::print_ligandmap(){
     cout << "Map of CharmmIDs and atom types:\n" << endl;
     cout << "RESNAME:ATNAME : ATTYPE\n" << endl;
     for (std::map<string,int>::const_iterator i = ligandmap.begin(); i != ligandmap.end(); ++i) {
       cout << i->first << " : " << i->second << endl;
     }
}

void GcemFormDef::print_linkmap(){
     cout << "Multimap of link AtomIDs:\n" << endl;
     cout << "AtomID : AtomID\n" << endl;
     for (std::multimap<AtomID,AtomID>::const_iterator i = linkmap.begin(); i != linkmap.end(); ++i) {
       cout << i->first << " : " << i->second << endl;
     }
}

void GcemFormDef::print_atommap(){
     cout << "Map of CharmmIDs and atom types:\n" << endl;
     cout << "RESNAME:ATNAME : CHARGE\n" << endl;
     for (std::map<CharmmID,pair<float,int> >::const_iterator i = atommap.begin(); i != atommap.end(); ++i) {
       cout << i->first << " : " << (i->second).first << (i->second).second << endl;
     }
}

// GcemFormDef.cc ends here
