/*
    helper object for GcemMolecule
    creates GcemSites from GcemSiteDefs and AtomSets

    Copyright (c) 2007-2011 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.
*/
#ifndef _GcemInstanceFilter_h
#define _GcemInstanceFilter_h 1

#include <vector>
#include <map>
#include <list>
#include <string>
#include "MEAD_e/CharmmID.h"
#include "MEAD_e/InstanceID.h"
#include "MEAD_e/GcemMolecule.h"
#include "MEAD_e/GcemSite.h"
#include "MEAD_e/globals.h"
#include <fstream>
#include <boost/regex.hpp>
#include <boost/filesystem.hpp>
// these are the irregularly shaped n-dimensional matrices
#include "./irreg_matrices/vector.hpp"    // 1D
#include "./irreg_matrices/matrix.hpp"    // 2D
#include "./irreg_matrices/tensor.hpp"    // 3D
#include "./irreg_matrices/d4_tensor.hpp" // 4D

using std::vector;
using std::list;
using std::map;
using std::pair;
using std::string;

//!wrap!
class GcemInstanceFilter {
public:
  GcemInstanceFilter(){}
  ~GcemInstanceFilter () {}

private:
  friend class GcemMolecule;
  void filter_instances(const bool, GcemMolecule &molecule) const;
  void filter_instances_gs(const bool, GcemMolecule &molecule) const;
  // with electrostatic terms, return number of removed instances
  int cap_instances(const int, GcemMolecule &molecule) const;
  int filter_instances_full(const bool, GcemMolecule &molecule) const;
  int filter_instances_gs_full(const bool, GcemMolecule &molecule) const;
};



#endif

// GcemInstanceFilter.h ends here
