/*  A GCEM molecule containing titratable and/or flexible sites

    Copyright (C) 2007-2011 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.
*/
#include "MEAD_e/GcemMolecule.h"

using namespace std;
using namespace boost;
namespace fs = boost::filesystem;

GcemMolecule::GcemMolecule(const GcemMolecule& a)
: name(a.name), mmg(a.mmg), pbg(a.pbg), conmap(a.conmap), rotmap(a.rotmap),
  sites(a.sites), background(a.background), input_pqr(a.input_pqr),
  sitedef(a.sitedef), chmpara(a.chmpara), rlib(a.rlib), do_postproc(a.do_postproc),
  use_filter(a.use_filter), use_empirical(a.use_empirical), coulomb_14_scalefac(a.coulomb_14_scalefac),
  filter_cutoff(a.filter_cutoff), filter_cutoff_gs(a.filter_cutoff_gs), eps_ff(a.eps_ff),
  use_term_l_e(a.use_term_l_e), use_term_l_lj(a.use_term_l_lj),
  use_term_l_bond(a.use_term_l_bond), use_term_l_angle(a.use_term_l_angle),
  use_term_l_dihedral(a.use_term_l_dihedral), use_term_l_improper(a.use_term_l_improper),
  use_term_ml_e(a.use_term_ml_e), use_term_ml_lj(a.use_term_ml_lj),
  use_term_ml_bond(a.use_term_ml_bond), use_term_ml_angle(a.use_term_ml_angle),
  use_term_ml_dihedral(a.use_term_ml_dihedral), use_term_ml_improper(a.use_term_ml_improper),
  use_term_s_e(a.use_term_s_e), use_term_s_lj(a.use_term_s_lj),
  use_term_s_bond(a.use_term_s_bond), use_term_s_angle(a.use_term_s_angle),
  use_term_s_dihedral(a.use_term_s_dihedral), use_term_s_improper(a.use_term_s_improper),
  use_term_ms_e(a.use_term_ms_e), use_term_ms_lj(a.use_term_ms_lj),
  use_term_ms_bond(a.use_term_ms_bond), use_term_ms_angle(a.use_term_ms_angle),
  use_term_ms_dihedral(a.use_term_ms_dihedral), use_term_ms_improper(a.use_term_ms_improper),
  print_chm(a.print_chm), print_debug(a.print_debug),
  epsin1(a.epsin1), epsin2(a.epsin2), epsin3(a.epsin3), epscore(a.epscore),
  epshead(a.epshead),epsext(a.epsext),zminh(a.zminh),zminc(a.zminc),zmaxc(a.zmaxc),zmaxh(a.zmaxh),
  membholer(a.membholer), membholex(a.membholex), membholey(a.membholey),
  ionicstr(a.ionicstr),ionicstr1(a.ionicstr1),ionicstr2(a.ionicstr2),
  has_memb(a.has_memb), do_cap(a.do_cap), inside(a.inside),
  meadpath(a.meadpath), eps1set(a.eps1set), eps2set(a.eps2set), eps3set(a.eps3set),
  channelset(a.channelset), elycavset(a.elycavset),
  capacitance(a.capacitance), vmemb_inter(a.vmemb_inter), grid_detail(a.grid_detail)
{
}

GcemMolecule::GcemMolecule()
: name("")
{
 set_default();
}

GcemMolecule::GcemMolecule (const string& nm)
: name(nm)
{
 set_default();
}

GcemMolecule::GcemMolecule (const string& nm, const string& rlibn, const string& chmtopn, const string& chmparn)
: name(nm)
{
 set_default();
 //prepare_gcem(nm, rlibn, chmtopn, chmparn);
}

GcemMolecule::GcemMolecule (const string& nm, const string& rlibn, const string& chmtopn, const string& chmparn, const bool up)
: name(nm)
{
 set_default();
 use_empirical = up;
 //prepare_gcem(nm, rlibn, chmtopn, chmparn);
}

GcemMolecule::GcemMolecule (const string& nm, const string& rlibn, const string& chmtopn, const string& chmparn, const double ctof)
: name(nm)
{
 set_default();
 use_filter = true;
 filter_cutoff_gs = ctof;
 filter_cutoff    = ctof + 10;
 //prepare_gcem(nm, rlibn, chmtopn, chmparn);
}

GcemMolecule::GcemMolecule (const string& nm, const string& rlibn, const string& chmtopn, const string& chmparn, const double ctof, const bool up)
: name(nm)
{
 set_default();
 use_empirical = up;
 use_filter = true;
 filter_cutoff_gs = ctof;
 filter_cutoff    = ctof + 10;
 //prepare_gcem(nm, rlibn, chmtopn, chmparn);
}

void GcemMolecule::set_default()
{
  gcemdir              = string("gcem_input");
  capacitance          = 0;
  vmemb_inter          = 0;
  eps_ff               = 1;
  coulomb_14_scalefac  = 1;
  filter_cutoff        = 30;
  filter_cutoff_gs     = 10;
  epsin1               = 1;
  epsin2               = 4;
  epsin3               = 80;
  epsext               = 80;
  epscore              = 2;
  epshead              = 15;
  ionicstr             = 0.15;
  ionicstr1            = 0.15;
  ionicstr2            = 0.15;
  has_memb             = false;
  has_con              = false;
  zmaxh                = 0;
  zmaxc                = 0;
  zminc                = 0;
  zminh                = 0;
  meadpath             = "";
  // trigger post-processing
  do_postproc          = false;
  // capacitance, membrane, connectivities
  do_cap               = false;
  has_memb             = false;
  has_con              = false;
  // output flags
  print_chm            = false;
  print_debug          = false;
  // use knowledge based potential for intrinsic energy of Site
  use_empirical        = false;
  use_filter           = false;
  // terms involving ligand atoms and no model atoms
  // are floored to lowest energy instance of same form to avoid double counting in Gmodel
  use_term_l_e         = true;
  use_term_l_lj        = true;
  use_term_l_bond      = false;
  use_term_l_angle     = false;
  use_term_l_dihedral  = true;
  use_term_l_improper  = true;
  // terms involving ligand atoms and model atoms (ligand model interactions belongs to mmback)
  // are floored to lowest energy instance of same form to avoid double counting in Gmodel
  use_term_ml_e        = false; // done later with LPBE (G_model_back in GCEM)
  use_term_ml_lj       = true;
  use_term_ml_bond     = false;
  use_term_ml_angle    = false;
  use_term_ml_dihedral = true;
  use_term_ml_improper = true;
  // terms involving no ligand and no model atoms (internal site)
  use_term_s_e         = true; // are floored to lowest energy instance of same form to avoid double counting in Gmodel
  use_term_s_lj        = true;
  use_term_s_bond      = false;
  use_term_s_angle     = false;
  use_term_s_dihedral  = true;
  use_term_s_improper  = true;
  // terms involving no ligand but model atoms (site/model interaction mmback)
  use_term_ms_e        = false; // done later with LPBE (G_model_back in GCEM), and then should be floored again to be consistent!?
  use_term_ms_lj       = true;
  use_term_ms_bond     = false;
  use_term_ms_angle    = false;
  use_term_ms_dihedral = true;
  use_term_ms_improper = true;
  // skip Lennard-Jones Interactions among non-flexible sites
  skip_stat_pair_mmg   = false;
  // skip intrinsic energies of non-flexible sites
  skip_stat_mmint      = true;
  // very fine and large inner grids, ensures high-quality solvation and interaction energies
  grid_detail          = 3;
}

void GcemMolecule::postproc_gcem()
{
  // restore GcemMolecule from preprocessor / last run
  read_restart_file(string(name + ".restart.dat"));

  // connectivity definitions not defined in preprocessing run,
  // but membrane was defined -> retry:
  // set has_con and read connectivities to membrane sides if nm.con exists
  if (has_memb && !has_con){
   read_con(string(name + ".con"));
  }

  // read Electrostatic Energies from MEAD output files
  read_mead_out();
  write_gmct_input();
}

void GcemMolecule::prepare_filtered_gcem()
{prepare_filtered_gcem((int)0, (bool)false);}
void GcemMolecule::prepare_filtered_gcem(const int cap_max_ninst, const bool keep_predef)
{
  // filter instances once again if rotameric flexibility was included,
  // reduce error caused by inflation of dielectric volume in second run
  if (use_filter){
   // temporary helper object
   GcemInstanceFilter ifltr;
   int n_removed_insts     = ifltr.filter_instances_full(keep_predef,*this);
   int n_removed_insts_gs  = ifltr.filter_instances_gs_full(keep_predef,*this);
   int n_removed_insts_cap = ifltr.cap_instances(cap_max_ninst,*this);
   // even if no instance was eliminated, we might want to refine the electrostatics settings
   if (1 == 1 || n_removed_insts != 0 || n_removed_insts_gs != 0){
    // writer restart file for possible next refinement cycle
    write_restart_file(string("filtered." + name + ".restart.dat"));
    // write structure files for electrostatics, job script(s) and QMPB  input script
    // temporary helper object
    GcemSolverInput gsi;
    gsi.write_gcem_input("filtered." + gcemdir, name + ".qmpb.in", *this);
   }
  }
}

void GcemMolecule::prepare_gcem()
{
  prepare_gcem(name, "rotlib", "top.inp", "par.inp", bool(false));
}

void GcemMolecule::prepare_gcem(const string& nm, const string& rlibn, const string& chmtopn,
                                const string& chmparn, const bool keep_predef)
{
  // read structure from pqr file
  input_pqr = AtomSet(nm);
  input_pqr.read();
  // set has_con and read connectivities to membrane sides if nm.con exists
  if (has_memb){
   read_con(string(nm + ".con"));
  }
  // read site definitions and parameters
  read_rot(string(nm + ".rot"));
  // only read rotamer library if there are flexible sites
  if (contains_rotmap_value(1) || contains_rotmap_value(2)){
   rlib.set_name(rlibn);
   // decides automatically read from binary if rlibn.dat exists otherwise expects rlibn.txt
   rlib.read();
  }
  if (!(!contains_rotmap_value(1) && !contains_rotmap_value(2) && skip_stat_mmint && skip_stat_pair_mmg)){
   chmpara.read(chmtopn, chmparn);
  }
  if (print_chm) chmpara.print();
  // read .est files
  read_site_defs();
  // coordinate construction of sites
  create_sites();
  /* THIS IS FOR DEBUGGING / COMPARISON TO CHARMM ONLY !!!
  round_coordinates();
  */
  // calculate MM part of intrinsic and interaction energies
  calc_mmint();
  // temporary helper object
  GcemInstanceFilter ifltr;
  if (use_filter) ifltr.filter_instances(keep_predef,*this);
  calc_mmg();
  if (use_filter) ifltr.filter_instances_gs(keep_predef,*this);
  // write QMPB input script and structure files for electrostatics
  // temporary helper object
  GcemSolverInput gsi;
  gsi.write_gcem_input(gcemdir, nm + ".qmpb.in", *this);
  // write energie matrices to .mmint and .mmg
  write_mmint(gcemdir + "/" + nm + ".mmint");
  write_mmg(gcemdir + "/" + nm + ".mmg");
  // save GcemMolecule to file for postprocessor run
  write_restart_file(string(name + ".restart.dat"));
}

void GcemMolecule::create_sites()
{
  // temporary helper object
  GcemSiteCreator site_creator;
  
  site_creator.create_sites(*this);
}

void GcemMolecule::calc_mmint()
{
  // calculate intrinsic energies
  // we need to loop over all instances of all sites
  // and calculate Gint from the pure CHARMM energy function or a combination
  // of CHARMM energy function and a knowledge based potential derived from the
  // relative propensities in the rotamer database

  // temporary helper object
  GcemCharmmEnergyCalculator ener_calc;
  if (use_empirical)
   ener_calc.calc_mmint_propensities(*this);
  else
   ener_calc.calc_mmint_charmm(*this);
  if (contains_rotmap_value(4))
   ener_calc.calc_mmint_charmm_qm(*this);
}

void GcemMolecule::calc_mmg()
{
  // temporary helper object
  GcemCharmmEnergyCalculator ener_calc;
  ener_calc.calc_mmg(*this);
}

void GcemMolecule::write_mmint(const string& fn)
{

  blab2 << "Entering GcemMolecule::write_mmint" << endl;

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_mmint failed to open file for writing, ",
            fn.c_str());

   list<string> ligands = get_ligands();

   for (list<GcemSite>::const_iterator i=sites.begin(); i!=sites.end(); ++i){
    int c((i->get_sid()).get_chain());
    int s((i->get_sid()).get_site());
    string str_c, str_s;
    stringstream ss1,ss2;
    ss1 << c; ss1 >> str_c;
    ss2 << s; ss2 >> str_s;
    int iid = -1;
    for (list<GcemInstance>::const_iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
     iid++;
     string str_in, slbl, ilbl;
     double gmmint(j->get_gmmint());
     list<string> liglist;
     stringstream ss3;
     //int in((j->get_instid()).get_instance());
     ss3 << iid; ss3 >> str_in;
     slbl = (i->get_sid()).get_sitename() + "-" + str_c + "-" + str_s;
     ilbl = (j->get_instid()).get_instname() + "-" + str_in;
     int w1 = slbl.size();
     if (w1<13){w1=13;}
     int w2 = ilbl.size();
     if (w2<10){w2=10;}
     of << right << setw(w1) << slbl << "  " << setw(w2) << ilbl
        << setw(9) << setprecision(3) << fixed;

     if (gmmint > -10000 && gmmint < 10000)
      of << setw(9) << setprecision(3) << fixed << gmmint;
     else
      of << " " << setprecision(6) << scientific << gmmint;

     for (list<string>::const_iterator l=ligands.begin();l!=ligands.end();l++)
      of << setw(3) << j->ligands_bound(*l);
     of << endl;

    }
   }

  if (! of.good())
    ::error("GcemMolecule::write_mmint failed to write file, ",
            fn.c_str());
  of.close();

  blab1 << "Exiting GcemMolecule::write_mmint" << endl;

}

void GcemMolecule::write_mmg(const string& fn)
{

  blab2 << "Entering GcemMolecule::write_mmg" << endl;

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_mmg failed to open file for writing, ",
            fn.c_str());

   for (list<GcemSite>::const_iterator i=sites.begin(); i!=sites.end(); ++i){
    int ci((i->get_sid()).get_chain());
    int si((i->get_sid()).get_site());
    string str_ci, str_si;
    stringstream ss1i,ss2i;
    ss1i << ci; ss1i >> str_ci;
    ss2i << si; ss2i >> str_si;
    int iidi = -1;
    for (list<GcemInstance>::const_iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
     iidi++;
     int inj((j->get_instid()).get_instance());
     string str_inj;
     stringstream ss3j;
     ss3j << iidi; ss3j >> str_inj;
     for (list<GcemSite>::const_iterator ii=sites.begin(); ii!=sites.end(); ++ii){
      int cii((ii->get_sid()).get_chain());
      int sii((ii->get_sid()).get_site());
      string str_cii, str_sii;
      stringstream ss1ii,ss2ii;
      ss1ii << cii; ss1ii >> str_cii;
      ss2ii << sii; ss2ii >> str_sii;
      int iidii = -1;
      for (list<GcemInstance>::const_iterator jj=ii->instances.begin(); jj!=ii->instances.end(); ++jj){
       iidii++;
       int injj((jj->get_instid()).get_instance());
       string str_injj;
       stringstream ss3jj;
       ss3jj << iidii; ss3jj >> str_injj;
       string slbl1, slbl2, ilbl1, ilbl2;
       slbl1 = (i->get_sid()).get_sitename() + "-" + str_ci + "-" + str_si;
       ilbl1 = (j->get_instid()).get_instname() + "-" + str_inj;
       slbl2 = (ii->get_sid()).get_sitename() + "-" + str_cii + "-" + str_sii;
       ilbl2 = (jj->get_instid()).get_instname() + "-" + str_injj;
       int w11 = slbl1.size();
       if (w11<13){w11=13;}
       int w21 = ilbl1.size();
       if (w21<10){w21=10;}
       int w12 = slbl2.size();
       if (w12<13){w12=13;}
       int w22 = ilbl2.size();
       if (w22<10){w22=10;}
       double gmmg(get_mmg(j->get_instid(),jj->get_instid()));
       of << right << setw(w11) << slbl1 << "  " << setw(w21) << ilbl1
          << right << setw(w12) << slbl2 << "  " << setw(w22) << ilbl2;
       if (gmmg > -10000 && gmmg < 10000)
        of << setw(9) << setprecision(3) << fixed << gmmg << endl;
       else
        of << " " << setprecision(6) << scientific << gmmg << endl;
      }
     }
    }
   }

  if (! of.good())
    ::error("GcemMolecule::write_mmg failed to write file, ", fn.c_str());
  of.close();


  blab1 << "Exiting GcemMolecule::write_mmg" << endl;

}

void GcemMolecule::print_mmint() const
{

  blab2 << "Entering GcemMolecule::print_mmint" << endl;

  for (list<GcemSite>::const_iterator i=sites.begin(); i!=sites.end(); ++i){
   cout << "\n---------------------------------------------------------------------\n" << endl;
   cout << "printing intrinsic MM energy of site " << i->get_sid() << " :\n" << endl;
   cout << "InstanceID      Gmmint      Gmmsite   Gmmmodelback   Gmmback" << endl;
   for (list<GcemInstance>::const_iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
    cout << j->get_instid() << "      " << j->get_gmmint() << "      "
         << j->get_gmmsite()  << "   " << j->get_gmmmodelback()  << "   " << j->get_gmmback() << endl;
   }
   cout << "\n---------------------------------------------------------------------\n" << endl;
  }

  blab1 << "Exiting GcemMolecule::print_mmint" << endl;

}

void GcemMolecule::print_mmg() const
{

  blab2 << "Entering GcemMolecule::print_mmg" << endl;

  cout << "\n---------------------------------------------------------------------\n" << endl;
  cout << "printing interaction energies " << ":\n" << endl;
  cout << "SiteID   InstanceID      SiteID   InstanceID      Gmmg" << endl;

  for (list<GcemSite>::const_iterator i=sites.begin(); i!=sites.end(); ++i){
   for (list<GcemInstance>::const_iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
    list<GcemSite>::const_iterator ii = i;
    for (ii++; ii!=sites.end(); ++ii){
     for (list<GcemInstance>::const_iterator jj=ii->instances.begin(); jj!=ii->instances.end(); ++jj){
      cout << i->get_sid()  << "   " << j->get_instid()  << "      "
           << ii->get_sid() << "   " << jj->get_instid() << "      "
           << setw(20) << setprecision(6) << fixed
           << get_mmg(j->get_instid(), jj->get_instid()) << endl;
     }
    }
   }
  }

  cout << "\n---------------------------------------------------------------------\n" << endl;

  blab1 << "Exiting GcemMolecule::print_mmg" << endl;

}

void GcemMolecule::read_site_defs(){

  blab2 << "Entering GcemMolecule::read_site_defs" << endl;

  // read .est files for all titratable sites
  for (map<SiteID,int>::const_iterator i = rotmap.begin(); i!= rotmap.end(); ++i){
   // titratable and flexible + titratable sites
   if (i->second == 1 || i->second == 3){
    string nm = i->first.get_sitename();
    if(!contains_sitedef(nm)){
     GcemSiteDef sd(nm);
     sd.read_est();
     insert_sitedef(sd);
     //////// debug ////////
     if(print_debug){
      sd.print_est();
     }
     ///////////////////////
    }
   }
   else if (i->second == 4){
    string nm = i->first.get_sitename();
    if(!contains_sitedef(nm)){
     GcemSiteDef sd(nm, i->first);
     sd.read_qst(i->first.get_fn_prefix() + ".qst");
     insert_sitedef(sd);
     //////// debug ////////
     if(print_debug){
      sd.print_qst();
     }
     ///////////////////////
    }
   }
  }

  blab1 << "Exiting GcemMolecule::read_site_defs" << endl;
}

void GcemMolecule::read_rot ()
{
  string rot_fn = name + DOT + "rot";
  read_rot(rot_fn);
}

void GcemMolecule::read_con ()
{
  string con_fn = name + DOT + "con";
  read_con(con_fn);
}

void GcemMolecule::read_rot (const string& fn_string){

  blab2 << "GcemMolecule::read_rot: starting\n";

  const char *fn = fn_string.c_str();
  string line;
  int nsites = 0; int ntflex = 0; int nt = 0; int nflex = 0; int nq = 0; int nign = 0;

  boost::regex re_rot("^\\s*(\\w+)-(\\d+)-(\\d+)\\s+(\\d+)");

  boost::match_results<string::const_iterator> match;
  boost::match_flag_type flags = boost::match_default;

  ifstream rotfile(fn);

  if (!rotfile)
    ::error ("GcemMolecule::read_rot: Cannot open input file, ", fn, "\n");

  // start of parsing
  while (!rotfile.eof() ){

   getline (rotfile, line);
   string::const_iterator start, end;
   start = line.begin();
   end   = line.end();

    if (boost::regex_search(start, end, match, re_rot, flags)){
     int cn, sn, sitetype;
     string sitelabel = string(match[1].first, match[1].second);
     istringstream ist1(string(match[2].first, match[2].second));
     ist1 >> cn;
     istringstream ist2(string(match[3].first, match[3].second));
     ist2 >> sn;
     istringstream ist3(string(match[4].first, match[4].second));
     ist3 >> sitetype;
     if (sitetype == 0){
      // dont need to consider non-flexible and non-titrateable sites
      nign++;
      continue;
     }

     pair<map<SiteID,int>::iterator,bool> tp;
     tp = insert_rotmap_entry(SiteID(sitelabel, 0, cn, sn), sitetype);
     if (tp.second){
      nsites++;
      if      (sitetype == 1){ntflex++;}
      else if (sitetype == 2){nflex++;}
      else if (sitetype == 3){nt++;}
      else if (sitetype == 4){nq++;}
      else {
       cerr << "GcemMolecule::read_rot: Sitetype must be one of:"
            << "   0 = non-titratable non-flexible\n"
            << "   1 =     titratable     flexible\n"
            << "   2 = non-titratable     flexible\n"
            << "   3 =     titratable non-flexible\n"
            << "   4 =     QM site\n" << endl;
       ::error ("GcemMolecule::read_rot: unknown site type, ", line.c_str(), "\n");
      }
     }
    }


  } // end of parsing

  rotfile.close();

  ///// debug /////
  if (print_debug){
   for (map<SiteID,int>::const_iterator i = rotmap.begin(); i!= rotmap.end(); ++i){
    cout << i->first << "   " << i->second << endl;
   }
  }
  /////////////////

  blab2  << "GcemMolecule::read_rot: finished with " <<
  nsites << " sites of which " <<
  ntflex << " are titratable and flexible, " <<
  nflex  << " are non-titratable and flexible, " <<
  nt     << " are titratable and non-flexible, " <<
  nq     << " are QM sites\n" <<
  nign   << " non-titratable and non-flexible sites were ignored." << endl;

  blab1 << "GcemMolecule::read_rot: exiting\n";
}

void GcemMolecule::read_con(const string& fn_string){

  blab2 << "GcemMolecule::read_con: starting\n";

  const char *fn = fn_string.c_str();
  string line;
  int nsites = 0;
  int nin    = 0;
  int nout   = 0;

  boost::regex re_con("^\\s*(\\w+)-(\\d+)-(\\d+)\\s+(\\d+)");

  boost::match_results<string::const_iterator> match;
  boost::match_flag_type flags = boost::match_default;

  ifstream confile(fn);

  if (!has_memb){
   blab1 << "GcemMolecule::read_con: No membrane defined, exiting\n";
   return;
  }

  if (!confile){
   cerr << "WARNING: File with connectivity definitions " << fn << " not found while membrane is defined." << endl
        << "         Will assume equal potentials on membrane sides!\n" << endl;
   has_con = false;
  }
  else has_con = true;

  if (has_con) {
   // start of parsing
   while (!confile.eof()){

   getline (confile, line);
   string::const_iterator start, end;
   start = line.begin();
   end   = line.end();

    if (boost::regex_search(start, end, match, re_con, flags)){
     int cn, sn, connectivity;
     string sitelabel = string(match[1].first, match[1].second);
     istringstream ist1(string(match[2].first, match[2].second));
     ist1 >> cn;
     istringstream ist2(string(match[3].first, match[3].second));
     ist2 >> sn;
     istringstream ist3(string(match[4].first, match[4].second));
     ist3 >> connectivity;

     pair<map<SiteID,int>::iterator,bool> tp;
     tp = insert_conmap_entry(SiteID(sitelabel, 0, cn, sn), connectivity);
     if (tp.second){
      nsites++;
      if      (connectivity == 1){nin++;}
      else if (connectivity == 0){nout++;}
      else {
       cerr << "GcemMolecule::read_con: connectivity must be one of:"
            << "   1 = inside\n"
            << "   0 = outside\n" << endl;
       ::error("GcemMolecule::read_con: unknown connectivity, ", line.c_str(), "\n");
      }
     }
    }
   } // end of parsing

   confile.close();

   ///// debug /////
   if (print_debug){
    for (map<SiteID,int>::const_iterator i = conmap.begin(); i!= conmap.end(); ++i){
     cout << i->first << "   " << i->second << endl;
    }
   }
   /////////////////

   blab2  << "GcemMolecule::read_con: finished with " <<
   nsites << " sites of which " <<
   nin    << " are connected to the inner side and " <<
   nout   << " are connected to the outer side."     << endl;

  } // end has_con

  blab1 << "GcemMolecule::read_con: exiting\n";
}

// this function calls GcemSolverOutput functions for the actual tasks
void GcemMolecule::read_mead_out(){

  blab2 << "GcemMolecule::read_mead_out: starting\n";

  // clear old interaction energies from possible previous run
  pbg.clear();

  // read Electrostatic Energies from MEAD output files
  GcemSolverOutput solver_output;
  solver_output.read_mead_out(*this, gcemdir, name, sites, has_memb, has_con, do_cap, &capacitance, &vmemb_inter);

  int sid1=0, sid2=0, iid1=0, iid2=0;
  double tmpd=0;

  // make sparse interaction matrix
  for (list<GcemSite>::iterator s1=sites.begin(); s1!=sites.end(); ++s1){
   sid2=0;
   for (list<GcemSite>::iterator s2=sites.begin(); s2!=s1; ++s2){
    iid1=0;
    for (list<GcemInstance>::iterator i1=s1->instances.begin(); i1!=s1->instances.end(); ++i1){
     iid2=0;
     for (list<GcemInstance>::iterator i2=s2->instances.begin(); i2!=s2->instances.end(); ++i2){
      tmpd = 0.5 * (solver_output.inter(sid1,iid1,sid2,iid2) + solver_output.inter(sid2,iid2,sid1,iid1));
      if (tmpd > 5.0E-05 || tmpd < -5.0E-05) insert_pbg(i1->get_instid(), i2->get_instid(), tmpd);
      iid2++;
     }
     iid1++;
    }
    sid2++;
   }
   sid1++;
  }

  // force field terms are corrected by relative Born solvation energy for
  // transfer from force field parametrization environment to solvent environment
  // for each form need instance with minimum transfer energy as reference
  for (list<GcemSite>::iterator i=sites.begin(); i!=sites.end(); ++i){
   // find reference forms of instances with minimum (gmmint + gpbborn_sol - gpbborn_ff)
   map<string,vector<float> > min_energy_terms;
   map<string, int > min_energy_conformers;
   map<string,vector<float> >::iterator ri;
   map<string, int >::iterator ri2;
   map<string, list<GcemInstance> > imap(i->get_form_map());
   for (map<string, list<GcemInstance> >::const_iterator m=imap.begin(); m!=imap.end(); ++m){
    float tmpf1 = 0, tmpf2 = 0, tmpf3 = 0, tmpf4 = 0;
    float minene = numeric_limits<float>::max();
    int confnum_ref = 0;
    for (list<GcemInstance>::const_iterator j=m->second.begin(); j!=m->second.end(); ++j){
     float tmpfa = j->get_gpbborn_prot();
     float tmpfb = j->get_gpbborn_sol();
     float tmpfc = j->get_gpbborn_ffe();
     float tmpfd = j->get_gmmint();
     float tmpene = (tmpfb - tmpfc) + tmpfd;
     if (tmpene < minene){
      tmpf1 = tmpfa;
      tmpf2 = tmpfb;
      tmpf3 = tmpfc;
      tmpf4 = tmpfd;
      confnum_ref = j->get_confnum();
      minene = tmpene;
     }
    }
    vector<float> inner_tmpvec;
    inner_tmpvec.push_back(tmpf1);
    inner_tmpvec.push_back(tmpf2);
    inner_tmpvec.push_back(tmpf3);
    inner_tmpvec.push_back(tmpf4);
    min_energy_terms.insert(pair<string, vector<float> >(m->first, inner_tmpvec));
    min_energy_conformers.insert(pair<string, int >(m->first, confnum_ref));
   }
   for (list<GcemInstance>::iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
    ri = min_energy_terms.find(j->get_name());
    ri2 = min_energy_conformers.find(j->get_name());
    if (ri != min_energy_terms.end() && ri2 != min_energy_conformers.end()){
     j->set_gpbborn_prot_ref(ri->second[0]);
     j->set_gpbborn_sol_ref(ri->second[1]);
     j->set_gpbborn_ffe_ref(ri->second[2]);
     j->set_gmmint_ref(ri->second[3]);
     j->set_confnum_ref(ri2->second);
    } else { // this should never happen
      ::error("GcemMolecule::read_mead_out failed to find form");
    }
   }
  }

  blab1 << "GcemMolecule::read_mead_out: exiting\n";
}

void GcemMolecule::write_gmct_input() {
  blab2 << "Entering GcemMolecule::write_gmct_input" << endl;

  // subdirectory for results, gmct input
  if (!fs::is_directory("results")){
   if (fs::exists("results")) fs::remove_all("results");
   fs::create_directory("results");
  }
  if (!fs::is_directory("results/conf")){
   if (fs::exists("results/conf")) fs::remove_all("results/conf");
   fs::create_directory("results/conf");
  }

  string fn_gint  = "results/conf/" + name + ".gint";
  write_gint(fn_gint);
  string fn_inter = "results/conf/" + name + ".inter";
  write_inter(fn_inter);
  if (has_memb && has_con){
   string fn_vmemb = "results/conf/" + name + ".vmemb";
   write_vmemb(fn_vmemb);
  }

  string fn_sum = "results/" + name + ".gcem.sum";
  write_sum(fn_sum);

  string fn_conf = "results/" + name + ".conf";
  write_conf(fn_conf);

  blab1 << "GcemMolecule::write_gmct_input: exiting\n";
}

void GcemMolecule::write_conf(const string &fn) {
  blab2 << "GcemMolecule::write_conf: starting\n";

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_conf failed to open file for writing, ",
            fn.c_str());
  // name of conf directory, global conformational energy,
  // capacitance, membrane potential interaction of background
  of << "conf   "
     << " " << setw(14) << setprecision(6) << scientific << (double)0
     << " " << setw(14) << setprecision(6) << scientific << capacitance
     << " " << setw(14) << setprecision(6) << scientific << vmemb_inter
     << " " << endl;

  if (! of.good())
    ::error("GcemMolecule::write_conf failed to write file, ",
            fn.c_str());
  of.close();

  blab2 << "GcemMolecule::write_conf: exiting\n";
}

void GcemMolecule::write_sum(const string &fn) {
  blab2 << "GcemMolecule::write_sum: starting\n";

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_sum failed to open file for writing, ", fn.c_str());

   of << "Sum file written by MEAD GCEM 0.5" << endl << endl;
   of << "energies by default in [kcal/mol]" << endl << endl;

   int sid1 = 0;
   for (list<GcemSite>::const_iterator s1=sites.begin(); s1!=sites.end(); ++s1){
    of << endl << "////////////////////////////////////////////////////////" << endl << endl;
    int chain1((s1->get_sid()).get_chain());
    int site1((s1->get_sid()).get_site());
    string str_c1, str_s1, slbl1;
    stringstream ss11,ss12;
    ss11 << chain1; ss11 >> str_c1;
    ss12 << site1;  ss12 >> str_s1;
    int iid1 = 0;
    slbl1 = (s1->get_sid()).get_sitename() + "-" + str_c1 + "-" + str_s1;
    of << "Site: " << (sid1+1) << "   " << slbl1 << endl;
    for (list<GcemInstance>::const_iterator i1=s1->instances.begin(); i1!=s1->instances.end(); ++i1){
     string str_in1, ilbl1;
     double gmmint(i1->get_gmmint());
     double gmmint_ref(i1->get_gmmint_ref());
     double gpbint(i1->get_gpbint());
     double gmodel(i1->get_gmodel());
     double epsref(i1->get_epsref());
     double istrref(i1->get_istrref());
     if (istrref < 0) istrref = ionicstr;
     if (epsref < 1)   epsref = epsext;
     double gint = gmmint - gmmint_ref + gpbint + gmodel;
     stringstream ss3;
     ss3 << iid1; ss3 >> str_in1;
     ilbl1 = (i1->get_instid()).get_instname() + "-" + str_in1;
     int w1 = slbl1.size();
     if (w1<13){w1=13;}
     int w2 = ilbl1.size();
     if (w2<10){w2=10;}
     of << endl << "--------------------------------------------------------" << endl << endl;
     of << right << setw(5) << (sid1+1) << " " << setw(5) << (iid1+1)
        << setw(w1) << slbl1 << "  " << setw(w2) << ilbl1 << endl << endl;

     of << "sum of intrinsic energy contributions: " << setw(9) << setprecision(3) << fixed << gint << endl << endl;

     of << " model energy: "
        << setw(9) << setprecision(3) << fixed
        << i1->get_gmodel() << endl << endl;

     of << " dielectric constant of reference environment: "
        << setw(9) << setprecision(3) << fixed
        << epsref << endl << endl;

     of << " ionic strength of reference environment in [mol/l]: "
        << setw(9) << setprecision(3) << fixed
        << istrref << endl << endl;

     of << " conformer number          : " << i1->get_confnum()     << endl;
     of << " reference conformer number: " << i1->get_confnum_ref() << endl << endl;

     of << " sum of MM contributions                       : " << setw(9) << setprecision(3) << fixed << gmmint << endl;
     of << " sum of MM contributions of reference conformer: " << setw(9) << setprecision(3) << fixed << gmmint_ref << endl;
     of << " difference                                    : " << setw(9) << setprecision(3) << fixed << (gmmint - gmmint_ref) << endl;

     of << "  (ligand+site)-(ligand+site)        interaction (ligand contributions relative to lowest energy conformer): "
        << setw(9) << setprecision(3) << fixed
        << i1->get_gmmsite() << endl;
     of << "  (ligand+site)-(model background)   interaction (ligand contributions relative to lowest energy conformer): "
        << setw(9) << setprecision(3) << fixed
        << i1->get_gmmmodelback() << endl;
     of << "  (ligand+site)-(rest of background) interaction                                                           : "
        << setw(9) << setprecision(3) << fixed
        << i1->get_gmmback() << endl;
     if (i1->get_has_propensity())
      of << "  rotamer propensity                                                                                       : "
         << setw(9) << setprecision(3) << fixed
         << i1->get_propensity() << endl;
     of << endl;

     of << " sum of PB contributions (Born and background energies): " << setw(9) << setprecision(3)
        << fixed << gpbint << endl;
     // flexible, non-titratable sites don't have this term
     if (get_rotmap_value(s1->get_sid()) != 2){
      of << "  Born energy:" << endl;
      of << "   site in (Three|Six|N)Dielectric: " << setw(9) << setprecision(3) << fixed
         << i1->get_gpbborn_prot() << endl;
      of << "   site in           TwoDielectric: " << setw(9) << setprecision(3) << fixed
         << i1->get_gpbborn_sol() << endl;
      of << "   difference                     : " << setw(9) << setprecision(3) << fixed
         << (i1->get_gpbborn_prot()-i1->get_gpbborn_sol()) << endl;
     }
     
     of << "  background energy:" << endl
        << "   site in (Three|Six|N)Dielectric: " << setw(9) << setprecision(3) << fixed
        << i1->get_gpbback_prot() << endl;
     of << "   model site in TwoDielectric    : " << setw(9) << setprecision(3) << fixed
        << i1->get_gpbback_sol() << endl;
     of << "   difference                     : " << setw(9) << setprecision(3) << fixed
        << (i1->get_gpbback_prot()-i1->get_gpbback_sol()) << endl;
     if (has_memb && has_con){
     of << "  membrane potential interaction energy at Vmemb = 1 V: "
        << "  " << setw(9) << setprecision(3) << fixed
        << i1->get_gpbvmemb() << endl;
     }

     if (get_rotmap_value(s1->get_sid()) == 1 || get_rotmap_value(s1->get_sid()) == 2){
     of << "  relative Born energy for MM energy correction:" << endl;
//      of << "   site in (Three|Six|N)Dielectric (this instance, reference instance)           : "
//         << setw(9) << setprecision(3) << fixed << i1->get_gpbborn_prot() << ", "
//         << setw(9) << setprecision(3) << fixed << i1->get_gpbborn_prot_ref() << endl;
      of << "   site in TwoDielectric reference environment(this instance, reference instance): "
         << setw(9) << setprecision(3) << fixed << i1->get_gpbborn_sol() << ", "
         << setw(9) << setprecision(3) << fixed << i1->get_gpbborn_sol_ref() << endl;
      of << "   site in TwoDielectric FF environment       (this instance, reference instance): "
         << setw(9) << setprecision(3) << fixed << i1->get_gpbborn_ffe() << ", "
         << setw(9) << setprecision(3) << fixed << i1->get_gpbborn_ffe_ref() << endl;
      of << "   relative difference  FF -> reference environment                              : "
         << setw(9) << setprecision(3) << fixed
         << (i1->get_gpbborn_sol()     - i1->get_gpbborn_ffe())
           -(i1->get_gpbborn_sol_ref() - i1->get_gpbborn_ffe_ref()) << endl;
     }

     iid1++;
    }
    sid1++;
   }

  if (! of.good())
    ::error("GcemMolecule::write_sum failed to write file, ", fn.c_str());
  of.close();

  blab2 << "GcemMolecule::write_sum: exiting\n";
}

void GcemMolecule::write_gint(const string &fn) {
  blab2 << "GcemMolecule::write_gint: starting\n";

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_gint failed to open file for writing, ",
            fn.c_str());

   list<string> ligands = get_ligands();

   int sid1 = 0;
   for (list<GcemSite>::const_iterator s1=sites.begin(); s1!=sites.end(); ++s1){
    if (s1->get_ninst()==0) continue;
    int chain1((s1->get_sid()).get_chain());
    int site1((s1->get_sid()).get_site());
    string str_c1, str_s1;
    stringstream ss11,ss12;
    ss11 << chain1; ss11 >> str_c1;
    ss12 << site1;  ss12 >> str_s1;
    int iid1 = 0;
    for (list<GcemInstance>::const_iterator i1=s1->instances.begin(); i1!=s1->instances.end(); ++i1){
     string str_in1, slbl1, ilbl1;
     double gmmint(i1->get_gmmint());
     double gmmint_ref(i1->get_gmmint_ref());
     double gpbint(i1->get_gpbint());
     double gmodel(i1->get_gmodel());
     double gint = gmmint - gmmint_ref + gpbint + gmodel;
     list<string> liglist;
     stringstream ss3;
     ss3 << iid1; ss3 >> str_in1;
     slbl1 = (s1->get_sid()).get_sitename() + "-" + str_c1 + "-" + str_s1;
     ilbl1 = (i1->get_instid()).get_instname() + "-" + str_in1;
     int w1 = slbl1.size();
     if (w1<13){w1=13;}
     int w2 = ilbl1.size();
     if (w2<10){w2=10;}
     of << right << setw(5) << (sid1+1) << " " << setw(5) << (iid1+1)
        << setw(w1) << slbl1 << "  " << setw(w2) << ilbl1 << "   "
        << setw(14) << setprecision(6) << scientific << gint;

     for (list<string>::const_iterator l=ligands.begin();l!=ligands.end();l++){
      // each ligand needs a separate potential on each membrane side
      // exept electrons - the electronic potential is considered to
      // equilibrate between the phases, due to the long range over
      // which ET can occur, actually the phases on the two membrane sides
      // can have different redox potential, but we mostly look at a single
      // protein complex embedded in the membrane
      if (has_memb && has_con){
       if ((*l)=="electron"){
        of << setw(3) << i1->ligands_bound(*l);
       } // connected to inside
       else if (get_conmap_value(s1->get_sid()) == 1){
        of << setw(3) << i1->ligands_bound(*l);
        of << setw(3) << 0;
       } // connected to outside
       else{
        of << setw(3) << 0;
        of << setw(3) << i1->ligands_bound(*l);
       }
      } // no membrane or both phases equal
      else {
        of << setw(3) << i1->ligands_bound(*l);
      }
     }
     of << endl;
     iid1++;
    }
    sid1++;
   }

  if (! of.good())
    ::error("GcemMolecule::write_gint failed to write file, ",
            fn.c_str());
  of.close();

  blab2 << "GcemMolecule::write_gint: exiting\n";
}

void GcemMolecule::write_vmemb(const string &fn) {
  blab2 << "GcemMolecule::write_gint: starting\n";

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_gint failed to open file for writing, ",
            fn.c_str());

   list<string> ligands = get_ligands();

   int sid1 = 0;
   for (list<GcemSite>::const_iterator s1=sites.begin(); s1!=sites.end(); ++s1){
    if (s1->get_ninst()==0) continue;
    int chain1((s1->get_sid()).get_chain());
    int site1((s1->get_sid()).get_site());
    string str_c1, str_s1;
    stringstream ss11,ss12;
    ss11 << chain1; ss11 >> str_c1;
    ss12 << site1;  ss12 >> str_s1;
    int iid1 = 0;
    for (list<GcemInstance>::const_iterator i1=s1->instances.begin(); i1!=s1->instances.end(); ++i1){
     string str_in1, slbl1, ilbl1;
     list<string> liglist;
     stringstream ss3;
     ss3 << iid1; ss3 >> str_in1;
     slbl1 = (s1->get_sid()).get_sitename() + "-" + str_c1 + "-" + str_s1;
     ilbl1 = (i1->get_instid()).get_instname() + "-" + str_in1;
     int w1 = slbl1.size();
     if (w1<13){w1=13;}
     int w2 = ilbl1.size();
     if (w2<10){w2=10;}
     of << right << setw(5) << (sid1+1) << " " << setw(5) << (iid1+1)
        << setw(w1) << slbl1 << "  " << setw(w2) << ilbl1 << "   "
        << setw(14) << setprecision(6) << scientific << i1->get_gpbvmemb() << endl;

     iid1++;
    }
    sid1++;
   }

  if (! of.good())
    ::error("GcemMolecule::write_gint failed to write file, ",
            fn.c_str());
  of.close();

  blab2 << "GcemMolecule::write_gint: exiting\n";
}

void GcemMolecule::write_inter(const string &fn) {
  blab2 << "GcemMolecule::write_inter: starting\n";

  int sid1=0, sid2=0, iid1=0, iid2=0;
  double tmpd=0;

  ofstream of(fn.c_str());
  if (!of.good())
    ::error("GcemMolecule::write_inter failed to open file for writing, ",
            fn.c_str());

  for (list<GcemSite>::const_iterator s1=sites.begin(); s1!=sites.end(); ++s1){
   if (s1->get_ninst()==0) continue;
   int chain1((s1->get_sid()).get_chain());
   int site1((s1->get_sid()).get_site());
   string str_c1, str_s1;
   stringstream ss11,ss12;
   ss11 << chain1; ss11 >> str_c1;
   ss12 << site1;  ss12 >> str_s1;
   sid2=0;
   for (list<GcemSite>::const_iterator s2=sites.begin(); s2!=sites.end(); ++s2){
    if (s2->get_ninst()==0) continue;
    int chain2((s2->get_sid()).get_chain());
    int site2((s2->get_sid()).get_site());
    string str_c2, str_s2;
    stringstream ss21,ss22;
    ss21 << chain2; ss21 >> str_c2;
    ss22 << site2;  ss22 >> str_s2;
    iid1=0;
    for (list<GcemInstance>::const_iterator i1=s1->instances.begin(); i1!=s1->instances.end(); ++i1){
     string str_in1, slbl1, ilbl1;
     stringstream ss3;
     ss3 << iid1; ss3 >> str_in1;
     slbl1 = (s1->get_sid()).get_sitename() + "-" + str_c1 + "-" + str_s1;
     ilbl1 = (i1->get_instid()).get_instname() + "-" + str_in1;
     int w1 = slbl1.size();
     if (w1<13){w1=13;}
     int w2 = ilbl1.size();
     if (w2<10){w2=10;}
     iid2=0;
     for (list<GcemInstance>::const_iterator i2=s2->instances.begin(); i2!=s2->instances.end(); ++i2){
     string str_in2, slbl2, ilbl2;
     stringstream ss3;
     ss3 << iid2; ss3 >> str_in2;
     slbl2 = (s2->get_sid()).get_sitename() + "-" + str_c2 + "-" + str_s2;
     ilbl2 = (i2->get_instid()).get_instname() + "-" + str_in2;
     int w3 = slbl2.size();
     if (w3<13){w3=13;}
     int w4 = ilbl2.size();
     if (w4<10){w4=10;}
     double thisinter = get_pbg(i1->get_instid(), i2->get_instid())
                      + get_mmg(i1->get_instid(), i2->get_instid());
     of << right << setw(5) << (sid1+1) << " " << setw(5) << (iid1+1)
        << setw(w1) << slbl1 << "  " << setw(w2) << ilbl1 << "      "
        << right << setw(5) << (sid2+1) << " " << setw(5) << (iid2+1)
        << setw(w3) << slbl2 << "  " << setw(w4) << ilbl2 << "   "
        << right << setw(14) << setprecision(6) << scientific
        << thisinter << endl;
      iid2++;
     }
     iid1++;
    }
    sid2++;
   }
   sid1++;
  }

  if (!of.good())
    ::error("GcemMolecule::write_inter failed to write file, ",
            fn.c_str());
  of.close();


  blab1 << "GcemMolecule::write_inter: exiting\n";
}

list<string> GcemMolecule::get_ligands() const
{
  // collect ligand types
  set<string> all_ligs;
  for (list<GcemSite>::const_iterator i=sites.begin(); i!=sites.end(); ++i){
   for (list<GcemInstance>::const_iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
    for (map<string,int>::const_iterator l=j->ligandmap.begin(); l!=j->ligandmap.end(); ++l){
     if (l->second > 0) all_ligs.insert(l->first);
    }
   }
  }
  // force protons and electrons to precede other ligands
  list<string> ligands;
  set<string>::const_iterator liter = all_ligs.find((string) "proton");
  if (liter!=all_ligs.end()) ligands.push_back((string) "proton");
                                   liter = all_ligs.find((string) "electron");
  if (liter!=all_ligs.end()) ligands.push_back((string) "electron");
  for (set<string>::const_iterator l = all_ligs.begin();l!=all_ligs.end();l++){
   if ((*l)!=(string) "proton" && (*l)!=(string) "electron") ligands.push_back(*l);
  }
  return ligands;
}

/*
// this is for debugging purposes only !!!
// the structure coordinates are rounded to 3 decimal places
// to get exactly the same energies as from CHARMM when
// rereading the written structure files

void GcemMolecule::round_coordinates()
{

  cerr << endl
       << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl
       << "!  Entering GcemMolecule::round_coordinates            !" << endl
       << "!  WARNING: DO NOT USE THIS IN ACTUAL CALCULATIONS!!!  !" << endl
       << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl << endl;

  for (list<GcemSite>::iterator i=sites.begin(); i!=sites.end(); ++i){
   for (list<GcemInstance>::iterator j=i->instances.begin(); j!=i->instances.end(); ++j){
    for (AtomSet::iterator k=j->site_struct.begin(); k!=j->site_struct.end(); ++k){
     Coord c(k->second.coord);
     stringstream ss1, ss2, ss3;
     ss1 << setprecision(3) << fixed << k->second.coord.x; ss1 >> k->second.coord.x;
     ss2 << setprecision(3) << fixed << k->second.coord.y; ss2 >> k->second.coord.y;
     ss3 << setprecision(3) << fixed << k->second.coord.z; ss3 >> k->second.coord.z;
    }
    for (AtomSet::iterator k=j->model_struct.begin(); k!=j->model_struct.end(); ++k){
     Coord c(k->second.coord);
     stringstream ss1, ss2, ss3;
     ss1 << setprecision(3) << fixed << k->second.coord.x; ss1 >> k->second.coord.x;
     ss2 << setprecision(3) << fixed << k->second.coord.y; ss2 >> k->second.coord.y;
     ss3 << setprecision(3) << fixed << k->second.coord.z; ss3 >> k->second.coord.z;
    }
   }
  }

  cerr << "Exiting GcemMolecule::round_coordinates" << endl;

}
*/

// read/write GcemMolecule from/to a binary restart file
void GcemMolecule::write_restart_file(const string &fn) const {
  blab2 << "GcemMolecule::write_restart_file: starting\n";
  ofstream ofs(fn.c_str(), ios::binary|ios::trunc);
  if (!ofs.good())
    ::error("GcemMolecule::write_restart_file could not open file for writing, ", fn.c_str());
  boost::archive::binary_oarchive oa(ofs);
  oa << *this;
  ofs.close();
  if (!ofs.good())
    ::error("GcemMolecule::write_restart_file failed to write file, ", fn.c_str());
  blab1 << "GcemMolecule::write_restart_file: exiting\n";
}
void GcemMolecule::read_restart_file(const string &fn) {
  blab2 << "GcemMolecule::read_restart_file: starting\n";
  std:ifstream ifs(fn.c_str(), ios::binary);
  if (!ifs.good())
    ::error("GcemMolecule::read_restart_file could not open file for reading, ", fn.c_str());
  boost::archive::binary_iarchive ia(ifs);
  ia >> *this;
  ifs.close();
  if (!ifs.good())
    ::error("GcemMolecule::read_restart_file failed to read file, ", fn.c_str());
  blab1 << "GcemMolecule::read_restart_file: exiting\n";
}


// GcemMolecule.cc ends here
