/*  
    helper object for GcemMolecule
    creates the MEAD solver input for the calculation
    of electrostatic energies

    parsing functions for output files of
    my_2diel_solver
    my_Ndiel_solver, my_3diel_solver, my_memb_solver
    my_membpot_solver

    Copyright (c) 2007-2011 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.
*/
#ifndef _GcemSolverInput_h
#define _GcemSolverInput_h 1

#include <stdio.h>
#include <sstream>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <list>
#include <string>
#include "MEAD_e/CharmmID.h"
#include "MEAD_e/InstanceID.h"
#include "MEAD_e/GcemMolecule.h"
#include "MEAD_e/GcemSite.h"
#include "MEAD_e/globals.h"
#include "MEAD_e/CubeLatSpec.h"
#include "MEAD_e/FinDiffMethod.h"
#include <boost/regex.hpp>
#include <boost/filesystem.hpp>
#include <boost/algorithm/string.hpp>
// these are the irregularly shaped n-dimensional matrices
#include "./irreg_matrices/vector.hpp"    // 1D
#include "./irreg_matrices/matrix.hpp"    // 2D
#include "./irreg_matrices/tensor.hpp"    // 3D
#include "./irreg_matrices/d4_tensor.hpp" // 4D

using std::vector;
using std::list;
using std::map;
using std::pair;
using std::string;

class GcemMolecule;

//!wrap!
class GcemSolverInput {
public:
  GcemSolverInput(){}
  ~GcemSolverInput () {}

private:
  friend class GcemMolecule;

  // write structure files, job script and QMPB input script
  void write_gcem_input(const string&, const string&, const GcemMolecule &molecule) const;
  // read .ogm and .mgm if existent, otherwise create grids
  pair<FinDiffMethod,FinDiffMethod> get_fdm(const string&, const GcemMolecule &molecule) const;  
  // provides vector math functions, does not store data
  GcemGeometryCalculator geom_calc;
};



#endif

// GcemSolverInput.h ends here
