/*  class InstanceID is a key type for an associative array of
    of atom sets each representing an instance of a site

    TRU 2007

    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#include "MEAD_e/InstanceID.h"

InstanceID::InstanceID()
: sitename(""), instname(""), conformer(0), chain(0), site(0), instance(0)
{
}

InstanceID::InstanceID(const string& rn, const string& inn, const int cn,
                       const int chn, const int sn, const int f)
: conformer(cn), chain(chn), site(sn), instance(f), sitename(rn), instname(inn)
{
}

// at the moment the conformer is not needed here
string InstanceID::get_fn_prefix() const
{
 stringstream ss1;
 ss1 << sitename << "-" << chain << "-" << site << "_" << instname << "-" << instance;
 string result;
 ss1 >> result;
 return result;
}

ostream& InstanceID::print(ostream &ost) const
{
  ost << sitename << "-" << conformer << "-" << chain << "-" << site << "_" << instname << "-" << instance;
  return ost;
}


// InstanceID.cc ends here
