/*  container for sidechain rotamer
    torsion angles and occupation probability

    Copyright (C) 2007 Thomas Ullmann
    thomas.ullmann@uni-bayreuth.de

    Copyright (C) 1990--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#ifndef _RotData_h
#define _RotData_h 1

#include <string>
#include <boost/archive/basic_archive.hpp>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>

using std::string;
using std::ostream;
using std::istream;

//!wrap!
class RotData {
public:
  float p, x1, x2, x3, x4;
  RotData ();
  RotData (const RotData &);
  RotData (const float, const float, const float, const float, const float);
  ~RotData() {}
  //!nowrap!
  RotData& operator = (const RotData& a);
  bool operator== (const RotData& b) const;
  bool operator!= (const RotData& a) const;
  //!nowrap!+
  void print() const;
  void print(ostream& os) const;
  //!nowrap!-
private:
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version)
  {
    ar & p;
    ar & x1;
    ar & x2;
    ar & x3;
    ar & x4;
  }
};

// torsions are equal when x1, x2, x3, x4 are equal
inline bool RotData::operator== (const RotData& b) const
{
 if (x1 == b.x1 && x2 == b.x2 && x3 == b.x3 && x4 == b.x4){
  return true;
 } else {return false;}
}

inline bool RotData::operator!= (const RotData& a) const
{
  return !(*this == a);
}

ostream& operator <<(ostream& os, const RotData& a);

extern RotData nil_RotData;

inline bool operator<(const RotData& a, const RotData& b)
{
 if (a.x1 == b.x1 && a.x2 == b.x2 && a.x3 == b.x3 && a.x4 == b.x4){
  return false;
 } else if (a.x1 == b.x1 && a.x2 == b.x2 && a.x3 == b.x3){
  return (a.x4 < b.x4);
 } else if (a.x1 == b.x1 && a.x2 == b.x2){
  return (a.x3 < b.x3);
 } else if (a.x1 == b.x1){
  return (a.x2 < b.x2);
 } else {
  return (a.x1 < b.x1);
 }
}

inline bool operator>(const RotData& a, const RotData& b)
{
 if (a.x1 == b.x1 && a.x2 == b.x2 && a.x3 == b.x3 && a.x4 == b.x4){
  return false;
 } else if (a.x1 == b.x1 && a.x2 == b.x2 && a.x3 == b.x3){
  return (a.x4 > b.x4);
 } else if (a.x1 == b.x1 && a.x2 == b.x2){
  return (a.x3 > b.x3);
 } else if (a.x1 == b.x1){
  return (a.x2 > b.x2);
 } else {
  return (a.x1 > b.x1);
 }
}

#if SWIGPP_LITERAL_INCLUDE

%addmethods RotData {
   void write() { self->print(cout); }
   bool operator<(const RotData& a, const RotData& b);
   bool operator>(const RotData& a, const RotData& b);
};

#endif // SWIGPP_LITERAL_INCLUDE

#endif

// RotData.h ends here
