/*
   R. Thomas Ullmann 2009, based on Mirco Tills objects
   irregularly shaped 4 dimensional array
   boost serialization function
*/
#ifndef D4TENSOR_
#define D4TENSOR_
#include "tensor.hpp"
#include <iostream>
#include <iomanip>
#include <boost/archive/basic_archive.hpp> 
#include <boost/serialization/split_member.hpp>

template<class T>
    class d4_tensor{
        private:
            friend class boost::serialization::access;
            template<class Archive>
            void serialize(Archive & ar, const unsigned int file_version){
             boost::serialization::split_member(ar, *this, file_version);
            }
            template<class Archive>
            void save(Archive & ar, const unsigned int version) const
            {
             ar << dim1;
             ar << dim2;
             ar << dim3;
             ar << dim4;
             ar << start1;
             ar << start2;
             ar << start3;
             ar << start4;
             ar << is_irreg;
             if (is_irreg){
              for(int i = start1; i < start1 + dim1; i++)
               ar << irreg[i];
              ar << irreg_mat;
              ar << irreg_ten;
             }
             for(int i = start1; i <= get_end1(); i++){
              for(int j = start2; j <= get_end2(i); j++){
               for(int k = start3; k <= get_end3(i,j); k++){
                for(int l = start4; l <= get_end4(i,j,k); l++){
                 ar << arr[i][j][k][l];
                }
               }
              }
             }
            }
            template<class Archive>
            void load(Archive & ar, const unsigned int version)
            {
             kill_arr();
             ar >> dim1;
             ar >> dim2;
             ar >> dim3;
             ar >> dim4;
             ar >> start1;
             ar >> start2;
             ar >> start3;
             ar >> start4;
             ar >> is_irreg;
             end1 = start1 + dim1 - 1;
             if (is_irreg){
              end2 = 0;
              end3 = 0;
              end4 = 0;
              irreg  = new int[dim1];
              irreg -= start1;
              for(int i = start1; i <= end1; i++)
               ar >> irreg[i];
              ar >> irreg_mat;
              ar >> irreg_ten;
              space += sizeof(int) * dim1;
              space += irreg_mat.get_space();
              space += irreg_ten.get_space();
              allocate_arr(start1, start2, start3, start4, dim1, irreg, irreg_mat, irreg_ten);
              for(int i = start1; i < start1 + dim1; i++){
               for(int j = start2; j < irreg[i] + start2; j++){
                for(int k = start3; k < irreg_mat(i,j) + start3; k++){
                 for(int l = start4; l < irreg_ten(i,j,k) + start4; l++){
                  ar >> arr[i][j][k][l];
                 }
                }
               }
              }
             }
             else{      
              end2 = start2 + dim2 - 1;
              end3 = start3 + dim3 - 1;
              end4 = start4 + dim4 - 1;
              allocate_arr(start1, end1, start2, end2, start3, end3, start4, end4);
              for(int i = start1; i <= end1; i++){
               for(int j = start2; j <= end2; j++){
                for(int k = start3; k <= end3; k++){
                 for(int l = start4; l <= end4; l++){
                  ar >> arr[i][j][k][l];
                 }
                }
               }
              }
             }
            }
            T  result;
            T *iptr;  
            int dim1, dim2, dim3, dim4;
            int start1, start2, start3, start4;
            int end1, end2, end3, end4;
            float space;
            int *irreg;
            bool is_irreg;
            matrix<int> irreg_mat;
            tensor<int> irreg_ten;
        public:
            T ****arr;
            d4_tensor(): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(false), arr(NULL){
            };
            d4_tensor(const d4_tensor<T> &ini_t): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(false), arr(NULL){
                init_arr(ini_t);
            };
            d4_tensor(const int x, const int x2, const int y, const int y2, const int z, const int z2, const int k, const int k2): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(false), arr(NULL){
                init_arr(x,x2,y,y2,z,z2,k,k2);
            };
            d4_tensor(const int x, const int x2, const int y, const int y2, const int z, const int z2, const int k, const int k2, const T ini): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(false), arr(NULL){
                init_arr(x,x2,y,y2,z,z2,k,k2,ini);
            };
            d4_tensor(const int x, const int x2, int *sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(true), arr(NULL){
                init_arr(x,x2,sizes1,sizes2,sizes3);
            };
            d4_tensor(const int x, const int x2, int *sizes1, matrix<int> &sizes2, tensor<int> &sizes3, const T ini): dim1(0), dim2(0), dim3(0), dim4(0), start1(0), start2(0), start3(0), start4(0), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(true), arr(NULL){
                init_arr(x,x2,sizes1,sizes2,sizes3);
            };
            d4_tensor(const int x, const int x2, int *sizes1, const int z, const int z2, int *sizes2): dim1(x2-x+1), dim2(0), dim3(z2-z+1), dim4(0), start1(x), start2(1), start3(z), start4(1), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(true), arr(NULL){
                init_arr(x,x2,sizes1,z,z2,sizes2);
            };
            d4_tensor(const int x, const int x2, int *sizes1, const int z, const int z2, int *sizes2, const T ini): dim1(x2-x+1), dim2(0), dim3(z2-z+1), dim4(0), start1(x), start2(1), start3(z), start4(1), end1(0), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(true), arr(NULL){
                init_arr(x,x2,sizes1,z,z2,sizes2,ini);
            };
            d4_tensor(const int s1, const int s2, const int s3, const int s4, const int d1, int *sizes1, matrix<int> sizes2, const T ini): dim1(d1), dim2(0), dim3(0), dim4(0), start1(s1), start2(s2), start3(s3), start4(s4), end1(s1+d1-1), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(true), arr(NULL){
                init_arr(s1,s2,s3,dim1,vec<int>(sizes1, s1, d1),sizes2,ini);
            };
            d4_tensor(const int s1, const int s2, const int s3, const int s4, const int d1, int *sizes1, matrix<int> sizes2): dim1(d1), dim2(0), dim3(0), dim4(0), start1(s1), start2(s2), start3(s3), start4(s4), end1(s1+d1-1), end2(0), end3(0), end4(0), space(sizeof(d4_tensor)), irreg(NULL), is_irreg(true), arr(NULL){
                init_arr(s1,s2,s3,dim1,vec<int>(sizes1, s1, d1),sizes2);
            };
            void init_arr(const d4_tensor<T> &ini_t){
                if (ini_t.is_irreg){
                 init_arr(ini_t.start1,ini_t.end1,ini_t.irreg,ini_t.irreg_mat,ini_t.irreg_ten);
                } else {
                 init_arr(ini_t.start1, ini_t.end1, ini_t.start2, ini_t.end2, ini_t.start3, ini_t.end3, ini_t.start4, ini_t.end4);
                }
                for(int h1=start1; h1<=end1; h1++){
                    for(int h2=start2; h2<=get_end2(h1); h2++){
                        for(int h3=start3; h3<=get_end3(h1,h2); h3++){
                            for(int h4=start4; h4<=get_end4(h1,h2,h3); h4++){
                                arr[h1][h2][h3][h4] = ini_t(h1,h2,h3,h4);
                            };
                        };
                    };
                };
            }
            void init_arr(const int x, const int x2, const int y, const int y2, const int z, const int z2, const int k, const int k2){
                kill_arr();
                is_irreg = false;
                dim1 = x2 - x + 1;
                dim2 = y2 - y + 1;
                dim3 = z2 - z + 1;
                dim4 = k2 - k + 1;
                start1 = x;
                start2 = y;
                start3 = z;
                start4 = k;
                end1 = x2;
                end2 = y2;
                end3 = z2;
                end4 = k2;
                allocate_arr(start1, end1, start2, end2, start3, end3, start4, end4);
            }
            void init_arr(const int x, const int x2, const int y, const int y2, const int z, const int z2, const int k, const int k2, const T ini){                
                init_arr(x,x2,y,y2,z,z2,k,k2);
                for(int h1=x; h1<=x2; h1++){
                    for(int h2=y; h2<=y2; h2++){
                        for(int h3=z; h3<=z2; h3++){
                            for(int h4=k; h4<=k2; h4++){
                                arr[h1][h2][h3][h4] = ini;
                            };
                        };
                    };
                };
            };
            void init_arr(const int x, const int y, const int z, const int k, const int d1, vec<int> &sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3){
             kill_arr();
             is_irreg= true;
             dim1 = d1;
             dim2 = 0;
             dim3 = 0;
             dim4 = 0;
             start1 = x;
             start2 = y;
             start3 = z;
             start4 = k;
             end1 = x + d1 - 1;
             end2 = 0;
             end3 = 0;
             end4 = 0;
             allocate_arr(x,y,z,k,d1,sizes1,sizes2,sizes3);    
            }
            void init_arr(const int x, const int y, const int z, const int k, const int d1, vec<int> &sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3, const T ini){
                init_arr(x,y,z,k,d1,sizes1,sizes2,sizes3);
                for(int h1=x; h1<=end1; h1++){
                    for(int h2=y; h2<=get_end2(h1); h2++){
                        for(int h3=z; h3<=get_end3(h2); h3++){
                            for(int h4=k; h4<=get_end4(h3); h4++){
                                arr[h1][h2][h3][h4] = ini;
                            };
                        };
                    };
                };
            };
            void init_arr(const int x, const int x2, const vec<int> &sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3){
                kill_arr();
                is_irreg = true;
                dim1 = x2 - x +1;
                dim2 = 0;
                dim3 = 0;
                dim4 = 0;
                start1 = x;
                start2 = 1;
                start3 = 1;
                start4 = 1;
                end1 = x2;
                end2 = 0;
                end3 = 0;
                end4 = 0;
                irreg  = new int[x2-x+1];
                irreg -= start1;
                space += sizeof(int)*(x2-x+1);
                irreg_mat.init_arr(x,x2,sizes1);
                space += irreg_mat.get_space();
                irreg_ten.init_arr(x,x2,sizes1,sizes2);
                space += irreg_ten.get_space();
                for(int i=x; i<= x2; i++){
                    irreg[i] = sizes1(i);
                    for(int j=1; j<= sizes1(i); j++){
                        irreg_mat(i,j) = sizes2(i,j);
                        for(int k=1; k<= sizes2(i,j); k++){
                            irreg_ten(i,j,k) = sizes3(i,j,k);
                        }
                    }
                }
                allocate_arr(start1, start2, start3, start4, dim1, irreg, irreg_mat, irreg_ten);
            };
            void init_arr(const int x, const int x2, const vec<int> &sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3, const T ini){
                init_arr(x,x2,sizes1,sizes2,sizes3);
                for(int h1=x; h1<=x2; h1++){
                    for(int h2=1; h2<=sizes1(h1); h2++){
                        for(int h3=1; h3<=sizes2(h1,h2); h3++){
                            for(int h4=1; h4<=sizes3(h1,h2,h3); h4++){
                                arr[h1][h2][h3][h4] = ini;
                            };
                        };
                    };
                };
            };
            void init_arr(const int x, const int x2, int *sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3){
                vec<int> tmp_vec(x,x2,0);
                for(int i=x; i<=x2; i++){
                    tmp_vec(i) = sizes1[i];
                };
                init_arr(x,x2,tmp_vec,sizes2,sizes3);
            };
            void init_arr(const int x, const int x2, int *sizes1, const int z, const int z2, int *sizes2){
                kill_arr();
                is_irreg = true;
                dim1 = x2 - x + 1;
                dim2 = 0;
                dim3 = z2 - z + 1;
                dim4 = 0;
                start1 = x;
                start2 = start1;
                start3 = z;
                start4 = start3;
                end1 = x2;
                end2 = 0;
                end3 = z2;
                end4 = 0;
                irreg  = new int[dim1];
                irreg -= start1;
                space += sizeof(int)*(dim1);
                irreg_mat.init_arr(x,x2,sizes1,0);
                space += irreg_mat.get_space();
                irreg_ten.init_arr(x,x2,sizes1,z,z2,0);
                space += irreg_ten.get_space();
                for(int i=x; i<= x2; i++){
                    irreg[i] = sizes1[i];
                    for(int j=start2;j<start2+sizes1[i];j++){
                        irreg_mat(i,j) = dim3;
                        for(int k=z; k<=z2; k++){
                            irreg_ten(i,j,k) = sizes2[k];
                        }
                    }
                }
                allocate_arr(start1, start2, start3, start4, dim1, irreg, irreg_mat, irreg_ten);
            };
            void init_arr(const int x, const int x2, int *sizes1, const int z, const int z2, int *sizes2, const T ini){
                init_arr(x,x2,sizes1,z,z2,sizes2);
                for(int h1=get_start1(); h1<=get_end1(); h1++){
                    for(int h2=get_start2(); h2<=get_end2(h1); h2++){
                        for(int h3=get_start3(); h3<=get_end3(h1,h2); h3++){
                            for(int h4=get_start4(); h4<=get_end4(h1,h2,h3); h4++){
                                arr[h1][h2][h3][h4] = ini;
                            }
                        }
                    }
                }
            };
            void init_arr(const int x, const int x2, vec<int> sizes1, const int z, const int z2, vec<int> sizes2, const T ini){
                init_arr(x,x2,&sizes1(x),z,z2,&sizes2(x),ini);
            };
            void allocate_arr(const int x, const int x2, const int y, const int y2, const int z, const int z2, const int k, const int k2){
                const int d1 = x2 - x + 1;
                const int d2 = y2 - y + 1;
                const int d3 = z2 - z + 1;
                const int d4 = k2 - k + 1;
                try{
                    arr = new T***[d1];
                    space += d1 * sizeof(T***);
                    arr = arr - start1;
                    arr[x] = new T**[d1*d2+1];
                    space += (d1 * d2 +1) * sizeof(T**);
                    arr[x] -= start2;
                    arr[x][y] = new T*[d1*d2*d3+1];
                    space += (d1 * d2 *d3 +1) * sizeof(T*);
                    arr[x][y] -= start3;
                    arr[x][y][z] = new T[d1*d2*d3*d4+1];
                    space += (d1 * d2 *d3*d4 +1) * sizeof(T);
                    arr[x][y][z] -= start4;
                    // set second dimension
                    for(int i=y+1; i<=y2; i++){arr[x][i] = arr[x][i-1]+d3;}
                    // set third dimension
                    for(int i=x+1; i<=x2; i++){
                        arr[i] = arr[i-1] + d2;
                        arr[i][y] = arr[i-1][y] + d2*d3;
                        arr[i][y][z] = arr[i-1][y][z] + d2*d3*d4;
                        for(int j=y+1;j<=y2;j++){
                            arr[i][j] = arr[i][j-1]+d3;
                        }
                    };
                    for(int i=x; i<=x2; i++){
                        for(int j=y+1;j<=y2;j++){
                            arr[i][j][z] = arr[i][j-1][z] + d3*d4;
                        };
                    };
                    for(int i=x; i<=x2; i++){
                        for(int j=y;j<=y2;j++){
                            for(int h=z+1; h<=z2; h++){
                                arr[i][j][h] = arr[i][j][h-1] + d4;
                            };
                        };
                    };
                } catch (std::bad_alloc&){
                    cerr << "Insufficient memory for d4_tensor!" << endl;
                    cerr << "dimensions: " << d1 << " " << d2 << " " << d3 << " " << d4 << endl;
                    exit(1);
                };
            };
            void allocate_arr(const int s1, const int s2, const int s3, const int s4, const int d1, int *sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3){
                try {
                    arr = new T***[d1];
                    arr -= s1;
                    space += sizeof(T***)*d1;
                    for (int i=s1;i<s1+d1;++i){
                        arr[i] = new T**[sizes1[i]];
                        arr[i] -= s2;
                        space += sizeof(T**)*sizes1[i];
                    }
                    for (int i=s1;i<s1+d1;i++){
                        for (int j=s2;j<s2+sizes1[i];++j){
                            arr[i][j] = new T*[sizes2(i,j)];
                            arr[i][j] -= s3;
                            space += sizeof(T*)*sizes2(i,j);
                        }
                    }
                    for (int i=s1;i<s1+d1;i++){
                        for (int j=s2;j<s2+sizes1[i];++j){
                            for(int t=s3;t<s3+sizes2(i,j);t++){
                                arr[i][j][t] = new T[sizes3(i,j,t)];
                                arr[i][j][t] -= s4;
                                space += sizeof(T)*sizes3(i,j,t);
                            };
                        }
                    }
                }catch (std::bad_alloc&){
                    cerr << "Insufficent memory for d4_tensor!" << endl;
                    exit(1);
                };
            }
            void allocate_arr(const int s1, const int s2, const int s3, const int s4, const int d1, const vec<int> &sizes1, const matrix<int> &sizes2, const tensor<int> &sizes3){
                allocate_arr(s1,s2,s3,s4,d1,&sizes1(s1), sizes2, sizes3);
            };
            void kill_arr(){
             // deallocate data array
             if(is_irreg && arr != NULL){
                for (int i=start1;i<=end1;i++){
                        for (int j=start2;j<start2+irreg[i];j++){
                                for(int t=start3;t<start3+irreg_mat(i,j);t++){
                                        arr[i][j][t] += start4;
                                        delete [] arr[i][j][t];
                                        space -= sizeof(T)*irreg_ten(i,j,t);
                            }
                            arr[i][j] += start3;
                            delete [] arr[i][j];
                            space -= sizeof(T*)*irreg_mat(i,j);
                        }
                        arr[i] += start2;
                        delete [] arr[i];
                        space -= sizeof(T**)*irreg[i];
                };
                arr += start1;
                delete [] arr;
                space -= sizeof(T***)*dim1;
             } else if(arr != NULL){
                    arr[start1][start2][start3] += start4;
                    delete [] arr[start1][start2][start3];
                    arr[start1][start2] += start3;
                    delete [] arr[start1][start2];
                    arr[start1] += start2;
                    delete [] arr[start1];
                    arr += start1;
                    delete [] arr;
             };
             arr = NULL;
             // deallocate book-keeping arrays
             if (is_irreg){
              space -= irreg_ten.get_space();
              irreg_ten.kill_arr();
              space -= irreg_mat.get_space();
              irreg_mat.kill_arr();
              if (irreg != NULL){
               irreg += start1;
               delete [] irreg;
               irreg = NULL;
               space -= sizeof(int)*dim1;
              }
              is_irreg = false;
             }

            };
            int get_dim1() const {return dim1;}
            int get_dim2(const int x) const {if (irreg!=NULL) return irreg[x]; else return end2 - start2 + 1;}
            int get_dim3(const int x, const int y) const {if (irreg!=NULL) return irreg_mat(x,y); else return end3 - start3 + 1;}
            int get_dim4(const int x, const int y, const int z) const {if (irreg!=NULL) return irreg_ten(x,y,z); else return end4 - start4 + 1;}
            int get_start1() const {return start1;}
            int get_start2() const {return start2;}
            int get_start3() const {return start3;}
            int get_start4() const {return start4;}
            int get_end1() const {return end1;}
            int get_end2(const int x)                           const {if (is_irreg) return start2 + irreg[x] - 1;         else return end2;}
            int get_end3(const int x, const int y)              const {if (is_irreg) return start3 + irreg_mat(x,y) - 1;   else return end3;}
            int get_end4(const int x, const int y, const int z) const {if (is_irreg) return start4 + irreg_ten(x,y,z) - 1; else return end4;}
            double get_space() const {return space;};
            ~d4_tensor(){
                // deallocate arrays
                kill_arr();
                //if (space != sizeof(d4_tensor)){
                //    cerr << "Could not free all memory of d4_tensor. Remaining space = " << space << endl;
                //    exit(1);
                //}
            };
            inline T &operator()(const int x, const int y, const int z, const int k){
                return arr[x][y][z][k];
            };
            inline T *operator()(const int x, const int y, const int z){
                return arr[x][y][z];
            };
            inline T **operator()(const int x, const int y){
                return arr[x][y];
            };
            inline T ***operator()(const int x){
                return arr[x];
            };
            inline T &operator()(const int x, const int y, const int z, const int k) const {
                return arr[x][y][z][k];
            };
            inline T *operator()(const int x, const int y, const int z) const {
                return arr[x][y][z];
            };
            inline T **operator()(const int x, const int y) const {
                return arr[x][y];
            };
            inline T ***operator()(const int x) const {
                return arr[x];
            };
            friend ostream& operator<<(ostream &output, d4_tensor &ten){
                for(int i=ten.start1;i<=ten.end1;i++){
                   for(int j=ten.start2;j<=ten.get_end2(i);j++){
                      cout << "------------------------------" << endl << i << ", " << j << endl;
                      for(int k=ten.start3;k<=ten.get_end3(i,j);k++){
                         for(int l=ten.start4;l<=ten.get_end4(i,j,k);l++){
                          cout << setw(10) << fixed << setprecision(2) << ten(i,j,k,l) << " ";
                         } cout << endl;
                      } cout << endl;
                   } cout << endl;
                } cout << endl;
                return output;
            }
            d4_tensor<T>& operator=(const d4_tensor<T> &ref){
                kill_arr();
                init_arr(ref);
            };
    };

#endif
