/*
   R. Thomas Ullmann 2009, based on Mirco Tills objects
   irregularly shaped 2 dimensional array
   boost serialization function
*/
#ifndef MATRIX_
#define MATRIX_
#include "vector.hpp"
#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <boost/archive/basic_archive.hpp> 
#include <boost/serialization/split_member.hpp>

using namespace std;

/* Matrix template
  Template class for a one dimensional vector. Cares about all the memory allocation things and initialisatio if asked.
  Any given range is possible, no need to start at 0!
  Examples: Interger vector:
  matrix<int> example; followed by example.init_arr(1,100,1,100) gives you an interger matrix from 1 to 100 and 1 to 100 with 10000 elemets.
  matrix<int> example(1,100,1,100); Same matrix as above;
  matrix<int> example; followed by example.init_arr(1,100,1,100,0); Same as above, but all elements initialised to 0;
  matrix<int> example; followed by example.init_arr(1,100,vec<int>); Gives you a irreg matrix with vec(i) elements in each row.
  example(1,1) = 1; Assignes 1 to the first element in the first dimension.
*/

template<class T>
class matrix{
    private:
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, const unsigned int file_version){
         boost::serialization::split_member(ar, *this, file_version);
        }
        template<class Archive>
        void save(Archive & ar, const unsigned int version) const
        {
         ar << dim1;
         ar << dim2;
         ar << start1;
         ar << start2;
         ar << is_irreg;
         if (is_irreg){
          for(int i = start1; i < start1 + dim1; i++)
           ar << irreg[i];
         }
         for(int i = start1; i < start1 + dim1; i++){
          for(int j = start2; j <= get_end2(i); j++){
           ar << arr[i][j];
          }
         }
        }
        template<class Archive>
        void load(Archive & ar, const unsigned int version)
        {
         kill_arr();
         ar >> dim1;
         ar >> dim2;
         ar >> start1;
         ar >> start2;
         end1 = start1 + dim1 - 1;
         ar >> is_irreg;
         if (is_irreg){
          end2 = 0;
          irreg  = new int[dim1];
          irreg -= start1;
          for(int i = start1; i < start1 + dim1; i++)
           ar >> irreg[i];
          try{
              arr = new T*[dim1];
              arr -= start1;
              space += sizeof(T*)*dim1;
              for (int i=start1;i<=end1;++i){
                  arr[i] = new T[irreg[i]];
                  arr[i] -= start2;
                  space += sizeof(T)*irreg[i];
              }
          }catch (std::bad_alloc&){
              exit(1);
          }
          for(int i = start1; i < start1 + dim1; i++){
           for(int j = start2; j < irreg[i] + start2; j++){
            ar >> arr[i][j];
           }
          }
         }
         else{      
          end2 = start2 + dim2 - 1;
          try{
              arr = new T*[dim1];
              arr -= start1;
              space += dim1 * sizeof(T*);
	      arr[start1] = new T[dim1*dim2];
              arr[start1] -= start2;
              space += (dim1*dim2)*sizeof(T);
              for (int i=start1+1;i<=end1;++i){
                  arr[i] = arr[i-1]+dim2;
              };
          }catch (std::bad_alloc&){
              cout << "Unable to allocate array with dims " << dim1 << " " << dim2 << endl;
              exit(1);
          };
          for(int i = start1; i < start1 + dim1; i++){
           for(int j = start2; j < start2 + dim2; j++){
            ar >> arr[i][j];
           }
          }
         }
        }
        int dim1;   /* Number of rows */
        int dim2;   /* Number of columns */
        int start1; /* Starting index of the first dimension */
        int start2; /* Starting index of the second dimension */
        int end1;   /* Starting index of the first dimension */
        int end2;   /* Starting index of the second dimension */
        int *irreg; /*< 2nd dimension at index of first dimension */
        bool is_irreg;
        float space; /*< Memory usage */
    public:
        T **arr; /*< The array itself */
        /* Constructor without allocation and initialisation
         init_arr has to be called for allocating memory
        */
        matrix(): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){};
        /* Constructor with allocation but without initialisation
            \param d1 Starting index for rows
            \param d1_2 Last index for rows
            \param d2 Starting index for columns
            \param d2_2 Last index for columns
        */
        matrix(const int d1,const int d1_2,const int d2,const int d2_2): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
            init_arr(d1,d1_2,d2,d2_2);
        }
        /* Constructor with allocation and initialisation
            \param d1 Starting index for rows
            \param d1_2 Last index for rows
            \param d2 Starting index for columns
            \param d2_2 Last index for columns
            \param ini initialization value
        */
        matrix(const int d1,const int d1_2,const int d2,const int d2_2, const T ini): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
            init_arr(d1,d1_2,d2,d2_2,ini);
        };
        // Constructor with allocation and initialization
        matrix(const int d1,const int d1_2,int *sizes): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
            init_arr(d1,d1_2, sizes);
        };
        matrix(const int d1,const int d1_2,int *sizes, const T ini): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
            init_arr(d1,d1_2, sizes, ini);
        };
        matrix(const int d1,const int d1_2, vec<int> &sizes, const T ini): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
           init_arr(d1,d1_2, &sizes(d1), ini);
        };
        matrix(const int d1,const int d1_2, vec<int> &sizes): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
           init_arr(d1,d1_2, &sizes(d1));
        };
        // copy constructor
        matrix(const matrix<T> &ini_mat): dim1(0), dim2(0), start1(0), start2(0), end1(0), end2(0), irreg(NULL), is_irreg(false), space(sizeof(matrix)), arr(NULL){
            irreg = 0;
            arr = 0;
            init_arr(ini_mat);
        }
        // Initialise matrix from another matrix
       void init_arr(const matrix<T> &ini_mat){
            if (ini_mat.is_irreg)
             init_arr(ini_mat.get_start1(),ini_mat.get_end1(),ini_mat.irreg);
            else
             init_arr(ini_mat.get_start1(),ini_mat.get_end1(),ini_mat.get_start2(),ini_mat.get_end2());
            for(int i=ini_mat.get_start1(); i<= ini_mat.get_end1(); i++){
                for(int j=ini_mat.get_start2(); j<=ini_mat.get_end2(i); j++){
                    arr[i][j] = ini_mat(i,j);
                };
            };
        };
       /* Initialise a regular matrix
            Set values for matrix. Try to allocate memory: dim1 pointers to T in the first dimension.
            For fast memory allocation the elemetns for all columns are allocated at once. Reduces new calls.
            \param x Starting index for rows
            \param x2 Last index for rows
            \param y Starting index for columns
            \param y2 Last index for columns
            \param ini initialization value
      */
       void init_arr(const int x, int x2, const int y, const int y2, const T ini){
           init_arr(x,x2,y,y2);
           for(int i=x; i<= x2; i++){
               for(int j=y; j<= y2; j++){
                   arr[i][j] = ini;
               }
           }
       };
       void init_arr(const int x, int x2, const int y, const int y2){
            kill_arr(); // prevent memory leaks
            is_irreg = false;
            dim1 = x2 - x + 1;
            dim2 = y2 - y + 1;
            start1 = x;
            start2 = y;
            end1 = x2;
            end2 = y2;
            if(dim1 <=0){dim1 = 1; x2 = x;}
            try{
                arr = new T*[dim1];
                arr -= start1;
                space += dim1 * sizeof(T*);
		arr[start1] = new T[dim1*dim2];
                arr[start1] -= y;
                space += (dim1*dim2)*sizeof(T);
                for (int i=start1+1;i<=x2;++i){
                    arr[i] = arr[i-1]+dim2;
                };
            }catch (std::bad_alloc&){
                cout << "Unable to allocate array with dims " << dim1 << " " << dim2 << endl;
                exit(1);
            };
        };
        /* Initialise a irreg matrix and values in it.
            Set values for matrix. Try to allocate memory: dim1 pointers to T in the first dimension.
            Second dimension is allocated row by row.
            All values are initialised to ini
            \param x Starting index for rows
            \param x2 Last index for rows
            \param &sizes vector storing the column dimensions
            \param ini initialization value in second variant
        */
        void init_arr(const int x, const int x2, const vec<int> &sizes, const T ini){
            init_arr(x,x2,sizes);
            for(int i=get_start1(); i<=get_end1();i++){
                for(int j=get_start2(); j<=get_end2(i);j++){
                    arr[i][j] = ini;
                };
            };
        }
        void init_arr(const int x, int x2, const vec<int> &sizes){
            kill_arr(); // prevent memory leaks
            is_irreg = true;
            dim1 = x2 - x + 1;
            dim2 = 0;
            start1 = x;
            start2 = start1;
            end1 = x2;
            end2 = 0;
            irreg  = new int[dim1];
            irreg -= start1;
            space += sizeof(int)*(dim1);
            for(int i=x; i<= x2; i++){
                irreg[i] = sizes(i);
            };
            try{
                arr = new T*[dim1];
                arr -= start1;
                space += sizeof(T*)*dim1;
                for (int i=start1;i<=end1;++i){
                    arr[i] = new T[sizes(i)];
                    arr[i] -= start2;
                    space += sizeof(T)*(sizes(i));
                };
            }catch (std::bad_alloc&){
                exit(1);
            };

        };
        // with integer vec instead simple array
        void init_arr(const int x, const int x2, int *sizes){
            vec<int> tmp_vec(x,x2);
            for(int i=x; i<=x2; i++){
                tmp_vec(i) = sizes[i];
            };
            init_arr(x,x2,tmp_vec);
        };
        void init_arr(const int x, const int x2, int *sizes, const T ini){
            vec<int> tmp_vec(x,x2);
            for(int i=x; i<=x2; i++){
                tmp_vec(i) = sizes[i];
            };
            init_arr(x,x2,tmp_vec, ini);
        };
        // Adding a row to the matrix
        void add_row(){
            if(arr && !is_irreg){
                T **tmp_arr;
                try{
                    tmp_arr = new T*[dim1+1];
                    tmp_arr = tmp_arr - start1;
                    tmp_arr[start1] = new T[((dim1+1)*dim2)+1];
                    for (int i=start1+1;i<=start1 + dim1;++i){
                        tmp_arr[i] = tmp_arr[i-1]+dim2;
                    };
                 }catch (std::bad_alloc&){
                    exit(1);
                };
                for(int i=start1; i<=start1 + dim1 - 1; i++){
                    for(int j=start2; j<=start2 + dim2 - 1; j++){
                        tmp_arr[i][j] = arr[i][j];
                    };
                };
                kill_arr();
                arr = tmp_arr;
                for(int i=start1; i<=dim1 + 1; i++){
                    arr[i] = tmp_arr[i];
                };
                tmp_arr = 0;
                dim1 = dim1 + 1;
            }
        };
        // number of rows
        int get_dim1() const {return dim1;}
        // number of columns
        int get_dim2() const {
            if(!is_irreg) return dim2;
            else return 0;
        }
        int get_dim2(const int x) const {
            if(!is_irreg) return dim2;
            else return irreg[x];            
        }
        // starting index of dimension 1
        int get_start1() const {return start1;};
        // starting index of dimension 2
        int get_start2() const {return start2;};
        // last index of dimension 1
        int get_end1() const {return end1;};
        // last index of dimension 2
        int get_end2() const {if (!is_irreg) return end2; else return 0;};
        int get_end2(const int x) const {if (!is_irreg) return end2; else return irreg[x] + start2 - 1;};
        // memory usage
        double get_space() const {return space;}
        // sum all elements
	double get_sum(const int row) const{
         T sum = 0;
         for(int i=start2; i<= get_end2(row); i++) sum += arr[row][i];
	 return sum;
        }
        // deallocate memory
        void kill_arr(){
            if(arr != NULL){
                if(is_irreg){
                    for (int i=start1;i<=end1;++i){
                        arr[i] += start2;
                        delete [] arr[i];
                        space -= get_dim2(i)*sizeof(T);
                    };
                    arr += start1;
                    delete [] arr;
                    space -= dim1*sizeof(T*);
                    arr = NULL;
                }else{
                    arr[start1] += start2;
                    delete [] arr[start1];
                    space -= dim1*dim2*sizeof(T);
                    arr += start1;
                    delete [] arr;
                    space -= dim1*sizeof(T*);
                    arr = NULL;
                }
            }
            // deallocate book-keeping array
            if (is_irreg){
               if (irreg != NULL){
                irreg += start1;
                delete [] irreg;
                space -= dim1*sizeof(int);
                irreg = NULL;
               }
               is_irreg = false;
            }
        };
        // destructor
        ~matrix(){
            // deallocate arrays
            kill_arr();
            //if (space != sizeof(matrix)){
            //    cerr << "Could not free all memory of matrix. Remaining space = " << space << endl;
            //    exit(1);
            //}
            start1 = start2 = end1 = end2 = dim1 = dim2 = 0;
        };
        // Index operators
        T &operator()(const int x, const int y) const {
                return arr[x][y];
        };
        T &operator()(const int x, const int y){
                return arr[x][y];
        };
        T *operator()(const int x){
            return arr[x];

        };
        // output operator
        friend ostream& operator<<(ostream &output, matrix &mat){
            if(!mat.is_irreg){
                for(int i=mat.start1;i<=mat.start1+mat.dim1-1;i++){
                    for(int j=mat.start2;j<=mat.start2+mat.dim2-1;j++){
                        cout << setw(10) << fixed << setprecision(2) << mat(i,j) << " ";
                    }
                    cout << endl;
                }
            } else if(mat.dim2 == 0){
                for(int i=mat.start1;i<=mat.start1 + mat.dim1 - 1;i++){
                    for(int j=mat.start2;j<=mat.get_end2(i);j++){
                        cout << fixed << setprecision(2) << mat(i,j) << " ";
                    }
                    cout << endl;
                }
            }
            return output;
        }
        matrix<T>& operator=(const matrix<T> &ref){
            kill_arr();
            init_arr(ref);
        };
};

#endif
