/*
   R. Thomas Ullmann 2009, based on Mirco Tills objects
   irregularly shaped 3 dimensional array
   boost serialization function
*/
#ifndef TENSOR_
#define TENSOR_
#include "matrix.hpp"
#include <iostream>
#include <iomanip>
#include <boost/archive/basic_archive.hpp> 
#include <boost/serialization/split_member.hpp>

template<class T>
class tensor{
    private:
        friend class boost::serialization::access;
        template<class Archive>
        void serialize(Archive & ar, const unsigned int file_version){
         boost::serialization::split_member(ar, *this, file_version);
        }
        template<class Archive>
        void save(Archive & ar, const unsigned int version) const
        {
         ar << dim1;
         ar << dim2;
         ar << dim3;
         ar << start1;
         ar << start2;
         ar << start3;
         ar << is_irreg;
         if (is_irreg){
          for(int i = start1; i < start1 + dim1; i++)
           ar << irreg[i];
          ar << irreg_mat;
         }
         for(int i = start1; i <= get_end1(); i++){
          for(int j = start2; j <= get_end2(i); j++){
           for(int k = start3; k <= get_end3(i,j); k++){
            ar << arr[i][j][k];
           }
          }
         }
        }
        template<class Archive>
        void load(Archive & ar, const unsigned int version)
        {
         kill_arr();
         ar >> dim1;
         ar >> dim2;
         ar >> dim3;
         ar >> start1;
         ar >> start2;
         ar >> start3;
         ar >> is_irreg;
         end1 = start1 + dim1 - 1;
         if (is_irreg){
          end2 = 0;
          end3 = 0;
          irreg  = new int[dim1];
          irreg -= start1;
          for(int i = start1; i <= end1; i++)
           ar >> irreg[i];
          ar >> irreg_mat;
          space += sizeof(int) * dim1;
          space += irreg_mat.get_space();
          allocate_arr(start1, start2, start3, dim1, irreg, irreg_mat);
          for(int i = start1; i < start1 + dim1; i++){
           for(int j = start2; j < irreg[i] + start2; j++){
            for(int k = start3; k < irreg_mat(i,j) + start3; k++){
             ar >> arr[i][j][k];
            }
           }
          }
         }
         else{      
          end2 = start2 + dim2 - 1;
          end3 = start3 + dim3 - 1;
          allocate_arr(start1, end1, start2, end2, start3, end3);
          for(int i = start1; i < end1; i++){
           for(int j = start2; j < end2; j++){
            for(int k = start3; k < end3; k++){
             ar >> arr[i][j][k];
            }
           }
          }
         }
        }
        int dim1, dim2, dim3;
        int start1, start2, start3;
        int end1, end2, end3;
        float space;
        int *irreg;
        bool is_irreg;
        matrix<int> irreg_mat;
    public:
        T ***arr;
        tensor(): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
        };
        tensor(const tensor<T> &ini_t): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), space(sizeof(tensor)), irreg(NULL), arr(NULL), is_irreg(false){
           init_arr(ini_t);
        };
        tensor(const int x, const int x2, const int y, const int y2, const int z, const int z2): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
            init_arr(x,x2,y,y2,z,z2);
        };
        tensor(const int x, const int x2, const int y, const int y2, const int z, const int z2, const T ini): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
            init_arr(x,x2,y,y2,z,z2,ini);
        };
        tensor(const int x, const int x2, int *sizes, const int z, const int z2): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
            init_arr(x,x2,sizes,z,z2);
        };
        tensor(const int x, const int x2, int *sizes, const int z, const int z2, const T ini): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
            init_arr(x,x2,sizes,z,z2,ini);
        };
        tensor(const int x, const int x2, int *sizes1, const matrix<int> &sizes2): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
             init_arr(x,x2,sizes1,sizes2);
        };
        tensor(const int x, const int x2, int *sizes1, const matrix<int> &sizes2, const T ini): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
             init_arr(x,x2,sizes1,sizes2,ini);
        };
        tensor(const int x, const int x2, const int y, const int y2, int *sizes): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
             init_arr(x,x2,y,y2,sizes);
        };
        tensor(const int x, int x2, const int y, const int y2, int *sizes, const T ini): dim1(0), dim2(0), dim3(0), start1(0), start2(0), start3(0), end1(0), end2(0), end3(0), space(sizeof(tensor)), irreg(NULL), is_irreg(false), arr(NULL){
             init_arr(x,x2,y,y2,sizes,ini);
        };
        void init_arr(const tensor<T> &ini_t){
           if (ini_t.is_irreg){
            //vec<int> tmpvec(ini_t.start1, ini_t.end1);
            //for(int i=ini_t.start1; i<=ini_t.end1;i++) tmpvec(i) = ini_t.irreg[i];
            init_arr(ini_t.start1, ini_t.end1, ini_t.irreg, ini_t.irreg_mat);
           } else {
            init_arr(ini_t.start1, ini_t.end1, ini_t.start2, ini_t.end2, ini_t.start3, ini_t.end3);
           }
           for(int i=start1; i<=end1; i++){
               for(int j=start2; j<=get_end2(i); j++){
                   for(int k=start3; k<=get_end3(i,j); k++){
                       arr[i][j][k] = ini_t(i,j,k);
                   };
               };
           };
        }
        void init_arr(const int x, const int x2, const int y, const int y2, const int z, const int z2){
            is_irreg = false;
            dim1 = x2 - x + 1;
            dim2 = y2 - y + 1;
            dim3 = z2 - z + 1;
            start1 = x;
            start2 = y;
            start3 = z;
            end1 = x2;
            end2 = y2;
            end3 = z2;
            allocate_arr(x, x2, y, y2, z, z2);
        }
        void init_arr(const int x, const int x2, const int y, const int y2, const int z, const int z2, const T ini){
            is_irreg = false;
            init_arr(x,x2,y,y2,z,z2);
            for(int i=x; i<=x2; i++){
                for(int j=y; j<=y2;j++){
                    for(int k=z; k<=z2; k++){
                        arr[i][j][k] = ini;
                    };
                };
            };
        }
        void init_arr(const int x, const int x2, int *sizes1, const matrix<int> &sizes2){
            is_irreg = true;
            vec<int> tmp_vec(x,x2);
            for(int i=x; i<=x2; i++) tmp_vec(i) = sizes1[i];
            init_arr(x,x2,tmp_vec,sizes2);
        };
        void init_arr(const int x, const int x2, const vec<int> &sizes1, const matrix<int> &sizes2){
            is_irreg = true;
            dim1 = x2 - x + 1;
            dim2 = 0;
            dim3 = 0;
            start1 = x;
            start2 = 1;
            start3 = 1;
            end1   = x2;
            end2   = 0;
            end3   = 0;
            irreg  = new int[dim1];
            irreg -= start1;
            space += sizeof(int)*(x2-x+1);
            irreg_mat.init_arr(x,x2,sizes1,0);
            space += irreg_mat.get_space();
            for(int i=x; i<=x2;i++){
                irreg[i] = sizes1(i);
                for(int j=1; j<=sizes1(i);j++){
                    irreg_mat(i,j) = sizes2(i,j);
                }
            }
            allocate_arr(start1, start2, start3, dim1, irreg, irreg_mat);           
        };
        void init_arr(const int x, const int x2, vec<int> &sizes1, matrix<int> &sizes2, const T ini){
           init_arr(x,x2,sizes1,sizes2);
           for(int i=x; i<=x2;i++){
               for(int j=1; j<=sizes1(i);j++){
                   for(int k=1; k<=sizes2(i,j);k++){
                       arr[i][j][k] = ini;
                   };
               };
           };

        }
        void init_arr(const int x, const int x2, const int y, const int y2, int *sizes){
            vec<int> tmpvec(y,y2,0);
            for (int i=y;i<=y2;i++)
                tmpvec(i) = sizes[i];
            init_arr(x, x2, y, y2, tmpvec);
        }
        void init_arr(const int x, const int x2, const int y, const int y2, int *sizes, const T ini){
            vec<int> tmpvec(y,y2,0);
            for (int i=y;i<=y2;i++)
                tmpvec(i) = sizes[i];
            init_arr(x,x2,y,y2,tmpvec,ini);
        }
        void init_arr(const int x, const int x2, int *sizes, const int z, const int z2){
            vec<int> tmpvec(x,x2,0);
            for (int i=x;i<=x2;i++)
                tmpvec(i) = sizes[i];
            init_arr(x, x2, tmpvec, z, z2);
        }
        void init_arr(const int x, const int x2, int *sizes, const int z, const int z2, const T ini){
            vec<int> tmpvec(x,x2,0);
            for (int i=x;i<=x2;i++)
                tmpvec(i) = sizes[i];
            init_arr(x,x2,tmpvec,z,z2);
            for (int i = get_start1();i<=get_end1();i++){
                for (int j = get_start2();j<=get_end2(i);j++){
                    for (int k = get_start3();k<=get_end3(i,j);k++){
                        arr[i][j][k] = ini;
                    }
                }
            }
        };
        void init_arr(const int x, const int x2, vec<int> &sizes, const int z, const int z2, T ini){
            init_arr(x, x2, sizes, z, z2);
            for (int i = get_start1();i<=get_end1();i++){
                for (int j = get_start2();j<=get_end2(i);j++){
                    for (int k = get_start3();k<=get_end3(i,j);k++){
                        arr[i][j][k] = ini;
                    }
                }
            }
        }
        void init_arr(const int x, const int x2, const int y, const int y2, vec<int> &sizes, T ini){
            init_arr(x,x2,y,y2,sizes);
            for (int i = get_start1();i<=get_end1();i++){
                for (int j = get_start2();j<=get_end2(i);j++){
                    for (int k = get_start3();k<=get_end3(i,j);k++){
                        arr[i][j][k] = ini;
                    }
                }
            }
        };
        void init_arr(const int x, const int x2, const int y, const int y2, vec<int> &sizes){
            is_irreg = true;
            dim1 = x2 - x + 1;
            dim2 = y2 - y + 1;
            dim3 = 0;
            start1 = x;
            start2 = y;
            start3 = 1;
            end1   = x2;
            end2   = y2;
            end3   = 0;
            irreg = new int[dim1];
            irreg -= start1;
            space += sizeof(int)*(dim1);
            irreg_mat.init_arr(x,x2,y,y2,0);
            space += irreg_mat.get_space();
            for(int i=x; i<= x2; i++){
                irreg[i] = dim2;
                for(int j=y; j<=y2; j++){
                    irreg_mat(i,j) = sizes(j);
                }
            }
            allocate_arr(start1, start2, start3, dim1, irreg, irreg_mat);          
        };
        void init_arr(int x, int x2, vec<int> &sizes, int z, int z2){
            is_irreg = true;
            dim1 = x2 - x + 1;
            dim2 = 0;
            dim3 = z2 - z + 1;
            start1 = x;
            start2 = start1;
            start3 = z;
            end1   = x2;
            end2   = 0;
            end3   = z2;
            irreg  = new int[dim1];
            irreg -= start1;
            space += sizeof(int)*(dim1);
            irreg_mat.init_arr(x,x2,sizes,dim3);
            space += irreg_mat.get_space();
            for(int i=x; i<=x2; i++){
                irreg[i] = sizes(i);
                for(int j=start2; j<start2+sizes(i); j++){
                    irreg_mat(i,j) = dim3;
                }
            }
            allocate_arr(start1, start2, start3, dim1, irreg, irreg_mat);
        };
        int get_start1() const {return start1;};
        int get_start2() const {return start2;};
        int get_start3() const {return start3;};
        int get_end1() const {return end1;};
        int get_end2(const int x) const {if (!is_irreg) return end2; else return start2 + irreg[x] - 1;};
        int get_end3(const int x, const int y) const {if (!is_irreg) return end3; else return start3 + irreg_mat(x,y) - 1;};
        int get_dim1() const {return dim1;};
        int get_dim2() const {return dim2;};
        int get_dim3() const {return dim3;};
        int get_dim2(const int x) const {if (!is_irreg) return dim2; else return irreg[x];};
        int get_dim3(const int x, const int y) const {if (!is_irreg) return dim2; else return irreg_mat(x,y);};
        double get_space() const {return space;};
        int is_inside (const int i, const int i2, const int s, const int up_to = -1){
          if(up_to == -1 || up_to > dim3 + start3 - 1){
            up_to = dim3 + start3 - 1;
          };
          if(dim3 > 0){
            for(int j=1; j<=up_to; j++){
              if(arr[i][i2][j] == s){return 1;};
            };
          };
          return 0;
        };
        void allocate_arr(const int x, const int x2, const int y, const int y2, const int z, const int z2){
            dim1 = x2 - x + 1;
            dim2 = y2 - y + 1;
            dim3 = z2 - z + 1;
            try{
                arr = new T**[dim1];
                space += dim1 * sizeof(T**);
                arr -= start1;
                arr[x] = new T*[dim1*dim2+1];
                space += (dim1 * dim2) * sizeof(T*);
                arr[x] -= start2;
                arr[x][y] = new T[dim1*dim2*dim3+1];
                space += (dim1 * dim2 *dim3) * sizeof(T);
                arr[x][y] -= start3;
                // Since we dont run over all elements in the first dimension...
                for(int i=y+1; i<=y2; i++){arr[x][i] = arr[x][i-1]+dim3;}
                // Set the rest
                for(int i=x+1; i<=x2; i++){
                    arr[i] = arr[i-1] + dim2;
                    arr[i][y] = arr[i-1][y] + dim2*dim3;
                    for(int j=y+1;j<=y2;j++){
                        arr[i][j] = arr[i][j-1]+dim3;
                    };
                };
            } catch (std::bad_alloc&){
                cerr << "Insufficent memory for tensor!" << endl;
                cerr << "dimensions: " << dim1 << " " << dim2 << " " << dim3 << endl;
                exit(1);
            };

        };
        void allocate_arr(const int s1, const int s2, const int s3, const int d1, int *irr, const matrix<int> &irr_mat){
          const int e1 = s1 + d1 - 1;
          try{              
              arr = new T**[d1];
              arr = arr - s1;
              space += sizeof(T**)*d1;
              for (int i=s1;i<=e1;++i){
                  arr[i] = new T*[irr[i]];
                  arr[i] -= s2;
                  space += sizeof(T*)*irr[i];
              }
              for (int i=s1;i<=e1;i++){
                  for (int j=s2;j<irr[i]+s2;++j){
                      arr[i][j] = new T[irr_mat(i,j)];
                      arr[i][j] -= s3;
                      space += sizeof(T)*irr_mat(i,j);
                  }
              }
          } catch (std::bad_alloc&){
              cerr << "Not enough memory for tensor!" << endl;
              exit(1);
          }
        }
        void kill_arr(){
            if (arr) {
                if(irreg == NULL){
                    if(arr != NULL){
                     arr[start1][start2] += start3;
                     delete [] arr[start1][start2];
                     arr[start1] += start2;
                     delete [] arr[start1];
                     arr += start1;
                     delete [] arr;
                     space -= dim1 * sizeof(T**);
                     space -= (dim1 * dim2) * sizeof(T*);
                     space -= (dim1 * dim2 * dim3) * sizeof(T);
                    };
                }
                else{
                    for (int i=get_start1();i<=get_end1();i++){
                            for (int j=get_start2();j<=get_end2(i);++j){
                                arr[i][j] += start3;
                                delete [] arr[i][j];
                                space -= irreg_mat(i,j)*sizeof(T**);
                            }
                            arr[i] += start2;
                            delete [] arr[i];
                            space -= irreg[i]*sizeof(T*);
                        }
                        arr += start1;
                        delete [] arr;
                        space -= dim1*sizeof(T);
                    }
                    arr = NULL;
            }
            // deallocate book-keeping arrays
            if (is_irreg){
             space -= irreg_mat.get_space();
             irreg_mat.kill_arr();
             if (irreg != 0){
              irreg += start1;
              delete [] irreg;
              irreg = NULL;
              space -= dim1*sizeof(int);
             }
             is_irreg = false;
            }
        };
        ~tensor(){
            // deallocate arrays
            kill_arr();
            // FIXME !!! the size calculation is screwed up somethere, error occurs without memory leak
            //if (space != sizeof(tensor)){
            //    cerr << "Could not free all memory of tensor. Remaining space = " << space << endl;
            //    exit(1);
            //}
            //dim1 = dim2 = dim3 = start1 = start2 = start3 = end1 = end2 = end3 = 0;
        };
        T &operator()(const int x, const int y, const int z){
            return arr[x][y][z];
        };
        T &operator()(const int x, const int y, const int z) const {
            return arr[x][y][z];
        };
        T *operator()(const int x, const int y){
            return arr[x][y];
        }
        T **operator()(const int x){
            return arr[x];
        }
        void assign_vector(const int x, const int y, const T *data){
            for (int z = get_start3(); z<=get_end3(x,y); z++){
                 arr[x][y][z] = data[z];
            }
        }
        friend ostream& operator<<(ostream &output, tensor &ten){
            for(int i=ten.start1;i<=ten.end1;i++){
               cout << "------------------------------" << endl << i << endl;
               for(int j=ten.start2;j<=ten.get_end2(i);j++){
                  for(int k=ten.start3;k<=ten.get_end3(i,j);k++){
                     cout << setw(10) << fixed << setprecision(2) << ten(i,j,k) << " ";
                  } cout << endl;
               } cout << endl;
            }
            return output;
        }
        tensor<T>& operator=(const tensor<T> &ref){
            kill_arr();
            init_arr(ref);
        };
};

#endif
