/*  class muvec is a key type for an associative array
    keyed by a combination of chemical potentials

    TRU 2008
*/
#include "MEAD_e/muvec.h"

muvec::muvec()
: mv()
{
}

muvec::muvec(const vector<double>& v)
: mv(v)
{
}

muvec::muvec(const muvec& v)
: mv(v.mv)
{
}

std::string muvec::get_fn_prefix() const
{
 vector<double> conv;
 vector<int> prec;
 conv.push_back((double) PhysCond::get_pkconv()/PhysCond::get_econv());
 conv.push_back(4184000/(PhysCond::get_Avogadro_Number()*PhysCond::get_elementary_charge()));
 prec.push_back((int) 1);
 prec.push_back((int) 0);
 return get_fn_prefix(conv,prec);
}

string muvec::get_fn_prefix(const vector<double>& c, const vector<int>& p) const
{
 vector<double> conv(c);
 vector<int> prec(p);
 if (conv.size()<mv.size()){
  int nfill = mv.size() - conv.size();
  for (int i=0; i<nfill;i++){
   conv.push_back((double) 1.0);
  }
 }
 if (prec.size()<mv.size()){
  int nfill = mv.size() - prec.size();
  for (int i=0; i<nfill;i++){
   prec.push_back((int) 2);
  }
 }
 stringstream ss;
 string tmpstr,fn;
 for (int i=0;i<mv.size();i++){
  double ten_p_prec = pow((double)10,prec[i]);
  double val = round(ten_p_prec * mv[i] * conv[i]) / ten_p_prec;
  if (val==0) val *= -1;
  if    (prec[i] != 0) ss << setfill('0') << setw(5+prec[i]) << setprecision(prec[i]) << fixed <<   showpoint << val;
  else                 ss << setfill('0') << setw(5)         << setprecision(prec[i]) << fixed << noshowpoint << val;
  ss >> tmpstr;
  if (i!=0) fn += "_";
  fn += tmpstr;
 }
 return fn;
}

ostream& muvec::print(ostream &ost)
{
  if (mv.size() > 0){
   ost << setw(17) << scientific << setprecision(12) << mv[0];
   for (std::vector<double>::const_iterator i = ++mv.begin(); i != mv.end(); i++)
   ost << setw(17) << scientific << setprecision(12) << *i;
  }
  return ost;
}


// muvec.cc ends here
