/*  class muvec is a key type for an associative array of
    of atom sets each representing an instance of a site

    TRU 2007

    Copyright (c) 1993--1995 by Donald Bashford.

    This source code file is part of the MEAD (Macroscopic
    Electrostatics with Atomic Detail) package of objects and
    programs.  MEAD is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 1, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

    Donald Bashford can be contacted by electronic mail by the address,
    bashford@scripps.edu, or by paper mail at Department of Molecular
    Biology, The Scripps Research Institute, 10666 North Torrey Pines
    Road, La Jolla, California 92037.

*/
#ifndef _muvec_h
#define _muvec_h 1

#include "MEAD_e/PhysCond.h"
#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <ostream>
#include <vector>
#include <map>
#include <list>
#include <string>
#include <cmath>

using std::vector;
using std::list;
using std::map;
using std::pair;
using std::string;
using std::stringstream;
using std::ostream;
using namespace::std;

using namespace std;

class muvec {
public:
  muvec ();
  muvec (const muvec&);
  muvec (const std::vector<double>&);
  ~muvec () {}
  muvec& operator= (const muvec& v);
  bool operator== (const muvec&) const;
  bool operator!= (const muvec& a) const;
  bool operator<(const muvec&) const;
  bool operator>(const muvec&) const;
  //!nowrap!+
  ostream& print(ostream&);
  double get_mu(const int i) const
  {
    if (i < mv.size())
    {
      return mv[i];
    }
    else
    {
      error("muvec: ERROR, element not defined.");
      return 0.0; // quite clang
    }
  }
  int get_nmu() const {return mv.size();}
  std::string get_fn_prefix() const;
  std::string get_fn_prefix(const std::vector<double>&, const std::vector<int>&) const;
private:
  vector<double> mv;
};

inline muvec& muvec::operator= (const muvec& a)
{mv = a.mv; return *this;}

inline bool muvec::operator== (const muvec& a) const
{
 if (mv.size() != a.mv.size())
  return false;
 bool result = true;
 for (int i=0;i<mv.size();i++){
  if (mv[i] != a.mv[i]){
   result = false;
   break;
  }
 }
 return result;
}

inline bool muvec::operator!= (const muvec& a) const
{
 return !(*this == a);
}

inline bool muvec::operator>(const muvec& a) const
{
 if (mv.size() > a.mv.size())
  return true;
 else if (mv.size() < a.mv.size())
  return false;
 bool result = false;
 for (int i=0;i<mv.size();i++){
  if (mv[i] > a.mv[i]){
   result = true;
   break;
  }
  else if (mv[i] != a.mv[i])
   break;
 }
 return result;
}

inline bool muvec::operator<(const muvec& a) const
{
 if (mv.size() < a.mv.size())
  return true;
 else if (mv.size() > a.mv.size())
  return false;
 bool result = false;
 for (int i=0;i<mv.size();i++){
  if (mv[i] < a.mv[i]){
   result = true;
   break;
  }
  else if (mv[i] != a.mv[i])
   break;
 }
 return result;
}

inline ostream& operator<<(ostream& ost, muvec aid)
{return aid.print(ost);}

#endif

// muvec.h ends here
