#!/usr/bin/perl

use Getopt::ExPar;
use mead;
#use Benchmark;

#$t0 = new Benchmark;
my($OPT) = {};
my $epsin1;
my $epsin2;

&parse_command_line();
&check_parameter();

$epsvac = 1;


print "Starting solvate for molecule named $solute\n";
print "using the following physical conditions\n\n";
mead::PhysCond_print();
print "\n";
print "Interior dielectric constant, epsin = $epsin\n";
print "Vacuum dielectric constant, epsvac = $epsvac\n";
if(defined($OPT{blab1})) {print "Blab level set to 1\n";}
elsif (defined($OPT{blab2})) {print "Blab level set to 2\n";}
elsif (defined($OPT{blab3})) {print "Blab level set to 3\n";}
else {print "No blab level set.\n";}
print "\n";

#*****************************************************************************
#                            solvate starts here
#*****************************************************************************

$a = new mead::AtomSet($solute);
$a->read();
$acs = new mead::AtomChargeSet($a);
$rho = new mead::ChargeDist($acs);

$delp = new mead::TwoValueDielectricByAtoms($a, $epsin);
$eps = new mead::DielectricEnvironment($delp);

$eba = new mead::ElectrolyteByAtoms($a);
$ely = new mead::ElectrolyteEnvironment($eba);

$fdm = new mead::FinDiffMethod();
$fdm->read($solute.".ogm");

$interesting = new mead::Coord(0, 0, 0);
$fdm->resolve($a->geom_cent(), $interesting);
print "Using finite difference method with lattice levels:\n"; mead::write_fdm($fdm);

$phi = new mead::ElstatPot($fdm,  $eps, $rho, $ely);
$phi->solve();

$prod_sol = mead::mul_cd_ep($rho, $phi);
print "prod_sol = $prod_sol\n";


mead::PhysCond_epsext_set($epsvac);
mead::PhysCond_ionicstr_set(0.0);

$vac_ely = new mead::ElectrolyteEnvironment;
$delp2 = new mead::TwoValueDielectricByAtoms($a, $epsin);
$vac_eps = new mead::DielectricEnvironment($delp2);

$vac_phi = new mead::ElstatPot($fdm, $vac_eps, $rho, $vac_ely);
$vac_phi->solve();

$prod_vac = mead::mul_ep_cd($vac_phi, $rho);
print "prod_vac = $prod_vac\n";
$solvation_energy = ($prod_sol - $prod_vac) / 2 * mead::PhysCond_econv_get();
print "\n\nSOLVATION ENERGY = $solvation_energy\n";
print "(probably in kcal/mole)\n";

#$t1 = new Benchmark;
#$td = timediff($t1, $t0);
#print "the code took:",timestr($td),"\n";
#*****************************************************************************
#                            solvate ends here
#*****************************************************************************


sub parse_command_line {

my($OPT) = @_;
my(@PDT,@MM);

$Getopt::ExPar::evap_Help_Hooks{'P_HHBRFL'} = "\n\"Solute.pqr\" and \"solute.ogm\" are needed.\n\n";
$Getopt::ExPar::evap_Help_Hooks{'P_HHERFL'} = "\nFilename for solute missing!\n\n";

@PDT = split /\n/, <<'end-of-PDT';
PDT sample
  epsin, e: real
  epssol, es: real
  epsext, ee: real
  epsvac, ev: real = 1.0
  solrad, sr: real
  sterln, sl: real
  ionicstr, is: real
  T, t: real
  kBolt, kb: real
  conconv, cc: real
  econv, ec: real
  bohr_radius, br: real
  proton_charge, pc: real
  ReactionField, rf: switch
  ProteinField, pf: switch
  converge_oldway, co: switch
  blab1, b1: switch
  blab2, b2: switch
  blab3, b3: switch
PDTEND required_file_list
end-of-PDT

@MM = split /\n/, <<'end-of-MM';
	    ***************Solvate******************
	
	Usage:
	
	  ./solvate.pl -epsin n -epssol m solute protein

.epsin
	Dielectric constant of solute interior.
        In old versions of MEAD this had a default value,
        but this caused confusion, so now you must
        specify epsin explicitly.
.epssol
	Dielectric constant of the regions interior to the 
	protein but exterior to the solute.
        In old versions of MEAD this had a realdefault value,
        but this caused confusion, so now you must
        specify epsin explicitly.
.epsext
	Exterior dielectric constant.
.epsvac
	The dielectric constant of the other
        external media in the solvate program,
        presumably the vacuum.
.solrad
	Solvent probe radius used in rolling ball
	procedure to determine contact surface
	which is taken as the boundary between
	epsin and epsext.  (Angstroms, by default)
.sterln
	Ion exclusion layer thickness which is added
	to atomic radii to determine region inaccessible
	to salt so that kappa term in PB equation
	is zero.  (Angstroms, by default)
.ionicstr
	Ionic strength (moles/liter, by default).
.T
	Temperature (Kelvins by default)
.kBolt
	The Boltzmann constant in units of charge unit
	squared per length unit per temperature unit.
	You must adjust this if you do not use the
	default units of length, charge and temperature
	(protons squared)/(Angstrom * Kelvin, by default)
.conconv
	Conversion factor for going from concentration
	units used for ionic strength to units of
	particles per cubic length unit.  You must adjust
	this if you do not use the default units of length
	and concentration.
	((particles/cubic Angstrom)/(moles/liter), by default)
.econv
	Conversion factor for going from energy units
        of (charge units squared)/(length unit) to
        to whatever energy units you want for your output.
        You must adjust this if you do not use the
	default units of length and charge, or if you
	want output in different energy units.
	((kcal/mole)/(protons squared/Angstrom), by default)
.converge_oldway
	Revert to the old way of testing for convergence
        of the SOR method of solving the finite-difference
        representation of the Poisson--Boltzmann equation.
        The new method gives improved long-range accuracy
        for large lattices, but at a sometimes substantial
        computational cost.  See the NEWS file for further
        discussion.
.blab1
	Verbose level of the program. Now the program gives
	rough information about what´s going on.
.blab2
	Verbose level of the program. The program tells you
	quite a lot.
.blab3
	Verbose level of the program. You should know
	everything you wanted to know now.
end-of-MM

ExPar \@PDT, \@MM, \%OPT;

}

sub check_parameter() {

if(defined($OPT{epsin})) {
  #print "epsin ok\n"
} else {
  die("epsin not ok\n");
}
$epsin = $OPT{epsin};

if(defined($OPT{epssol})) { $epssol = $OPT{epssol};}


($solute) = $ARGV[$#ARGV];
if(-e $solute.".pqr") {
  print $solute.".pqr found\n";
} else {
  die("Solute pqr-file missing\n");
}
if(-e $solute.".ogm") {
  print $solute.".ogm found\n";
} else {
  die("Solute ogm-file missing\n");
}

if(defined($OPT{epsext})) {mead::PhysCond_epsext_set($OPT{epsext})};
if(defined($OPT{epsvac})) {$epsvac = $OPT{epsvac};}
if(defined($OPT{solrad})) {mead::PhysCond_solrad_set($OPT{solrad})};
if(defined($OPT{sterln})) {mead::PhysCond_sterln_set($OPT{sterln})};
if(defined($OPT{ionicstr})) {mead::PhysCond_ionicstr_set($OPT{ionicstr})};
if(defined($OPT{T})) {mead::PhysCond_T_set($OPT{T})};
if(defined($OPT{kBolt})) {mead::PhysCond_kBolt_set($OPT{kBolt})};
if(defined($OPT{conconv})) {mead::PhysCond_conconv_set($OPT{conconv})};
if(defined($OPT{econv})) {mead::PhysCond_econv_set($OPT{econv})};
if(defined($OPT{bohr_radius})) {mead::PhysCond_bohr_radius_set($OPT{bohr_radius})};
if(defined($OPT{proton_charge})) {mead::PhysCond_proton_charge_set($OPT{proton_charge})};

if(defined($OPT{ReactionField})) {$doReactionField = 1;};
if(defined($OPT{ProteinField})) {$doProteinField = 1;};
if(defined($OPT{converge_oldway})) {$doconverge_oldway = 1;};

if(defined($OPT{blab1})) {mead::blab1_set()}
if(defined($OPT{blab2})) {mead::blab2_set()}
if(defined($OPT{blab3})) {mead::blab3_set()}

print "Parameters OK\n\n";

}
